;+
; NAME:
;      LATLONINATOR
;
; PURPOSE:
;
;       The purpose of this program...
;
; AUTHOR:
;		Frank Seelos
;		Washington University in St. Louis (1999-2005)
;			seelos at rsmail dot wustl dot edu
;		Johns Hopkins Univeristy Applied Physics Laboratory (2005-)
;			frank dot seelos at jhuapl dot edu
;
; CATEGORY:
;
;		ENVI Utility
;
; CALLING SEQUENCE:
;
;
; OPTIONAL INPUTS:
;
;
; KEYWORDS:
;
;
; OUTPUTS:
;
; COMMON BLOCKS:
;		NONE
;
; DEPENDENCIES:
;
; PROCEDURE:
;
; EXAMPLE:
;
;
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;
; 06/01/2003 (fps)
;	Initial development of basic functionality
;	Date is approximate
;
; 03/24/2006 (fps)
;	Major code overhaul...
;	Added compiler option and error handling
;	Switched tiling to BIL - consistent with output interleave
;	Combined MAP and GEO output bands into one file or memory item
;	New ENVI automanaged widget interface that allows the user to select a
;		subset of MAP/GEO output bands
;	Output to memory option with ENVI memory check now supported
;	Added verificaton that input image is referenced
;	Much improved in-code commentary
;
; 03/30/2006 (fps)
;	New default output filename
;	Fixed bug where the header file would be written with the default base
;		filename even if the user changed the output image filename
; 07/15/2011 (fps)
;   	Added input band datum specification to geographic map projection initalization
;   	    Appears to fix offset issue with HiRISE data...
; 11/23/2011 (fps)
;   	Removed input band datum specification in call to establish geographic projection
;   	    This was apparrently an unnecessary hack due to a bad ENVI program configuration (envi_ex1.sav)
;
;
;;
;-
;###########################################################################
;
; LICENSE
;
;###########################################################################

pro latloninator, event

;Set compiler options
compile_opt strictarr

;Simple catch to prevent crash
catch, error
if (error NE 0) then begin
	;Convert !error_state structure to a string array
	help, !error_state, /struct, output = error_state_report
	help, !error_state, /struct

	error_dialog = dialog_message(error_state_report, /error, title = 'LATLONINATOR: Unexpected Error')

	if (strupcase(error_dialog) EQ 'OK') then return
endif

;Select input band from available bands list
envi_select, fid=in_fid, pos=in_pos, dims=in_dims, file_type=in_file_type, /band_only, title='LATLONINATOR: Select Input Band';, $
;			 /mask, m_fid = mask_fid

if (in_fid[0] eq -1) then begin
	print, 'LATLONINATOR: Cancel/Invalid FID'
	return
endif

;Gather some useful info
envi_file_query, in_fid, fname = in_filename, ns=ns, nl=nl, nb=nb


;Verify that the selected file is referenced
in_map_info = envi_get_map_info(fid = in_fid)
in_projection_info = envi_get_projection(fid = in_fid)

if (in_projection_info.type EQ 0) then begin
;	print, 'LATLONINATOR: No map information associated with selected file'
       ok = dialog_message('No map information associated with selected file.',$
                            title='LATLONINATOR')
       return
endif


;Construct default output filenames
last_slash = strpos(in_filename, '/', /reverse_search) > strpos(in_filename, '\', /reverse_search)
last_dot = strpos(in_filename, '.', /reverse_search)

in_file_directory = strmid(in_filename, 0, last_slash + 1)
in_file_basename = strmid(in_filename, last_slash + 1, last_dot - last_slash -1)
if (last_dot NE -1) then begin
	in_file_extension = strmid(in_filename, last_dot)
endif else begin
	in_file_extension = ''
endelse

geomap_coords_file = in_file_directory + in_file_basename + '_coord' + in_file_extension


;Gather user input info via ENVI auto managed widgets
auto_base = widget_auto_base(title = 'LATLONINATOR Parameters')
main_base = widget_base(auto_base, /row)
	geomap_base =	widget_base(main_base, /col)
		geomap_label = widget_label(geomap_base, value = 'Geo/Map Options')
		geomap_list = ['Longitude', 'Latitude', 'Meters East', 'Meters North']
		geomap_select = widget_multi(geomap_base, /auto_manage, list = geomap_list, uvalue = 'geomap_select', /no_range, $
								     default = replicate(1b, n_elements(geomap_list)), ysize = 100)
		geomap_outfm = widget_outfm(geomap_base, /auto_manage, /frame, default = geomap_coords_file, uvalue = 'geomap_outfm')

result = auto_wid_mng(auto_base)

if (result.accept EQ 0) then begin
	print, 'LATLONINATOR: Cancel
	return
endif

;Set GEO/MAP output parameters
geomap_dims = in_dims
geomap_dt = 4 ;floating point
geomap_band_indx = where(result.geomap_select EQ 1, geomap_nb)


;GEO/MAP memory check
geomap_mem_check = magic_mem_check(dims=in_dims, in_memory = result.geomap_outfm.in_memory, $
								   out_dt=geomap_dt, nb=geomap_nb, out_name = result.geomap_outfm.name)

if (geomap_mem_check.cancel EQ 1) then begin
	print, 'LATLONINATOR: Cancel'
	return
endif

;Store result of memory check in result structure
result.geomap_outfm.in_memory = geomap_mem_check.in_memory
result.geomap_outfm.name = geomap_mem_check.out_name

in_samples = in_dims[2] - in_dims[1] + 1
in_lines = in_dims[4] - in_dims[3] + 1

;Decalare output slice array
geomap_output_slice = make_array(in_samples, geomap_nb, value = 0.0, /float)

;Declare in-memory output array or open output file
if (result.geomap_outfm.in_memory EQ 1) then begin
	geomap_cube = make_array(in_samples, in_lines, geomap_nb, value = 0.0, /float)
	out_status_string = 'Output to Memory'
endif else begin
	openw, geomap_lun, result.geomap_outfm.name, /get_lun
	out_status_string = 'Output File: ' + result.geomap_outfm.name
endelse

;Generate geographic projection structure for use later
geoproj = envi_proj_create(/geographic)
;geoproj = envi_proj_create(/geographic, datum = in_projection_info.datum)

;Initialize ENVI tiling - use BIL to match desired output interleave
in_tile_id = envi_init_tile(in_fid, in_pos, num_tiles = num_in_tiles, interleave = 1, $
							xs = in_dims[1], xe = in_dims[2], ys = in_dims[3], ye = in_dims[4])

;Assemble string to be displayed in ENVI status window
status_string_array = ['Input File: ' + in_filename, $
					   out_status_string]

envi_report_init, status_string_array, base = status_base, title = 'LATLONINATOR', /interupt

envi_report_inc, status_base, num_in_tiles

;Main processing loop - one iteration for each tile
for i = long(0), num_in_tiles -1 do begin

	envi_report_stat, status_base, i, num_in_tiles, cancel=cancel

	if (cancel) then begin
		print, 'LATLONINATOR: Cancel'
		envi_report_init, base = status_base, /finish

		if (result.geomap_outfm.in_memory EQ 0) then begin
			free_lun, geomap_lun
		endif

		return
	endif

	;Grab tile data
	in_data = envi_get_tile(in_tile_id, i)

	x_pos_array = lindgen(in_samples)
	y_pos_array = replicate(i, in_samples)

	;Calculate MAP coordinates
	envi_convert_file_coordinates, in_fid, x_pos_array, y_pos_array, x_map, y_map, /to_map

	;Calculate GEO coordinates
	envi_convert_projection_coordinates, x_map, y_map, in_projection_info, x_geo, y_geo, geoproj

;stop

	;Populate output file spectral slice based on user selected bands
	for k = 0, geomap_nb -1 do begin
		indx = geomap_band_indx[k]

		case (indx) of
			0:begin
				geomap_output_slice[*,k] = x_geo
			end

			1:begin
				geomap_output_slice[*,k] = y_geo
			end

			2:begin
				geomap_output_slice[*,k] = x_map
			end

			3:begin
				geomap_output_slice[*,k] = y_map
			end

			else: begin
				return
			endelse

		endcase

	endfor

	;Write spectral slice to output file or store in output memory array
	if (result.geomap_outfm.in_memory EQ 1) then begin
		geomap_cube[*,i,*] = geomap_output_slice
	endif else begin
		writeu, geomap_lun, geomap_output_slice
	endelse

;stop

endfor

;Register memory item in ENVI session or generate header for output file
if (result.geomap_outfm.in_memory EQ 1) then begin
		envi_enter_data, geomap_cube, $
						 bnames = geomap_list[geomap_band_indx], $
						 map_info = in_map_info
endif else begin
		free_lun, geomap_lun

		envi_setup_head, fname = result.geomap_outfm.name, ns = in_samples, nl = in_lines, nb = geomap_nb, $
						 interleave = 1, data_type = 4, offset = 0, $
						 bnames = geomap_list[geomap_band_indx], $
						 map_info = in_map_info, /write, /open
endelse


envi_tile_done, in_tile_id

envi_report_init, base = status_base, /finish


END

