; Created to support file selection from ENVI for
; mro_crism_summary_flatten.pro
;
; Output files are written as BSQ.
;
; Created:  June 28, 2007 by
;	    Shannon Pelkey
;           Brown University
;
; Aug 22,2007   MFM:
;	--updated catch to call cat_error_report
;
; Aug 22, 2007  MFM:
;	--updated output header to pass through additional info from input 
;	  header, especially map_info.
;
; Aug 22,2007  MFM:
;	--bug fix- deleted statement "input_array=output_array" creating unused 
;	  array input_array and crashing when writing to file because 
;	  output_array undefined.
; Dec 5,2009  MFM
;   Use get_cat_header_data and write_cat_header for header IO
; Nov 17,2010  MFM
;	Add header parameter CAT_PRODUCT_VERSION
; Nov 18,2010 (mfm)
;	four more CAT header parameters

pro mro_crism_summary_flatten_envi, event

procedure_title = 'MRO CRISM SUMMARY FLATTEN'

;Set compiler options
compile_opt strictarr

;Simple catch to prevent crash
catch, error
if (error NE 0) then begin
  	cat_error_report, routine_name='mro_crism_summary_flatten_envi'
	return
endif

;Select input band from available bands list (allows for subsetting):
envi_select, fid=in_fid, pos=in_pos, dims=in_dims, title = procedure_title + ': Select Input File',$
             /no_dims

if (in_fid[0] eq -1) then begin
	print, procedure_title + ': Cancel/Invalid FID'
	return
endif


; Gather some useful info for processing, as well as other header info for 
; passthrough.
stat = get_cat_header_data(in_fid, NS=in_ns, NL=in_nl, NB=in_nb, $
	WAVELENGTH_UNITS=wu, BBL=bbl, BNAMES=in_bnames, FNAME=in_filename, $
	INTERLEAVE=interleave, OFFSET=hdr_offset, $
	DATA_IGNORE_VALUE=hdr_div, DATA_TYPE=hdr_datatype, $
	DEF_BANDS=def_bands, DESCRIP=hdr_desc, MAP_INFO=in_map_info, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files )

if (stat ne 0) then begin
	ok = dialog_message( 'Problem getting header data from '+file_basename(in_filename) $
	                     + string(10b) + 'Aborting procedure.', $
		                     title='Flatten Parameters Error')
	return
endif


;Set output parameters
out_dims = in_dims
out_pos = in_pos
out_ns = in_ns
out_nl = in_nl
out_bnames = in_bnames[in_pos]+'_flat'
out_nb = n_elements(out_bnames)


;Construct default output filenames
last_slash = strpos(in_filename, '/', /reverse_search) > strpos(in_filename, '\', /reverse_search)
last_dot = strpos(in_filename, '.', /reverse_search)
in_file_directory = strmid(in_filename, 0, last_slash + 1)
in_file_basename = strmid(in_filename, last_slash + 1, last_dot - last_slash -1)
if (last_dot NE -1) then begin
	in_file_extension = strmid(in_filename, last_dot)
endif else begin
	in_file_extension = ''
endelse
output_file_default = in_file_directory + in_file_basename + '_flat.img'


; Check to see if user wants to write to a file or keep in memory:
; Load into available bands list regardless.
base = widget_auto_base(title='Flatten Parameters')
wo = widget_outfm(base, uvalue='outf',default=output_file_default, /auto)
result = auto_wid_mng(base)

if (result.accept eq 0) then return
if ((result.outf.in_memory) ne 1) then outfile=result.outf.name else begin
  ;memory check
  mem_check = magic_mem_check(dims=in_dims, in_memory = result.outf.in_memory, $
	  	      	      out_dt=4, nb=out_nb, out_name = result.outf.name)
  if (mem_check.cancel EQ 1) then return    ; cacel selected
  ;Store result of memory check in result structure
  result.outf.in_memory = mem_check.in_memory
  result.outf.name = mem_check.out_name
  outfile=result.outf.name
endelse


;Declare in-memory output array or open output file
if (result.outf.in_memory EQ 1) then begin
	output_array = make_array(out_ns,out_nl,out_nb)
	out_status_string = 'Output to Memory'
endif else begin
	openw, out_lun, outfile, /get_lun
	out_status_string = 'Output File: ' + outfile
endelse


;Assemble string to be displayed in ENVI status window
status_string_array = ['Input File: ' + in_filename, out_status_string]
envi_report_init, status_string_array, base = status_base, title = procedure_title, /interupt
envi_report_inc, status_base, n_elements(in_pos)

;Initialize pseudo tiling
;Main processing loop - one iteration for each band
for k = 0, n_elements(in_pos) -1 do begin

	envi_report_stat, status_base, k, n_elements(in_pos), cancel=cancel
	if (cancel) then begin
		print, procedure_title + ': Cancel'
		envi_report_init, base = status_base, /finish
		if (result.outf.in_memory EQ 0) then begin
			free_lun, out_lun
		endif
		return
	endif

	;Grab spatial band and run function
	in_data = envi_get_data(fid = in_fid, pos = in_pos[k], dims=in_dims)
	out_data = mro_crism_summary_flatten(in_data)

	;Store in outputarray
	if (result.outf.in_memory EQ 1) then begin
		output_array[*,*,k] = out_data
	endif else begin
		writeu, out_lun, out_data
	endelse

endfor

envi_report_init, base = status_base, /finish



;============================
;    HEADER PREP
;============================


; Append history:
cat_history = cat_append_history(cat_history, 'FSP')

; Update input file list:
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(in_filename)]
endif else begin
	cat_input_files = [file_basename(in_filename)]
endelse

; Spectral subsetting...
if (n_elements(in_bnames) gt 1) then begin
	bnames_out = in_bnames[out_pos]
endif else begin
	bnames_out = -1
endelse
if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	def_bands = crism_def_bands(cat_crism_detector_id, wl_out)
endif


;Register memory item in ENVI session or write file and generate header for output file
if (result.outf.in_memory EQ 1) then begin
	envi_enter_data, output_array, bnames=out_bnames, $
	       data_ignore_value=65535., data_type=4, $
	       descrip='CAT CRISM DATA flattened parameters', $
	       map_info=in_map_info
endif else begin
	stat = write_cat_header(FNAME=outfile, $
	    NS=out_ns, NL=out_nl, NB=out_nb, $
	    DATA_TYPE=4, INTERLEAVE=0, OFFSET=0, $
	    BNAMES=out_bnames, DATA_IGNORE_VALUE=65535.0, $
	    DESCRIP='CAT CRISM DATA flattened parameters', $
	    MAP_INFO=in_map_info, $
	    CAT_START_TIME          = cat_start_time, $
	    CAT_SCLK_START          = cat_sclk_start, $
	    CAT_CRISM_OBSID         = cat_crism_obsid, $
	    CAT_OBS_TYPE            = cat_obs_type, $
	    CAT_PRODUCT_VERSION     = cat_product_version, $
	    CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $
	    CAT_BIN_MODE            = cat_bin_mode, $
	    CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	    CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	    CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	    CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	    CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	    CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	    CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	    CAT_HISTORY             = cat_history, $
	    CAT_INPUT_FILES         = cat_input_files )

	; Report problems with header if any indicated by status returned:
	if ((stat ne 0) and (~keyword_set(batch))) then begin
		if (stat eq -1) then begin
			msg = 'Problem writing header for '+file_basename(outfile) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
			ok = dialog_message(msg, title='Header Error')
		endif else if (stat eq -2) then begin
			msg = 'Problem writing header for '+file_basename(outfile) + string(10b) $
			      + 'Header already exists.'
			ok = dialog_message(msg, title='Header Error')
		endif else begin
			msg = 'Problem writing header for '+file_basename(outfile) + string(10b) $
			      + 'Unspecified error.'
			ok = dialog_message(msg, title='Header Error')
		endelse
	endif

endelse

return
end

