;******************************************************************************
;
; function apply_bbl_event
;
; CAT/ENVI event handler to apply bad bands list to an open CRISM file. 
;
; INPUT:
;
; OUTPUT:
;	Parameters:    None
;	Return value:  Vector with 1-to-1 correspondence to the wavelength vector 
;	               specified in the ENVI header of the selected file, and 
;                  with 1's (good bands) and 0's (bad bands).
;
; KEYWORDS: none
;
; EXAMPLE:
;	waves = [550., 700., 750., 800., 900., 950., 1000.]
;
;	If 700-800 are all bad bands and 1000. is also bad, the result is:
;	returns [1,0,0,0,1,1,0]
;
;	If no bands are bad, returns [1,1,...,1]
;
; NOTES:
;		
; USES:
;	No external procedures.
;
; HISTORY:
;	2007 Nov 20   F. Morgan: First version written ..
;	2009 Jul 09   F. Morgan. Check (hdr_fnd eq 0) not ~hrd_fnd
;	2009 Nov 02   F. Morgan: Eliminate wu arg on create_crism_bbl call.
;
;******************************************************************************

pro apply_bbl_event, ev


widget_control, ev.id, get_uvalue=uvalue

; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
	cat_error_report, routine_name='apply_bbl_event'
	return
endif

; BBL limits message
msg = 'CRISM BAD BANDS: Limits of procedure          ' + string([10b])  $
	+ 'Only works on file with existing ENVI header; ' + string([10b])  $
	+ 'will not work on data in memory. Must close   ' + string([10b])  $
	+ 'and reopen before BBL takes effect.           ' + string([10b])  $
	+ 'Do not apply until CAT processing complete.'
ok = dialog_message(msg, title='CRISM Bad Bands Limits')


; Select an open file:
envi_select, fid=in_fid, pos=in_bands, dims=sf_dims, $
	/file_only, /no_dims, /no_spec, title='Select an open file:'
if (in_fid[0] eq -1) then return                  ;cancel selected

; Get existing header data:
envi_file_query, in_fid, fname=fnm, ns=ns,nl=nl,nb=nb, $
		interleave=interleave, data_type=dt, $
		byte_swap=bo, wavelength_units=wu, wl=waves, fwhm=bwlbl,$
		data_ignore_value=div, def_bands=def_bands, $
		descrip=descrip
; also get map_info:
map_info = envi_get_map_info(fid=in_fid)

fwaves = float(waves)

; Create the bad bands list:
bbl = create_crism_bbl(fnm, fwaves)
if (bbl[0] eq -1) then begin
	; warning dialog, return after 'OK'
	msg = 'Unable to apply bad bands list for ' + strtrim(fnm,2) 
	ok = dialog_message(msg, title='Apply BBL warning')
	return
endif


;---------------------

; stuck here - need a way to elegantly insert one new parameter into a 
; header and cant find it.
;---------------------

; Instead, just write a line into the header file

; Find the header file name:
p = strpos(fnm,'.',/reverse_search)
hdrtry = strmid(fnm,0,p) + '.HDR'
hdr_fnd = cat_file_check(hdrtry, hdr_list)
if (hdr_fnd eq 0) then begin
	hdrtry = strtrim(fnm,2) + '.HDR'
	hdr_fnd = cat_file_check(hdrtry, hdr_list)
endif
if (hdr_fnd eq 0) then begin
	msg = 'Unable to apply bad bands list for ' + strtrim(fnm,2) 
	ok = dialog_message(msg, title='Apply BBL warning')
	return
endif
hdr = hdr_list[0]

hdrlen = file_lines(hdr)
openu, uhd, hdr, /get_lun
; Make sure BBL not already set:
s = ''
for i=0,hdrlen-1 do begin 
	readf,uhd,s
	p = strpos(s,'=')
	if (p ge 0) then begin
		if (strupcase(strtrim(strmid(s,0,p),2)) eq 'BBL') then begin
			msg = 'Bad bands already set in ' + strtrim(fnm,2) 
			ok = dialog_message(msg, title='Apply BBL warning')
			return
		endif
	endif
endfor

; Now at end, didnt find BBL already set, so write the BBL:
printf, uhd, 'BBL = {'
printf, uhd, bbl, format='((4X,10(I0,:,",")),$)'
printf, uhd, ' }'
free_lun, uhd
		
;envi_setup_head, fname=fnm, ns=ns,nl=nl,nb=nb, bbl=bbl, $
;		interleave=interleave, data_type=dt,$
;		byte_order=bo, offset=0, wavelength_unit=wu, wl=waves, fwhm=bwlbl,$
;		data_ignore_value=div, def_bands=def_bands, map_info=map_info,$
;		descrip=descrip

return
end
