; ENVI event handler routine to inspect a CRISM EPF by
;  projecting the segments for a user selected band.
;  To be used with TRDRs only.
;
; Note: hardcodeGNC = hardcoded for general naming conventions
;	
; Created: Jan 25, 2006
; By: Shannon Pelkey
; Modification History:
; AUG 08,2006 SMP: Made compatible with Windows platforms
; Sep 12,2006 SMP: Added check for flipp and/or mask; aborts if
;                  either detected.
; Sep 29,2006 SMP: Made geometry file location more automated and
;                  cleaned up code.
; Feb 16,2007 SMP: --Modified to create a .sav file
;                  --Changed use of 'sort' to 'bsort' to avoid issue
;                    on Windows.
; Jun 25,2007 SMP: --added name check to check for/abort if user
;                    selected a tile
; Feb 10,2009 MFM: --For checking file types from filename contents, 
;                    use only file basename, don't include path.
; Mar 15,2009 MFM: --Bugfix det='L' not 'S' after finding L_ in epfupbase
; Mar 15,2009 MFM: --Use get_lun instead of unit 1
; Mar 15,2009 MFM: --Bugfix use [0] subscript on wolap
; Sep 16,2009 MFM: --Replace find_cdr6file with find_cdr
; Sep 17,2009 MFM: --find_geofile returns '' not '-1' on fail
; Dec 08,2009 MFM: --Use cat_tmpfile_name to generate unique temp file name
; Dec 09,2009 MFM: --Force delete temp GLT header
; Jan 28,2010 MFM: --Case insensitivity corrections. 
; Jan 28,2010 MFM: --Try to find DDR's all at once instead of sequentially.
; Feb 01,2010 MFM: --Cut files that aren't raw PDS out of epflist. Messed up DDR search.
; Feb 23,2010 MFM: --Took datum keyword out of envi_proj_create. Made no 
;                    difference, required problematic custom datum.
; Mar 16,2012 MFM: --Abort if input is MTRDR - not set up for that yet.


pro epf_event, ev
  widget_control, ev.id, get_uvalue=uvalue

; Define ENVI functions used so able to create a sav file:
  FORWARD_FUNCTION envi_pickfile,envi_get_data,envi_get_file_ids, $
            envi_get_roi_data,envi_get_projection,envi_proj_create, $
            widget_slist,widget_auto_base,widget_menu,widget_outfm, $
            auto_wid_mng, pdsfile_is_mtrdr


  ; Simple catch to prevent crash:
  catch, error
  if (error NE 0) then begin
    cat_error_report, routine_name='epf_event'
    return
  endif


  if (uvalue eq 'plot_overlap') or (uvalue eq 'plot_subset') then plot=1 else plot=0


  ; See what files are open:
  origfids = envi_get_file_ids() 

  ; Check to see if user wants to write to a file or keep in memory:
  ; Load into available bands list regardless.
  base = widget_auto_base(title='EPF: View Overlap Projection.')  
  wo = widget_outfm(base, uvalue='outf', /auto)  
  result = auto_wid_mng(base)  
  if (result.accept eq 0) then return  
  if ((result.outf.in_memory) ne 1) then begin
    outfile=result.outf.name 
    inmem=0  
    if (plot eq 1) then begin
      pos=strpos(outfile,'.',/reverse_search)
      if (pos ne -1) then begin
        len=strlen(outfile)
        i_outfile=strtrim(strmid(outfile,0,pos),2)+'_i'+strtrim(strmid(outfile,pos,len-pos),2)
        e_outfile=strtrim(strmid(outfile,0,pos),2)+'_e'+strtrim(strmid(outfile,pos,len-pos),2)
        g_outfile=strtrim(strmid(outfile,0,pos),2)+'_g'+strtrim(strmid(outfile,pos,len-pos),2)
      endif else begin
        i_outfile='i_out.img'
        e_outfile='e_out.img'
        g_outfile='g_out.img'
      endelse
    endif
  endif else begin
    inmem = 1
  endelse

  ; Get files of interest...user is asked if they would like to use
  ;  all observations of the epf or only some:
  base = widget_auto_base(title='EPF Event')
  prompt ='Project band overlap for:'
  list = ['Complete EPF series.','Part of EPF series.']
  wm = widget_menu(base,prompt=prompt,list=list,rows=2,uvalue='menu',/exclusive,/auto)
  result = auto_wid_mng(base)
  if (result.accept eq 0) then return else menu=result.menu
  ; Check entry:
  if (menu eq 0) then seriesq='all'
  if (menu eq 1) then seriesq='some'

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; GET FILENAMES TO WORK WITH:
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; If doing all files in EPF, have user select one as an example;
  ;  if doing only some, user has to select them.

  PICK_EPFS:
  if (seriesq eq 'all') then begin
    epflist = envi_pickfile(title='Select a single EPF from series:', $
                   filter='*trr*.img;*TRR*.IMG')
  endif else begin
    epflist = envi_pickfile(title='Select EPF file(s) to include:', $
                   filter='*trr*.img;*TRR*.IMG',/multiple_files)
  endelse
  if (epflist[0] eq '') then return                ;cancel selected

  ; Check to make sure not a tile:
  if (pdsfile_is_tile(epflist[0])) then begin
    ok = dialog_message('EPF Utilities are not compatible with tiles.',$
                         title='EPF Utilities',/cancel)
    return
  endif

  ; Not set up to handle MTRDR epf yet
  if (pdsfile_is_mtrdr(epflist[0])) then begin
    ok = dialog_message('EPF Utilities are not set up for MTRDR yet.',$
                         title='EPF Utilities',/cancel)
    return
  endif

  ; Find the detector:
  epfupbase = strupcase(file_basename(epflist[0]))
  pos = strpos(epfupbase,'S_')
  if (pos ne -1) then DET='S' else begin
    pos=strpos(epfupbase,'L_')
    if (pos ne -1) then DET='L'else begin
      pos=strpos(epfupbase,'J_')
      if (pos ne -1) then DET='J' else begin
        base = widget_auto_base(title='Convert CRISM')  
        list = ['S', 'L','J']
        wm = widget_menu(base, list=list, uvalue='menu',rows=3,$
             prompt='Label info not found. What detector is this data from?',$
             /excl, /auto)  
        result = auto_wid_mng(base) 
        if (result.accept eq 0) then return   ;cancel selected
        det=list(result.menu)
      endelse
    endelse
  endelse

  ; Find the path+obsID (modififed 2/10/09 FM):
  epfdir = file_dirname(epflist[0], /mark_directory)
  epffnm = file_basename(epflist[0])
  pos = strpos(strupcase(epffnm),DET+'_')
  epfpath = epfdir + strtrim(strmid(epffnm,0,pos-8))

  ; Find the data format (radiance or IoF):
  pos = strpos(epfupbase,'_IF')
  if (pos ne -1) then begin
    datafmt='IF'
  endif else begin
    pos = strpos(epfupbase,'_RA')
    if (pos ne -1) then datafmt='RA' else datafmt=''
  endelse


  if (seriesq eq 'all') then begin
    ; Create filters related to file selection and find files:
    epffilter = '*'+'_'+datafmt+'*'+DET+'_TRR*.IMG'
    epflist = file_search(epfpath+epffilter, /fold_case)

    ; Drop any files that aren't raw PDS
    nepf = n_elements(epflist)
    pds_epf = replicate(0,nepf)
    for i=0,nepf-1 do if (file_is_pds(epflist[i])) then pds_epf[i]=1
    kgood = where((pds_epf eq 1), ngood)
    if (ngood eq 0) then begin
      msg = 'EPF projection requires raw PDS data. ' +string(10b) $
          + 'None of these EPFs are raw PDS files. Giving up.'
      ok = dialog_message(msg, title='EPF Utilities')
      return
    endif
    epflist = epflist[kgood]
  endif else begin
    ; User selected, make sure files are all from the same 
    ;  DETECTOR, SERIES, and type: 
    posarr = strpos(strupcase(file_basename(epflist)),DET+'_')
    w = where(posarr eq -1)
    if (w[0] ne -1) then begin
      ok = dialog_message('Files are not all from same DETECTOR.',$
                           title='Inspect EPF Error')
      goto,PICK_EPFS
    endif 
    posarr = strpos(strupcase(epflist),strupcase(epfpath))
    w = where(posarr eq -1)
    if (w[0] ne -1) then begin
      ok = dialog_message('Files are not all from same EPF SERIES.',$
                           title='Inspect EPF Error')
      goto,PICK_EPFS
    endif
    posarr = strpos(strupcase(epflist),datafmt)
    w = where(posarr eq -1)
    if (w[0] ne -1) then begin
      ok = dialog_message('Files are not all the same data format (IF vs. RA).',$
                           title='Inspect EPF Error')
      goto,PICK_EPFS
    endif

    ; And drop anything other than raw PDS files:
    nepf = n_elements(epflist)
    pds_epf = replicate(0,nepf)
    for i=0,nepf-1 do if (file_is_pds(epflist[i])) then pds_epf[i]=1
    kgood = where((pds_epf eq 1), ngood)
    if (ngood eq 0) then begin
      msg = 'EPF projection requires raw PDS data. ' +string(10b) $
            + 'None of the selected EPFs are raw PDS files.' +string(10b) $
            + 'Giving up.'
      ok = dialog_message(msg, title='EPF Utilities')
      return
    endif else if (ngood lt nepf) then begin
      msg = 'EPF projection requires raw PDS data. ' + string(10b) $
           + string(nepf-ngood,format='(I0.0)') + ' selected EPFs not raw PDS.' +string(10b) $
           + 'Remove these from list and continue?'
      ok = dialog_message(msg, title='EPF Utilities',/question,/cancel)
      if (ok eq 'Cancel') then return
      if (ok eq 'No') then goto, PICK_EPFS
      epflist = epflist[kgood]
    endif

  endelse


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; FIND RELATED GEOMETRY FILES:
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  ; First attempt to find en-masse in the same directory. This reduces 
  ; burden on user in the event the DDR's are all together in a tree where 
  ; CAT won't auto-locate them. Iterative DDR search would then prompt 
  ; for user selection for each. Maybe can locate them all following 
  ; pattern of one.
  got_all_ddrs = 0      ; flag for fail, reset if they're all found
  find_geofile, epflist[0], ddr0
  if (ddr0 ne '') then begin
    ddrpath = file_dirname(ddr0,/mark_directory)
    ddrbase = file_basename(ddr0)
    ; If ddrbase fits the pattern of a standard DDR filename, then 
    ; characters 11-21 are '_XX_DENNNY_' where XX is a hex segment number; 
    ; NNN = decimal digits that may differ between segments; and Y is 
    ; detector (S or L). May be upper or lower case. Need to wildcard the 
    ; segment number and the decimal digits to list al the DDR's
    segsec = strupcase(strmid(ddrbase,11,11))
    if (strlen(segsec) eq 11) then begin
      ; Is this a valid subsection of a standard ddr filename?
      ; This sets "standard_ddr" flag zero if chars 1-2 aren't hex segment number:
      standard_ddr = min(cat_is_digit(strmid(segsec,1,2),/allow_hex))
      ; ...or if expected underscores aren't there:
      if (strmid(segsec,0,1) ne '_') then standard_ddr=0
      if (strmid(segsec,3,1) ne '_') then standard_ddr=0
      ; ... or if characters 4-5 aren't de or DE:
      if (strupcase(strmid(segsec,4,2)) ne 'DE') then standard_ddr=0
      ; And detector has to be valid, and followed by underscore
      ddrdet = strupcase(strmid(segsec,9,2))
      if ((ddrdet ne 'S_') and (ddrdet ne 'L_')) then standard_ddr=0
      if (standard_ddr) then begin
        ; If all that's OK, can substitute wildcards into the 
        ; filename segment to build a filter for the DDR file set.
        ddrfilter = ddrpath + strmid(ddrbase,0,11) + '_*_DE*'+ddrdet $
               + strmid(ddrbase,22)
        ddrfiles = file_search(ddrfilter, /fold_case)
        nepf = n_elements(epflist)
        if ((ddrfiles[0] ne '') and (n_elements(ddrfiles) ge nepf)) then begin
          ; Now pick from these the DDR's that match the EPF segments.
          ; If can't get a match for any EPF, flag fail and search 
          ; for each sequentialy.
          nmatch = match_crism_segments(epflist, ddrfiles, ddr_match)
          if (nmatch eq nepf) then begin
            got_all_ddrs = 1
            ddrlist = ddr_match
          endif
        endif
      endif
    endif
  endif

  if (~got_all_ddrs) then begin
    ; Didn't get full DDR list; look for them sequentially.
    nfiles = n_elements(epflist)
    ddrlist = strarr(nfiles)
    for i=0,nfiles-1 do begin
      find_geofile,epflist[i],ddrx
      if (ddrx eq '') then return
      ddrlist[i] = ddrx
    endfor
  endif

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; CHECK DATA AND GEO COMPATIBILITY:
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Now we have the files of interest and their related geometry in the
  ; following variables: epflist, ddrlist
  ; Check compatibility:
  nfiles = n_elements(epflist)
  nddrs = n_elements(ddrlist)

  if (nfiles ne nddrs) then begin
    ok = dialog_message('Number of IMG and DDR files not equal.',$
                        title='Inspect EPF Error')
    return
  endif
  if (nfiles eq 1) then begin
    ok = dialog_message('There is only one file. Continue?',$
                        title='Inspect and EPF Error',/question,/cancel)
    if (ok eq 'Cancel') or (ok eq 'No') then return
  endif

  ; Find series names to be used below:
  pos = strpos(epfupbase,DET+'_')
  epfs = strtrim(strmid(file_basename(epflist),pos-20,14),2)             ;hardcodeGNC: AAAXXXXXXXX_XX
  epftag = strtrim(strmid(epfs[0],0,strlen(epfs[0])-2),2) ;hardcodeGNC: AAAXXXXXXXX_


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; GET WAVELENGTH TO INSPECT: Have user pick from CDR6-SW list.
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  ; Open CDR6-SW to get standard resampling wavelength array.
  get_sw_waves, epflist[0], sw_waves
  if (sw_waves[0] eq -1) then return
  waves = strtrim(crism_waves2microns(sw_waves),2)
  nwaves = n_elements(waves)
  bands = string(indgen(nwaves)+1)
  labels = strarr(nwaves)
  for i=0,nwaves-1 do labels[i]='Band: '+strtrim(bands[i],2)+'   ' + waves[i]
  base = widget_auto_base(title='Select Band')
  wm = widget_slist(base,list=labels,uvalue='bselect', $
                  prompt='Chose a SINGLE band to project:',/auto)
  result = auto_wid_mng(base)
  if (result.accept eq 0) then return else begin
    bindex = result.bselect
    label = labels(bindex)
    band = bands(bindex)
    wave = waves(bindex)
    bandstr = 'Band'+strtrim(band,2)+'('+strtrim(wave,2)+')'    ;used in names later on
  endelse

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; PROJECT BAND OF INTEREST FOR EACH FILE CHOSEN:
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Create a Geographic Lat/Lon projection structure to associate w/file 
  ;  (as is, file has an "Arbitrary" projection, which cannot be used
  ;  to create a GLT):
  proj = envi_proj_create(/geographic)

  ; Create arrays to use later:
  proj_fids = lonarr(n_elements(epflist))
  dims = lonarr(5,n_elements(epflist))
  pixs = dblarr(n_elements(epflist))
  out_bname = strarr(n_elements(epflist))
  if (plot eq 1) then begin
    i_proj_fids = lonarr(n_elements(epflist))
    e_proj_fids = lonarr(n_elements(epflist))
    g_proj_fids = lonarr(n_elements(epflist))
    i_out_bname = strarr(n_elements(epflist))
    e_out_bname = strarr(n_elements(epflist))
    g_out_bname = strarr(n_elements(epflist))
  endif

  for i=0,n_elements(epflist)-1 do begin
    ; Open data:
    envi_open_file,epflist[i],r_fid=epffid
    if (epffid[0] eq -1) then return 
    envi_file_query,epffid,dims=epfdims
    ; Open geometry:
    envi_open_file,ddrlist[i],r_fid=ddrfid
    if (ddrfid[0] eq -1) then return 
    envi_file_query,ddrfid,dims=geodims
    ; Make sure dimensions match:
    wne=where((epfdims(1:4) ne geodims(1:4)) eq 1)
    if (wne[0] ne -1) then begin
      ok = dialog_message('Data and Geo dimensions do not match.',$
                           title='Project Overlap Error')
      return
    endif

    ; Build GLT and GEOREF band with it: 
    tmp_glt_name = cat_tmpfile_name('glt_proj')
    ENVI_DOIT, 'ENVI_GLT_DOIT', DIMS=epfdims, I_PROJ=proj, O_PROJ=proj, $
         out_name=tmp_glt_name,r_fid=glt_fid,ROTATION=0, X_FID=ddrfid, $
         X_POS=4, Y_FID=ddrfid, Y_POS=3
    if (glt_fid[0] eq -1) then return

    ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=epffid,GLT_FID=glt_fid,pos=bindex,$
         out_name=epfs(i)+'_'+bandstr+'_proj',r_fid=proj_fid, background=65535.
    ; If making plots, need angles projected too:
    if (plot eq 1) then begin
      ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=ddrfid,GLT_FID=glt_fid,pos=0,$
           out_name=epfs(i)+'_i_proj',r_fid=i_proj_fid, background=65535.
      ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=ddrfid,GLT_FID=glt_fid,pos=1,$
           out_name=epfs(i)+'_e_proj',r_fid=e_proj_fid, background=65535.
      ENVI_DOIT, 'ENVI_GEOREF_FROM_GLT_DOIT',FID=ddrfid,GLT_FID=glt_fid,pos=2,$
           out_name=epfs(i)+'_g_proj',r_fid=g_proj_fid, background=65535.
    endif

    ; Remove files we won't use again from available bands list and 
    ; delete GLT file. Turns out envi_file_mng doesn't delete the header, 
    ; have to force delete it separately.
    envi_file_mng,id=glt_fid,/remove, /delete
    gltbn = strtrim(file_basename(tmp_glt_name),2)
    p = strpos(gltbn,'.',/reverse_search)
    if (p gt 0) then gltbn=strmid(gltbn,0,p)
    gltbn = file_dirname(tmp_glt_name, /mark_directory) + gltbn
    glt_hdr = file_search(gltbn+'*.hdr',/fold_case)
    k = where((glt_hdr ne ''),nk)
    if (nk gt 0) then begin
      file_delete,glt_hdr[k], /quiet,/noexpand_path
    endif
    ao = where(ddrfid eq origfids)
    if (ao[0] eq -1) then envi_file_mng,id=ddrfid,/remove
    ao = where(epffid eq origfids)
    if (ao[0] eq -1) then envi_file_mng,id=epffid,/remove

    ; Get dims and pixel size from newly projected file then close it: 
    envi_file_query,proj_fid,dims=proj_dims
    ;  (pixels are square so we only need to record one dimension)
    tmp=envi_get_projection(fid=proj_fid,pixel_size=ps)

    ; Build field arrays related to each projected band to use later:
    pixs(i) = ps[0]                               ;pixel size
    proj_fids(i) = proj_fid                       ;file ids 
    dims(*,i) = proj_dims(*)                      ;dimensions 
    out_bname(i) = epfs(i)+'_'+bandstr+'_proj'    ;name of projected bands
    if (plot eq 1) then begin
      i_proj_fids(i) = i_proj_fid           
      e_proj_fids(i) = e_proj_fid           
      g_proj_fids(i) = g_proj_fid          
      i_out_bname(i) = epfs(i)+'_i_proj'
      e_out_bname(i) = epfs(i)+'_e_proj'
      g_out_bname(i) = epfs(i)+'_g_proj'
    endif

  endfor

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; STACK PROJECTED LAYERS FROM EACH FILE:
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Sort pixels to get pixel size to use for layers:
  pixs_sorted = pixs(cat_bsort(pixs))           ;sort with smallest first

  ; Create band array to indicate which band from each file to stack:
  ;  (array only needs to be all zeros b/c all files only have one band)
  seg_bands = intarr(n_elements(epflist))   

  if (inmem eq 1) then begin

    ; If plotting, need to stack projected angles too:
    if (plot eq 1) then begin
      ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=i_proj_fids,pos=seg_bands,$
           dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
           out_dt=4,out_bname=i_out_bname,r_fid=i_olapfid,/IN_MEMORY, background=65535.
      if (i_olapfid eq -1) then return
      ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=e_proj_fids,pos=seg_bands,$
           dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
           out_dt=4,out_bname=e_out_bname,r_fid=e_olapfid,/IN_MEMORY, background=65535.
      if (e_olapfid eq -1) then return
      ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=g_proj_fids,pos=seg_bands,$
           dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
           out_dt=4,out_bname=g_out_bname,r_fid=g_olapfid,/IN_MEMORY, background=65535.
      if (g_olapfid eq -1) then return
    endif

    ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=proj_fids,pos=seg_bands,$
         dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
         out_dt=4,out_bname=out_bname,r_fid=olapfid,/IN_MEMORY, background=65535.
    if (olapfid eq -1) then return

  endif else begin

    ; If plotting, need to stack projected angles too:
    if (plot eq 1) then begin
      ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=i_proj_fids,pos=seg_bands,$
           dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
           out_dt=4,out_bname=i_out_bname,r_fid=i_olapfid,out_name=i_outfile, background=65535.
      if (i_olapfid eq -1) then return
      ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=e_proj_fids,pos=seg_bands,$
           dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
           out_dt=4,out_bname=e_out_bname,r_fid=e_olapfid,out_name=e_outfile, background=65535.
      if (e_olapfid eq -1) then return
      ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=g_proj_fids,pos=seg_bands,$
           dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
           out_dt=4,out_bname=g_out_bname,r_fid=g_olapfid,out_name=g_outfile, background=65535.
      if (g_olapfid eq -1) then return
    endif

    ; This one's the final output file...
    ENVI_DOIT,'ENVI_LAYER_STACKING_DOIT',fid=proj_fids,pos=seg_bands,$
         dims=dims,out_ps=[pixs_sorted[0],pixs_sorted[0]],out_proj=proj,$
         out_dt=4,out_bname=out_bname,r_fid=olapfid,out_name=outfile, background=65535.
    if (olapfid eq -1) then return

  endelse

  ;;;;;;;;;;;
  ; CLEAN UP: 
  ;;;;;;;;;;;
  ; Remove projected single bands from memory and disc:
  for i=0,n_elements(proj_fids)-1 do envi_file_mng,id=proj_fids(i),/remove,/delete 

  if (plot eq 1) then begin
    for i=0,n_elements(i_proj_fids)-1 do begin
      envi_file_mng,id=i_proj_fids(i),/remove,/delete
      envi_file_mng,id=e_proj_fids(i),/remove,/delete
      envi_file_mng,id=g_proj_fids(i),/remove,/delete
    endfor
  endif



  ;************************************************************************
  ; CONTINUING ON TO PLOT:
  ;************************************************************************
  if (plot eq 1) then begin
    ; Using the file id of projected data, get total dims, and # of segments:
    
    if (uvalue eq 'plot_overlap') then begin
       envi_file_query, olapfid, dims=totdims, nb=nsegs, bnames=bnames
    endif else begin
      ; Subsetting data:
      DATASELECT:
      envi_select,fid=olapfid,dims=subdims,pos=segs,/file_only,/no_spec,/roi,$
           title='Select Layered DATA and SPATIAL SUBSET:'
      if (olapfid[0] eq -1) then return    ; cancel chosen
      nsegs = n_elements(segs)
      envi_file_query,olapfid,bname=bnames
      pos = strpos(bnames[0],'_Band')
      if (pos eq -1) then begin
        ok = dialog_message('This does not appear to be the projected data.',$
                             title='Inspect EPF Error')
        goto, DATASELECT
      endif

    endelse

    ; Create arrays to be used later:
    averad = fltarr(nsegs)
    ave_i = fltarr(nsegs)
    ave_e = fltarr(nsegs)
    ave_g = fltarr(nsegs)

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ; SUBSETTING WITH ENTIRE OVERLAP AREA:
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (uvalue eq 'plot_overlap') then begin
      ; Create arrays to hold data pulled from memory and create a mask
      ;  showing the overlap area only:
      nx = totdims(2)+1
      ny = totdims(4)+1
      dataplanes = fltarr(nx,ny,nsegs)
      i_dataplanes = fltarr(nx,ny,nsegs)
      e_dataplanes = fltarr(nx,ny,nsegs)
      g_dataplanes = fltarr(nx,ny,nsegs)
      mask = intarr(nx,ny)
      for i=0,nsegs-1 do begin
        dataplanes(*,*,i) = envi_get_data(fid=olapfid, dims=totdims, pos=i)
        i_dataplanes(*,*,i) = envi_get_data(fid=i_olapfid, dims=totdims, pos=i)
        e_dataplanes(*,*,i) = envi_get_data(fid=e_olapfid, dims=totdims, pos=i)
        g_dataplanes(*,*,i) = envi_get_data(fid=g_olapfid, dims=totdims, pos=i)
      endfor
      for j=0,ny-1 do begin
        for i=0,nx-1 do begin
          wz = where(dataplanes(i,j,*) eq 0)
          if (wz[0] eq -1) then mask(i,j)=1 else mask(i,j)=0
        endfor
      endfor

      ; Use mask to find pixels covered by all angles and average pixel values:
      wolap = where(mask eq 1)
      if (wolap[0] eq -1) then begin   ; debug 3/15/09 - add 0 subscript 
        ok = dialog_message('There is no area with coverage at all angles.',$
                             title='EPF Error')
        return
      endif else begin
        for i=0,nsegs-1 do begin
          data = dataplanes(*,*,i)
          i_data = i_dataplanes(*,*,i)
          e_data = e_dataplanes(*,*,i)
          g_data = g_dataplanes(*,*,i)
          averad(i) = mean(data(wolap))
          ave_i(i) = mean(i_data(wolap))
          ave_e(i) = mean(e_data(wolap))
          ave_g(i) = mean(g_data(wolap))
        endfor
      endelse
    endif

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ; SUBSETTING BY USER DEFINED AREA:
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (uvalue eq 'plot_subset') then begin
      ; If subsetting by ROI:
      if (subdims[0] ne -1) then begin
        roidata = envi_get_roi_data(subdims[0],fid=olapfid,pos=segs)
        roidata_i = envi_get_roi_data(subdims[0],fid=i_olapfid,pos=segs)
        roidata_e = envi_get_roi_data(subdims[0],fid=e_olapfid,pos=segs)
        roidata_g = envi_get_roi_data(subdims[0],fid=g_olapfid,pos=segs)
        for i=0,nsegs-1 do begin
          averad(i) = mean(roidata(i,*))
          ave_i(i) = mean(roidata_i(i,*))
          ave_e(i) = mean(roidata_e(i,*))
          ave_g(i) = mean(roidata_g(i,*))
        endfor
      endif else begin
        ; If subsetting by dimensions:
        for i=0,nsegs-1 do begin     
          averad(i) = mean(envi_get_data(fid=olapfid,dims=subdims,pos=i))
          ave_i(i) = mean(envi_get_data(fid=i_olapfid,dims=subdims,pos=i))
          ave_e(i) = mean(envi_get_data(fid=e_olapfid,dims=subdims,pos=i))
          ave_g(i) = mean(envi_get_data(fid=g_olapfid,dims=subdims,pos=i))
        endfor
      endelse
    endif  


    ;Create dotted line plots:
    ; Plot observation angle progression
    x = findgen(nsegs)
    y = fltarr(nsegs,3)
    y(*,0) = ave_i
    y(*,1) = ave_e
    y(*,2) = ave_g
    envi_plot_data,x,y,plot_title=epftag+DET+': Angle Progression',xtitle='EPF segment',$
                   ytitle='Average Angle in ROI',yoff=10,$
                   plot_names=['Incidence','Emergence','Phase']

;     envi_plot_data,ave_i,averad,plot_title=epftag+DET+': '+bandstr,yoff=10,$ 
;                  xtitle='ave angle in ROI (deg)',ytitle='ave '+datafmt+' in ROI',plot_style=1
  
    envi_plot_data,ave_e,averad,plot_title=epftag+DET+': '+bandstr,yoff=350,$ 
                   xtitle='ave angle in ROI (deg)',ytitle='ave '+datafmt+' in ROI',plot_style=1,$
                   plot_names=['Emergence']
    envi_plot_data,ave_g,averad,plot_title=epftag+DET+': '+bandstr,yoff=690,$ 
                   xtitle='ave angle in ROI (deg)',ytitle='ave '+datafmt+' in ROI',plot_style=1,$
                   plot_names=['Phase']

  endif               ;end plot selection


end
