; ENVI event handler routine to analyze a CRISM TRDR or RTR by
;  running the MGM or recalculating the parameters.

; Program can injest PDS or CAT-ENVI data, but requires L or J data.
; Uses standard CDR6-SW wavelength array.
;
; Calls: 
;
; Created: October 2006
; By: Shannon Pelkey
;
; Modification History:
; May 10,2007 S.Pelkey:  --Added option to subset hyperspectral data to MS wavelengths
; Apr 29,2008  MFM: -- Made insensitive to filename case.
; Feb 10,2009 MFM: --For radiance/I/F decision, look at file basename only, 
;                    and look for _RA and _IF instead of just RA, IF to 
;                    prevent catching text in path. Use dataupbase for other
;                    filename based checks too.


PRO mgm_event, ev
  widget_control, ev.id, get_uvalue=uvalue

t1=systime(1)

; Define ENVI functions used so able to create a sav file:
FORWARD_FUNCTION magic_mem_check,widget_auto_base,widget_outfm,$
                 widget_menu,envi_get_slice

;Simple catch to prevent crash
catch, error
if (error NE 0) then begin
  cat_error_report, routine_name='mgm_event'
  return
endif


;************************************************************************************
; GET SETUP INFO & DEFINE OUTPUT ARRAY SIZE:
;************************************************************************************

  ; Have user pick an open data file to work with:
  envi_select,fid=fid,dims=outdims,pos=bands,/file_only,$
       /no_spec,title='Select CRISM File for MGM:'
  if (fid(0) eq -1) then return                  ;cancel selected
  ; Find output dimensions:
  nsout=outdims[2]-outdims[1]+1
  nlout=outdims[4]-outdims[3]+1
  nbin=n_elements(bands)
  nbout = 6                                      ;six OUTPUT PLANES FROM MGM4CRISM
  band_names=['BDLCP','BDHCP','NBDLCP:(BDLCP/BDLCP+BDHCP)','RMS','deltaRMS','NITER']

  ; Make sure the user selected a data file and find relvant info:
  envi_file_query,fid,fname=datafile,wl=waves,wavelength_units=wu
  dataup = strupcase(datafile)
  dataupbase = file_basename(dataup)
  pos1=strpos(strupcase(dataupbase),'TRR')
  if (pos1(0) eq -1) then pos1=strpos(dataupbase,'RTR')
  if (pos1[0] eq -1) then pos1=strpos(dataupbase,'MRRIF')
  if (pos1(0) eq -1) then pos1=strpos(dataupbase,'MRRAL')
  pos2=strpos(strupcase(datafile),'.IMG')
  if (pos1(0) eq -1) or (pos2(0) eq -1) then begin
    ok = dialog_message('Cannot verify that this is a standard radiance, IF, or AL IMG (img) file. Continue?',$
                         title='CRISM MGM Message',/question,/cancel)
    if (ok eq 'Cancel') or (ok eq 'No') then return
  endif

  ; If possible, get detector from filename and make sure not 'S':
  pos=strpos(dataupbase,'S_')
  if (pos ne -1) then det='S' else begin
    pos=strpos(dataupbase,'L')
    if (pos ne -1) then det='L' else begin
      pos=strpos(dataupbase,'J')
      if (pos ne -1) then det='J' else det=''
    endelse
  endelse
  pos=strpos(datafile,'MRR')
  if (pos ne -1) then det='J'
  ; If detector not found from filename, ask user:
  if (det eq '') then begin
    base = widget_auto_base(title='Correct CRISM')
    list = ['S','L','J']
    wm = widget_menu(base, list=list, uvalue='menu',rows=3,$
                     prompt='What detector is this data from?',$
                     /excl, /auto)
    result = auto_wid_mng(base)
    if (result.accept eq 0) then return   ;cancel selected
    det=list(result.menu)
  endif
  if (det eq 'S') then begin
    ok = dialog_message('VNIR product chosen for an IR procedure. Aborting.',$
                         title='MGM Error')
    return
  endif
  if (det eq 'J') then tile=1 else tile=0


  runtype=''
  ; If request is for hyperspectral data, user is asked if they would 
  ; like to use all bands or MS subset:
  if (nbin gt 100) then begin
    base=widget_auto_base(title='MGM Event')
    prompt='Run MGM:'
    list=['Using hyperspectral bands (slow).','With a multispectral subset (faster).']
    wm=widget_menu(base,prompt=prompt,list=list,rows=2,uvalue='menu',/exclusive,/auto)
    result=auto_wid_mng(base)
    if (result.accept eq 0) then return else menu=result.menu
    ; Check entry:
    if (menu eq 0) then runtype=''
    if (menu eq 1) then runtype='_mssub'
  endif


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Create default output filenames:
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  plat=!VERSION.OS_FAMILY     ; check platform
  if (plat eq 'unix') or (plat eq 'UNIX') or (plat eq 'Unix') then slash='/' else slash='\'
  last_slash = strpos(datafile, slash, /reverse_search)
  last_dot = strpos(datafile, '.', /reverse_search)
  if (last_slash ne -1) then in_file_directory = strmid(datafile, 0, last_slash + 1) else in_file_directory=''
  in_file_basename = strmid(datafile, last_slash + 1, last_dot - last_slash -1)
  if (last_dot NE -1) then begin
    in_file_extension = strmid(datafile, last_dot)
  endif else begin
    in_file_extension = ''
  endelse
  default = in_file_directory + in_file_basename + '_mgm'+ runtype + in_file_extension

  ; Check to see if user wants to write to a file or keep in memory:
  ; Load into available bands list regardless.
  base = widget_auto_base(title='Run MGM')
  wo = widget_outfm(base, uvalue='outf',default=default, /auto)
  result = auto_wid_mng(base)
  if (result.accept eq 0) then return
  if ((result.outf.in_memory) ne 1) then outfile=result.outf.name else begin
    ;memory check
    mem_check = magic_mem_check(dims=outdims,in_memory = result.outf.in_memory, $
                   out_dt=4, nb = nbout, out_name = result.outf.name)
    if (mem_check.cancel EQ 1) then return    ; cacel selected
    ;Store result of memory check in result structure
    result.outf.in_memory = mem_check.in_memory
    result.outf.name = mem_check.out_name
    outfile=result.outf.name
  endelse


;********************************************************************************
; MGM Request:
;********************************************************************************

  ; If in-memory, define BSQ array to hold data; if writing to file
  ;  find file names and open file for writing (which will be done BIL):
  if (result.outf.in_memory EQ 1) then begin
    mem_cube = fltarr(nsout,nlout,nbout)         ;six output planes from MGM
    out_status_string = 'Output to Memory'
  endif else begin
    openw, outlun, outfile, /get_lun
    out_status_string = 'Output File: ' + outfile
  endelse

  ;Assemble string to be displayed in ENVI status window
  status_string_array = ['Input File: ' + datafile, out_status_string]
  envi_report_init, status_string_array, base = status_base, title = 'Run MGM', /interupt

  envi_report_inc, status_base, n_elements(nlout)
  
  ; GET DATA FOR USER DEFINED SUBSET:
  ; Run line by line:
  outline=0
  for j=outdims[3],outdims[4] do begin
    envi_report_stat, status_base, outline, nlout, cancel=cancel
    ; If canceled:
    if (cancel) then begin
      print,'Run MGM: Cancel'
      envi_report_init, base = status_base, /finish
      if (result.outf.in_memory EQ 0) then free_lun, outlun
      return
    endif

    ; Get data slice for subset only:
    dataframe=envi_get_slice(fid=fid,line=j,xs=outdims[1],xe=outdims[2])

    ; Run CRISM adapted MGM code:
    mgm4crism, dataframe,waves,runtype, mgmdata

    ;Write spectral slice result to BIL output file or BSQ output memory array
    if (result.outf.in_memory EQ 1) then begin
	mem_cube[*,outline,*] = mgmdata
    endif else begin
	writeu, outlun, mgmdata
    endelse

    outline=outline+1
  endfor

; Stop counter:
envi_tile_done, in_tile_id
envi_report_init, base = status_base, /finish



;***************************************
; Load into memory or write hdr file:
;***************************************

  ; Load into available bands list (needs to be BSQ):
  if ((result.outf.in_memory) eq 1) then envi_enter_data,mem_cube,bnames=band_names,data_ignore_value=65535. else begin
    ; Write ENVI file:
    free_lun, outlun
    envi_setup_head, fname = outfile, ns = nsout, nl = nlout, nb = nbout, $
	   	     interleave = 1, data_type = 4, offset = 0, $
		     bnames = band_names, data_ignore_value=65535., $
                     descrip='MGM Output from '+in_file_basename+in_file_extension,/write, /open
  endelse


t2=systime(1)
etime=(t2-t1)/60.
print,'Elapsed time (min): ',strtrim(etime,2)

END
