;******************************************************************************
;
; pro tilemap_fix_event
;
; Event handler for fixing map info in tile headers from old CAT 
; versions.
;
; CAT versions <= 6.5 wrote tile headers with an undefined datum 
; specified in the map info. When those tiles are read by 
; ENVI versions >= 4.7, the projection defaults to arbitrary
; and invalid coordinates are reported. (Tile data OK 
; in other respects.)
;
; This routine fixes the tile header mapping info.
;
; The input data file must be a CAT-processed tile. 
;
; INPUT PARAMETERS: 
;	ev     Event struct
;
; OUTPUT PARAMETERS: None
;
; KEYWORDS: none
;
; NOTES:
;	Here we open the tile, then it gets closed and reopened 
;	when header is rewritten.
;
; TYPICAL CALL:
;
; USES:
;	cat_error_report
;	pdsfile_is_tile
;	file_is_cat
;	fix_tile_mapping
;
; HISTORY:
;	2010 Feb 25 F. Morgan: First version.
;
;******************************************************************************


pro tilemap_fix_event, ev

widget_control, ev.id, get_uvalue=uvalue

; Define ENVI functions used so able to create a sav file:
FORWARD_FUNCTION envi_select, envi_file_query

; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
	cat_error_report, routine_name='tilemap_fix_event'
	return
endif


;=======================================
;    PICK TILE FILE
;=======================================

; Have user pick an open data file to work with. 
envi_select,fid=in_fid,/file_only,/no_dims,/no_spec, $
      title='Select CRISM Tile to fix:'
if (in_fid[0] eq -1) then return      ;cancel selected
envi_file_query, in_fid, fname=tile_fnm

;=======================================
;    FILE CHECKS
;=======================================

; File must be a CRISM tile, CAT-processed.

cr = string(10b)
if (~pdsfile_is_tile(tile_fnm)) then begin
	msg = file_basename(tile_fnm) + cr $
	   + 'does not appear to be a tile.'+cr $
	   + 'Mapping correction will corrupt non-tile headers.'+cr $
	   + 'Continue with mapping correction?'
	ok = dialog_message(msg, title='Tile Mapping Correction', /cancel)
	if (ok eq 'Cancel') then return
endif

if (~file_is_cat(tile_fnm)) then begin
	msg = file_basename(tile_fnm) + cr $
	   + 'does not appear to be a CAT-generated file.'+cr $
	   + 'Mapping correction may produce incomplete headers.'+cr $
	   + 'Continue with mapping correction?'
	ok = dialog_message(msg, title='Tile Mapping Correction', /cancel)
	if (ok eq 'Cancel') then return
endif


stat = fix_tile_mapping(tile_fid, tile_fnm)

if (stat ne 0) then begin
	if (stat eq -1) then begin
		msg = file_basename(tile_fnm) + cr $
		   + 'Fix failed: invalid file spec.'
		ok = dialog_message(msg, title='Tile Mapping Correction')
	endif else if (stat eq -2) then begin
		msg = file_basename(tile_fnm) + cr $
		   + 'Fix failed: Label not found.'
		ok = dialog_message(msg, title='Tile Mapping Correction')
	endif else if (stat eq -2) then begin
		msg = file_basename(tile_fnm) + cr $
		   + 'Fix failed: Map info not found in label.'
		ok = dialog_message(msg, title='Tile Mapping Correction')
	endif else begin
		msg = file_basename(tile_fnm) + cr $
		   + 'Fix failed: Unspecified error.'
	endelse
endif

return
end
