;******************************************************************************
;
; function autolist_tile_vsadr, obsid, vsid, bin, wvfilt, adr_fnm_list, nfail
;
; Select SCLK-dependent VS ADRs for use with a CRISM tile.
;
; INPUT:
;	obsid      fltarr or lonarr          Array containing observation IDs for 
;	                                     MSP's in the tile
;	vsid       long                      Obs ID for volcano scan to use; 
;	                                      -1 to select from bench temps
;	bin        int                       Observation binmode code
;	wvfilt     int                       Observation wavelength filter code
;	tile_vers  string                    PRODUCT_VERSION_ID from tile label
;	                                     or CAT header 
;
; OUTPUT:
;	adr_fnm_list struct-arr              Array of structures listing ADR for each
;	                                     unique observation
;	                                  adr_fnm_list[i].id = observation ID
;	                                  adr_fnm_list[i].fnm = VS ADR filename ('' if not found)
;	nfail      in                        Number of observations with valid obsid where
;	                                     ADR search failed
;
; RETURN VALUE:
;	status:   0, success
;			 -1, observation index file not found
;			 -2, no valid input observation ID's 
;	         -3, no CDR's of a requested type available
;	         -4, Can't get volcano scan data 
;
; KEYWORDS: none
;
; NOTES:
;	Steps:  Have to find SCLK by looking up the observation ID (from tile DE) 
;	        and then pulling observation SCLK from an observation index file, 
;	        then select appropriate CDR for that time.
;
;	This routine is like autolist_tile_cdr4s.pro except with addition of 
;	volcano scan ID, and deletion of det argument (always L). 
;	Comments there apply here as well.
;
;	Later when you need a CDR for tile pixel [i,j]...
;		1) get obsid[i,j]
;		2) Look up the CDR list element for the ID...
;			       k = where((cdr_fnm_list.id eq obsid[i,j]), nk)
;		3) Pull the corresponding filenames; for cdr_type[j], use:
;			       CDR named cdr_fnm_list[k[0]].fnm[j]
;
;		(Implemented in pick_cdr_for_obs.pro)
;
; USES:
;	crm_bisect_idsrch   (above)
;	load_crismobs_info
;	vs_adr_filename
;
; HISTORY:
;	2009 Sep 19  F. Morgan: First version.
;	2010 Dec 20  MFM   Added input for tile version; added code for 
;	                   cal version; added parameter to select_dr call,
;	                   added batch parameter.
;	2012 Mar 09  MFM  Add 'forward_function'
;	2012 Mar 22  MFM  Add capability to select VS ADR based on thermal 
;	                  spectral shift in addition to sclk alone (input vsid= -1)
;	2012 Mar 26  MFM  Tile version 3 = cal version 3 
;
;******************************************************************************

function autolist_tile_vsadr, obsid, vsid, bin, wvfilt, tile_vers, $
                    adr_fnm_list, nfail, batch=batch

forward_function crm_bisect_idsrch, load_crismobs_info, vs_adr_filename

;====================================
;   OBSERVATION INDEX
;====================================

; Load the observation index and sort it by ID:
stat = load_crismobs_info(obs)
if (stat ne 0) then return, -1
obs = obs[sort(obs.id)]


;====================================
;  SOURCE CAL VERSION
;====================================

tilev = strtrim(tile_vers,2)
if (tilev eq '1') then begin
	cal_vers = '2'
endif else if ((tilev eq '2') or (tilev eq '3')) then begin
	cal_vers = '3'
endif else begin
	if (keyword_set(batch)) then begin
		print,'Assuming TRR3 data in tile!'
		cal_vers = '3'
	endif else begin
		msg = 'Is this a TRR2 tile?'
		ok = dialog_message(msg,title='Correct CRISM',/question,/cancel)
		if (ok eq 'Yes') then begin
			cal_vers = '2'
		endif else if (ok eq 'No') then begin
			cal_vers = '3'
		endif else begin
			return,-4
		endelse
	endelse
endelse

;====================================
;    UNIQUES OBS ID's IN THE TILE
;====================================
; Make a list of unique valid observation ID's represented in the tile...
; sort, unique, drop 65535:
xid = round(obsid)        ; in case it's float type, as in DDE
ss = xid[sort(xid)]       ; sorted ID's
tile_id = ss[uniq(ss)]    ; unique ID's
kgood = where((tile_id ne 65535L),ncdr)     ; NCDR=# CDR's to find for each type
if (ncdr eq 0) then return,-2
tile_id = tile_id[kgood]            ; Keep only the good ones (not 65535). 



;====================================
;   SEARCH CDR's
;====================================

; Initialize cdr filename array (output)
adr_fnm_list = replicate({id: -1L, fnm: ''}, ncdr)

; need a integer copy of vsid...
if (size(vsid,/type) eq 7) then begin
	vsid_int = 0L
	reads, vsid[0], vsid_int, format='(Z)'
endif else begin
	vsid_int = vsid
endelse

if (vsid_int lt 0) then begin
	; Need volcano scan data:
	stat = volcano_scan_list(vsdata)
	if (stat lt 0) then return,-4
endif

; Loop over the valid input obsid
nfail = 0L
for i=0L,ncdr-1L do begin
	; Load the observation ID...
	adr_fnm_list[i].id = tile_id[i]

	; And look for CDR's for this observation...
	; Find current obs id in observation index
	idx = crm_bisect_idsrch(obs, tile_id[i])
	if (idx lt 0) then begin
		adr_fnm_list[i].fnm = ''
		nfail++
	endif else begin
		if (vsid_int lt 0) then begin
			; Signal to look for a volcano scan matching the observation 
			; bench-temp-estimated spectral shift
			if (cal_vers eq '2') then req2=1 else req2=0
			this_id = pick_vs_by_tshift( obs[idx].bench_temp, obs[idx].sclk, $
			       vsdata, require_v2=req2 )
			if (this_id eq '') then begin
				nfail++
				continue
			endif
		endif else begin
			this_id = vsid
		endelse

		; Search for CDR valid at the SCLK of this observation; always batch 
		; or else you get tons of dialogs with every run
		adr_fnm_list[i].fnm = vs_adr_filename(this_id, 'L', bin, wvfilt, $
		            obs[idx].sclk, cal_vers, /batch)
		if (adr_fnm_list[i].fnm eq '') then nfail++
	endelse
endfor

return,0
end
