;******************************************************************************
;
; function cat_tmpfile_name, fnmtag, exten
;
; Create a CAT temp-file name. Routine figures out a directory to put it in, 
; name begins with input tag string, followed by date and time, then a serial 
; number that the routine will increment until a filename not already present
; results.
;
; INPUT:
;	fnmtag   string      A string to include in the tempfile name. (default = 'cattmp')
;	exten    string      File extension to apply (default = absent)
;
; OUTPUT:
;	Parameters: None
;	Return value: 
;		Temp file name; '' if fails
;
; KEYWORDS: None.
;
; NOTES:
;	Filename is:
;		tmp-path/[fnmtag]_yyyymmdd_hhmmss_n[.exten]
;
; 		where 'n' is the unique serial number.
;
;	If input tag contains invalid characters, omit it from filename.
;	(only allow alphanumeric, '_', '-', '.')
;
;	The serial number appended to the file is intended to prevent 
;	filename collisions should two CAT sessions try to write temp files 
;	to the same directory. It does not make collision completely 
;	impossible. Two sessions generating similar temp files within a disk 
;	access cycles of each other might both claim that the same serial 
;	number was free. However, that should be extremely unlikely. 
;	The only way to prevent that would be to incorporate some 
;	sort of process dependent info in the name, but an easy, 
;	portable way to do that isn't obvious to me. -FM
;
; USES:
;
; HISTORY:
;	2008 Nov 21 F. Morgan: First version written
;	2009 Jul 17 F. Morgan: Documentation edits.
;
;******************************************************************************

function cat_tmpfile_name, fnmtag, exten


; Tmp file path
tmp_path = (envi_get_configuration_values()).default_tmp_directory
if (~file_test(tmp_path)) then begin
	msg = 'Invalid temp directory in envi.cfg.'+string(10b) $
   		+ 'Continue writing temp files in current directoy?'
	ans = dialog_message(msg,  title='Invalid TMP directory',/question,/cancel)
	if ((ans eq 'No') || (ok eq 'Cancel')) then retall
	tmp_path = ''
endif
; Make sure its terminated with path separator
tmp_path = strtrim(tmp_path,2)
if (strmid(tmp_path,strlen(tmp_path)-1,1) ne path_sep()) then tmp_path += path_sep()


; Check tag for invalid characters (allow alphanumeric, _, -, .)
if (n_elements(fnmtag) eq 0) then fnmtag = 'cattmp'
xtag = strtrim(fnmtag,2)
if (stregex(xtag,'[^1234567890A-Za-z_\.\-]') ge 0) then begin
	; Invalid tag; use default
	tmp_name = tmp_path + 'cattmp_'
endif else if (strlen(fnmtag) eq 0) then begin
	; Tag absent; use default
	tmp_name = tmp_path + 'cattmp_'
endif else begin
	tmp_name = tmp_path + xtag + '_'
endelse

; Add date and time:
caldat, systime(/julian), mon,d,y, h,m,s
s = round(s)
tmp_name += string(y,format='(I4.4)') + string(mon,format='(I2.2)') + string(d,format='(I2.2)') $
			+ string(h,format='(I2.2)') + string(m,format='(I2.2)') + string(s,format='(I2.2)') 
tmp_name += '_'

; Extension:
if (n_elements(exten) gt 0) then begin
	xt = strtrim(exten,2)
	if (xt ne '') then begin
		if (strmid(xt,0,1) ne '.') then xt = '.'+xt
	endif
endif else begin
	xt = ''
endelse

; Add serial number:
ser = 1
tn = tmp_name + strtrim(ser,2) + xt
while (file_test(tn)) do begin
	ser++
	tn = tmp_name + strtrim(ser,2) + xt
endwhile

return, tn
end
