;******************************************************************************
;
; function crism_bad_bands, swaves, detector
;
; Given a vector of wavelengths and a matching vector of detector ID's, return a
; vector of 0's and 1's indicating which bands are good (1) and which are bad (0).
;
; INPUT:
;	swaves			fltarr		Vector of CRISM wavelengths (nm or microns at "sweet spot")
;	detector		int[arr]	Detector ID; 0=IR, 1=VNIR (see note)
;
; OUTPUT:
;	Parameters:    None
;	Return value:  Vector with 1-to-1 correspondence to the input wavelength vector, 
;                  with 1's (good bands) and 0's (bad bands).
;
; KEYWORDS: none
;
; EXAMPLE:
;	waves = [550., 700., 750., 800., 900., 950., 1000.]
;
;	If 700-800 are all bad bands and 1000. is also bad, the result is:
;	returns [1,0,0,0,1,1,0]
;
;	If no bands are bad, returns [1,1,...,1]
;
; NOTES:
;	Detector ID:
;		If all bands are from the same detector, "detector" input can be 
;		scalar, either 0 (all IR) or 1 (all VNIR). If wavelengths are from a 
;		joined array, supply an array matching "waves" in size with individual 
;		elements designating the detector from which that wavelength came.
;
;	Wavelengths:
;		Bad bands (bad columns on the CRISM detectors) are specified here by 
;		wavelengths at the detector "sweet spot." The CRISM detectors 
;		have spectral smile such that pixels along a detector row are not 
;		at exactly constant wavelengths (so without defining the bad band 
;		condition carefully, a wavelength mt might indicate different 
;		pixels depending on the spatial location of a spectrum). 
;		The "sweet spot" is chosen to select a set of colmns where the 
;		spectral variation is minimal. Sweet spot wavelengths are given
;		in the SW CDR's. 
;
;		The practical consequence of all this is that the input "waves" 
;		should be the sweet-spot wavelengths for an image in order to 
;		select the correct bad bands. 
;
;	Limits:
;		Took limits from Scott Murchie email 9/24/07, with clarification 
;		on precise meanings of limits on 11/1/07. In the conditionals I used 
;		wavelengths between pixels (couple nm off the boundary) so that some 
;		possible later small shifts in the SW CDR might not break this code. 
;		For example VNIR row with SW wavelength 403.62 is bad, the 410.12 row 
;		is good; I use the conditional (w lt 407.) to select bad bands here 
;		so that the SW CDR could shift up to ~3 nm and the same bands would still 
;		be selected. 
;		
; USES:
;	No external procedures.
;
; HISTORY:
;	2007 Nov 20   F. Morgan: First version written and tested.
;	2009 Mar 17   Don't trust wavelength units; check nm or microns
;	2009 Mar 31   MFM Use crism_waves2nm; units chceck in routine, not here
;	2009 Sep 22   MFM Eliminated unused wunits input argument.
;
;******************************************************************************

function crism_bad_bands, swaves, detector

; Wavelength in nm
wvnm = swaves
k = where((abs(wvnm-65535.0) gt 0.1), nk)
if (nk gt 0) then begin
	wvnm = crism_waves2nm(swaves)
endif else begin
	msg = 'BAD BANDS FAIL: no valid wavelengths.'
	ok = dialog_message(msg, title='Apply BBL warning')
	retall
endelse

; Convert scalar detector ID to vector...
nwv = n_elements(swaves)
det = detector
if (n_elements(detector) eq 1) then det=replicate(detector[0],nwv)

; Output vector, all good for now:
bbl = replicate(1,nwv)

; VNIR bad channels:
v = where((det eq 1),nv)
if (nv gt 0) then begin
	w = wvnm[v]
	; b/g = bad/good:  b403.6/g410.1, g637.96/b644.48, b683.59/g690.11, g1023.27/b1029.81
	k = where(((w lt 407.) or ((w gt 641.) and (w lt 686.)) or (w gt 1025.)), nvbad)
	if (nvbad gt 0) then bbl[v[k]]=0
endif

; IR bad channels:
ir = where((det eq 0),nir)
if (nir gt 0) then begin
	w = wvnm[ir]
	; b/g = bad/good:  b1014.45/g1021.00, g2687.98/b2694.58, b2701.19/g2760.68, g3923.47/b3930.15
	k = where(((w lt 1021.) or ((w gt 2692.) and (w lt 2703.)) or (w gt 3926.)), nirbad)
	if (nirbad gt 0) then bbl[ir[k]]=0
endif

return, bbl
end
