;
;  This is a routine to expand a crism EDR or TRDR data product into a
;  expanded cube.  It takes inputs that are generated using the generic
;  PDS reader, readpds, to read the data, but then uses the ROWNUM_TABLE
;  object from the PDS label represent the line numbers for each corresponding
;  row.  We then use this information to expand the cube to the full
;  640 x N x 480, where N is the number of frames collected in the
;  temporo-spatial direction.
;
;  The return value is a structure containing the number of objects
;  from the label, as well as each object.
;  To get the image data only, use result.image
;
;  Version History
;  1.0.0             Initial version
;  1.0.1 2005-08-29  Modified return value to be a structure rather than just
;                    the image portion. This allows us to read the housekeeping
;                    data from the table with the same software.
;  2.0.0 2006-01-10  Significant modification.  Was called read_crismedr(),
;                    but readpds can now read all objects after the restructuring
;                    of the EDR.  The only task remaining is to expand a crism
;                    cube, if so desired.  Another significant change is that this
;                    module assumes that the caller loaded the crism data with
;                    readpds directly, rather than this function calling readpds().
;
;  Howard Taylor
;  JHU/APL
;
;  June 8, 2006 S. Pelkey --modified code to catch problem files where
;                           structure elements equal -1
;

function expand_crism_cube, pds_structure

    ;
    ;  Assume that the CRISM EDR or TRDR has already been loaded using readpds().
    ;  Compare the input structure to a template to ensure all the tags are present
    ;
    reqelems = [ 'IMAGE', 'ROWNUM_TABLE' ]
    numelems = n_elements(reqelems)
    reqcheck = make_array ( numelems, /int, value=0 )
    zcheck = 0
    tags = tag_names(pds_structure)      ; Get tags.
    ntags = n_elements(tags)    ; # tags.

    for iii=0, ntags-1 do begin   ; loop thorugh all the tags
        ; Check to make sure structure elements have data 
        ; (check for 'ROWNUM_TABLE' makes sure it is a structure)
        if (tags[iii] ne 'ROWNUM_TABLE') then zc = (pds_structure.(iii)(0) eq -1) else $
           zc= ((size(pds_structure.(iii)))[2] ne 8)
        zcheck = [zcheck,zc]
        for jjj=0,numelems-1 do begin   ; loop through all the required tags
           reqcheck[jjj] += ( tags[iii] eq reqelems[jjj] )
        endfor
    endfor
    roi = where ( reqcheck eq 0, count )
    roi2 = where ( zcheck eq 1, count2 )
    if ( count gt 0 ) or ( count2 gt 0 ) then begin
;        print, "The argument does not contain at least the IMAGE and ROWNUM_TABLE objects."
        return, 100
    endif

    ;
    ;  Remember - in IDL speak, we prefer the following convention, regardless of
    ;             how the data is stored in the file:
    ;   x dimension = number of columns in spatial frame
    ;   y dimension = number of rows in a spatial frame
    ;   z dimension = number of wavelengths in the spectral cube
    ;
    image_sz = size(pds_structure.image, /stru )

    ;
    ;  Extract the row numbers from the rownum_table object
    ;
    rownums = pds_structure.rownum_table.column1

    ;
    ; prepare the return array based on the number of spectral bands in the cube
    ;
    expanded = make_array( 640, image_sz.dimensions[1], 480, /uint, value=0 );

    ;
    ;  determine the binning by inspecting the x dimension of the data object.
    ;  we need to use this knowledge to create an indexing array into the binned
    ;  data such that an edr with 64 pixels in the x direction ends up producing
    ;  a 3d cube with 640 pixels in the x direction, for example.
    ;
    colroi = rebin(indgen(image_sz.dimensions[0]),640)
    expanded[*,*,rownums] = pds_structure.image[colroi, *, *]

    ;  Because the input structure has already been defined, we cant just reuse
    ;  the pds_structure.image variable because the shape of that array is no
    ;  longer consistent.  To resolve this, we copy all tags from the input
    ;  structure except the IMAGE element.  After copying all of the other items,
    ;  we assign the IMAGE element to the expanded version of the cube we just
    ;  produced.
    ;
    for iii = 0, ntags - 1 do begin
        if ( strupcase(tags[iii]) ne 'IMAGE' ) then begin
            if ( iii eq 0 ) then begin
                data = create_struct ( tags[iii], pds_structure.(iii) )
            endif else begin
                data = create_struct ( tags[iii], pds_structure.(iii), data )
            endelse
        endif
    endfor
    data = create_struct ( 'image', expanded, data )

    return, data
end

