;;-----------------------------------------------------------------------------
;; Name: POINTPDS
;;
;; Purpose: To process the pointer to an object in a PDS file
;;
;; Calling Sequence: 
;;    result = pointpds (label, filename, objectname)
;;
;; Input:
;;    label - String array containing the PDS header
;;    filename - Scalar string containing the name of the PDS file to read
;;    objectname - The name of the object to process pointer information for
;;
;; Output:
;;    result - a structure containing the name of the datafile and the skip
;;            offset in bytes
;;
;; Optional input: none
;;
;; Examples:
;;    To obtain information from TABLE.LBL on a TABLE object:
;;       IDL> label = headpds ("TABLE.LBL", /SILENT)
;;       IDL> pointer = pointpds (label, "TABLE.LBL", "TABLE")
;;       IDL> help, /structure, pointer
;;            FLAG                      1
;;            DATAFILE         "TABLE.TAB"
;;            SKIP                   2056
;;
;; External routines: Pdspar, Remove
;;
;; Modification history:
;;    Written by Puneet Khetarpal, August 2002
;;    For a complete list of modifications, see changelog.txt file.
;;
;; Sept 27 2006: Shannon Pelkey
;;    Inserted kludgey fix to make pointpds run on windows platform too.
;;    Issue was that the program deduced filename by location of "/"
;;    in the directory path--and a forward slash is platform specific.
;;    Kludgey because uses a file_search on CAT_ENVI directory for check.
;;    Search on "slash" variable to find alteration.
;; Sept 28 2006: Shannon Pelkey
;;    Inserted platform/CAT specific work around when EDRHK.FMT file
;;    not found; located in CAT aux_files/
;; Oct 31 2006: Shannon Pelkey
;;    Added more refined check of platform and changed default
;;    CAT_ENVI location on Windows platforms to be in Program Files.
;; Sep 01 2008:  Frank Morgan
;;    Use cat_auxfile_path() to use config-defined aux file path. That, 
;;    along with file_dirname() and path_sep(), eliminates the kludgey stuff.
;; 2009 Jn 12 MFM call pds_remove, not remove()
;;
;;----------------------------------------------------------------------------

function pointpds, label, fname, objname

    ; error protection:
    on_error, 1

    ; obtain record bytes keyword value (must already exist):
    record_bytes = pdspar (label, "RECORD_BYTES")

    ; obtain pointer to objname:
    param = "\^" + objname
    point = pdspar (label, param, COUNT=pointercount)
    if (pointercount gt 1) then begin   ; if multiple pointers, then error
        message, "Error: multiple pointers to " + objname + " object found."
    endif

    ; clean and save pointer as backup:
    point = clean(point[0], /space)
    savepoint = point

    ; remove parentheses from string:
    point = pds_remove(point, ['(',')'])

    ; check for <BYTES> flag and remove it if found:
    byte_offset_flag = 0                  ; initialize byte offset flag to 0
    rightp = strpos (point, "<BYTES>")    ; get start position of <BYTES>
    if (rightp gt -1) then begin          ; if <BYTES> present
        byte_offset_flag++                ; increment byte offset flag
        point = strmid(point, 0, rightp)  ; remove <BYTES> from pointer
    endif

    ; check for double quotes and extract:
    rightp = strpos (point, '"')      ; get index for right quote position 
    leftp = -1                        ; init index for left quote position
    if (rightp gt -1) then begin      ; if right quote present
        leftp = strpos (point,'"', rightp + 1)  ; look for left quote position
    endif

    ; if there was a filename, save it:
    datafile = ""                                 ; init data file name
    if (rightp gt -1 && leftp gt -1) then begin   ; if file name found
        rightp++                                  ; increment right position 
        length = leftp - rightp                   ; determine length of fname
        datafile = strmid (point, rightp, length) ; extract file name

        ; remove the file name from the pointer string:
        length = strlen(point) - leftp            ; determine length left
        point = strmid (point, leftp + 1, length) ; extract the remaining 
    endif else if (rightp eq -1 xor leftp eq -1) then begin  ; else error
        message, "Error: badly formatted file pointer " + savepoint
    endif
 
    ; obtain bytes_offset or skip bytes:
    rightp = strpos (point, ",")                  ; look for a ',' char
    if (rightp gt -1) then begin                  ; if found
        rightp++                                  ; increment position    
        length = strlen(point)                    ; get length of string
        point = strmid (point, rightp, length - rightp) ; extract remaining 
    endif
    ; convert remaining string to long integer
    skip = (strlen(point) eq 0) ? 0 : long(clean(point, /space))

    ; assign the skip bytes for byte offset flag:
    if ((~ byte_offset_flag) && (skip ne 0)) then begin
        skip = (skip - 1) * record_bytes[0]    ; if records skip, then compute
    endif else if (byte_offset_flag && (skip ne 0)) then begin   ; else
        skip--                                 ; decrement skip bytes by 1
    endif

    ; if there is a datafile, then check:
    if (strlen(datafile) gt 0) then begin      ; if not a null filename

        ;===============   START REPLACED CODE 9/1/08  ==================
        ;; Insert kludgey fix to make pointpds run on windows platform too.
        ;;   Kludgey because uses CAT_ENVI directory for check.
        ;; Find the platform CAT is running on:
        ;plat=!VERSION.OS_FAMILY
        ;if (plat eq 'unix') or (plat eq 'UNIX') or (plat eq 'Unix') then begin
        ;    plat='unix'
        ;    slash='/' 
        ;endif else begin
        ;    plat='win'
        ;    slash='\' 
        ;endelse
        ;dir = fname                            ; set directory to filename
        ;rightp = strpos (dir, slash)           ; find a '/' in name
        ;last_slash = rightp                    ; save '/' position
        ;while (rightp ge 0) do begin           ; while there is more '/'
        ;    last_slash = rightp                
        ;    rightp = strpos (dir, slash, rightp + 1) ; find the last '/' position
        ;endwhile
        ;if (last_slash gt 0) then begin        ; if last '/' pos > 0
        ;    dir = strmid (dir, 0, last_slash + 1) ; extract dir name
        ;endif else begin                       ; else 
        ;    dir = ""                           ; no directory name needed
        ;endelse
        ;===============   END REPLACED CODE  ========================

        ; All the above is replaced by the single line...
        dir = file_dirname(fname) + path_sep()

        ; ...now with cat_auxfile_path() below, no other need for 'slash' or 'plat'


        ; if data file is in mixed case:
        fname = dir + datafile                     ; store fname with directory
        openr, unit, fname, error = err, /get_lun  ; open the file

        ; if problems locating .FMT file
        if (err ne 0) then begin
           sp1=strpos(fname,'EDRHK.FMT')
           sp2=strpos(fname,'TRDRHK.FMT')
           if (sp1(0) ne -1) or (sp2(0) ne -1) then begin
              if (sp1(0) ne -1) then fmtfile='EDRHK.FMT' else fmtfile='TRDRHK.FMT'
              fname = cat_auxfile_path() + fmtfile
;              if (plat eq 'unix') then fname='~/CAT_ENVI/aux_files/'+fmtfile else $
;               fname='C:\Program Files\CAT_ENVI\aux_files\'+fmtfile
              openr, unit, fname, error = err, /get_lun  ; try opening again
           endif
        endif
 
        ; if real name is in lower case:
        if (err ne 0) then begin                   ; if error found
            fname = dir + strlowcase (datafile)    ; try lowercase filename 
            openr, unit, fname, error = err, /get_lun
        endif
   
        ; if real name is in upper case: 
        if (err ne 0) then begin                   ; if still error
            fname = dir + strupcase (datafile)     ; try all upcase filename
            openr, unit, fname, error = err, /get_lun
        endif
    
        if (err ne 0) then begin  ; if still error, then issue error
            message, "Error: could not open data file: " + dir + datafile
        endif

    endif else begin          ; else open the file for errors
        openr, unit, fname, error = err, /get_lun
        if (err ne 0) then begin
            message, "Error: could not re-open " + fname
        endif
    endelse
    ; close file and free the unit
    close, unit
    free_lun, unit

    ; store pointer information in a structure:
    pointer = create_struct("datafile", fname, "skip", skip)      

    return, pointer
end
