;******************************************************************************
;
; function crism_sensid_waves, waves
;
; Returns the CRISM sensor id, derived from a vector of wavelengths assumed to match 
; bands from the sensor.
;
; INPUT:
;	waves	fltarr		Vector of wavelengths, either microns or nm, 65535's allowed.
;
; OUTPUT:
;	Parameters:    None
;	Return value:  'S' -> VNIR
;	               'L' -> IR
;	               'J' -> joined
;	               ''  -> no valid wavelengths
;
; KEYWORDS: none
;
; NOTES:
;	Current CDR's show VNIR range 365-1056, IR range 1000-3936 (approx). There 
;	is a litle variation (up to couple nm) between IR CDR's. Allowing some fudge 
;	room, take the assumption that there are VNIR wavelengths present iff the 
;	min valid wavelength < 990, and there are IR wavelengths present iff the 
;	maximum wavelength > 1065. 
;
;	In principal, the routine is not immune to failure. If only a few of the 
;	longest VNIR wavelengths are included (only > 990), they'l not be recognized
;	and it'll report IR instead of VNIR or joined. Similar problems occur if 
;	only a few of the shortest IR wavelengths are present. Hopefully, this will 
;	be very rare in practice.
;
; USES:
;	crism_waves2nm
;
; HISTORY:
;	2007 Nov 19   F. Morgan: First version written and tested.
;	2008 Nov 18   F. Morgan: Renamed routine crism_sensid_waves from crism_sensor_id.
;	2009 Mar 31   F. Morgan: Use crism_waves2nm instead of crism_microns2nm
;
;******************************************************************************

function crism_sensid_waves, waves

forward_function crism_waves2nm


; Working copy of valid wavelengths in nm:
waves_nm = float(waves)
waves_nm = crism_waves2nm(waves_nm)
k = where(((waves_nm lt 65000.) AND (waves_nm gt 0.)), nvw)
if (nvw eq 0) then return,''
waves_nm = waves_nm[k]

if (min(waves_nm) lt 990.) then begin
	; there are VNIR wavelengths present...
	if (max(waves_nm) gt 1065.) then return,'J'
	return,'S'
endif
return,'L'
end
