;******************************************************************************
;
; function file_is_cat
;
; Given a filename, determine whether it is a CRISM CAT file or not.
;
; INPUT:
;	fnm		string		File path+name
;
; OUTPUT:
;	Parameters: none
;	Return value: 0, not CAT file; 1, is CAT file
;
; KEYWORDS: none
;
; NOTES:
;	Returns 1 (true) iff the input file exists, AND is a CAT file. A CAT file 
;	is recognized if "_CAT" appears in the name (not case sensitive) OR
;	if ENVI finds a header with "CAT CRISM" in a description field.
;
; USES:
;	cat_file_check
;
; HISTORY:
;	2007 Aug 20 F. Morgan: First version written and tested.
;   2008 Apr 24 F. Morgan: Made case insensitive for pds files
;	2009 Feb 10 F. Morgan: Check only file basename for _CAT, don't 
;	                       include directory.
;	2009 Jul 09 F. Morgan: Explicitly check cat_file_check return against 0.
;	2009 Nov 26 F. Morgan: Documentation edit.
;
;******************************************************************************

function file_is_cat, fnm

forward_function cat_file_check

; Return 0 (false) if input file does not exist in any case.
if (cat_file_check(fnm, filesfound) eq 0) then return,0

; _CAT in name?
if (strpos(strupcase(file_basename(fnm)), '_CAT') ge 0) then return,1

; File exists but _CAT not in name. See if ENVI finds a header with "CAT CRISM"
; in the description:
envi_open_file, filesfound[0], /no_interactive_query, /no_realize, r_fid=fid
if (fid gt 0) then begin
	envi_file_query, fid, descrip=desc
	envi_file_mng, id=fid, /remove		; remove the file from memory
	if ((n_elements(desc) gt 0) && (strpos(desc,'CAT CRISM') ge 0)) then return,1
endif

; No _CAT in name, and no header with "CAT CRISM" in description... NOT a CAT file:
return,0
end
