function find_cat_config, reload=reload

;*********************************************************************
; Find a CAT config file. Search pattern:
;    Look in the first CAT_ENVI directory in the IDL path.
;    If not there, look in standard CAT install directory according to OS
;        (~/CAT_ENVI/ on mac;  X:\program files\CAT_ENVI\ on pc)
;    If still nothing found use envi_pickfile to let user search.
;    Or, if multiple copies are found, user selects 1 from a 
;    radio button list.
;
; Each of these searches uses file_search() and looks at the whole tree 
; below the noted starting points.
;
; File searches match crismcat*.cfg so user can have a variety of 
; config files for different applications, and pick the one they want
; at run time.
;
; KEYWORD:
;	RELOAD   Set when called for reloading config file, to allow 
;	         user selection if not found automatically.
;
; HISTORY:
;	2009 Feb 02  MFM: Added reload keyword; only go to user search 
;	             if RELOAD set, to avoid envi startup error.
;
;*********************************************************************


forward_function pick1_from_list, envi_pickfile

; Look first in the first CAT_ENVI directory that appears in the IDL path
catcfg_file = ''
p = strpos(strupcase(!PATH),path_sep()+'CAT_ENVI')
if (p ge 0) then begin 
	ch = strmid(!PATH,0,p+10)
	; If not the first directory in path, drop all the directories before it
	ch = strsplit(ch, ':;',/extract)
	cathome = ch[n_elements(ch)-1]
	catcfg_file = file_search(cathome, 'crismcat*.cfg', /fold_case, /fully_qualify_path)
endif
if (strlen(strtrim(catcfg_file[0])) eq 0) then begin
	; That failed, look in standard CAT home according to OS:
	if (strpos(strupcase(!version.os_family), 'WIN') ge 0) then begin
		; Windows; try all drive letters until find or fail
		drv = 'C'
		while ( (byte(drv))[0] le (byte('Z'))[0] ) do begin
			testname = drv+':\program files\CAT_ENVI\'
			if (file_test(testname)) then begin
				catcfg_file = testname
				break
			endif
			; Advance the drive letter by incrementing ascii code
			drv = string(byte(drv)+1B)
		endwhile
	endif else begin
		; Unix-like
		testname = '~/CAT_ENVI/'
		if (file_test(testname)) then catcfg_file = testname
	endelse
	catcfg_file = file_search(testname, 'crismcat*.cfg', /fold_case, /fully_qualify_path)
endif
if (n_elements(catcfg_file) gt 1) then begin
	; Need user to pick between multiple files found:
	catcfg_file = pick1_from_list(catcfg_file,'Select a CAT Config File')
	return, strtrim(catcfg_file[0],2)
endif else if (strtrim(catcfg_file[0]) ne '') then begin
	; Found exactly one file, return it:
	return, catcfg_file[0]
endif else begin
	if (keyword_set(reload)) then begin
		; Did not find any matching files. Have user select it interactively.
		; If user cancels, pickfile will return '' and that will be returned from here.
		catcfg_file = envi_pickfile( title='Select a CAT config file', /no_change)
		return, strtrim(catcfg_file[0],2)
	endif
endelse

end
