;**********************************************************************
;
; function obs_limits, daypath
;
; Find min and max obs IDs for a date specified by its PDS path
; eg  lims = obs_limits('/project/crism/flight/crism_pds_archive/trdr/TRDR/2007_144/' 
; returns lims = [23969, 24026]   [min, max]
;
; INPUT
;	daypath  string    Path to /trdr/TRDR/yyyy_ddd/ on pds archive tree
;
; RETURN
;	lonarr(2) = [min_obs_id, max_obs_id] in the specified path
;	            (=[-1,-1] if no matching directories found)
;
;**********************************************************************

function obs_limits, daypath

forward_function cat_term_path_sep

p = cat_term_path_sep(daypath)
f = file_search(p+'*')
if (f[0] eq '') then return,[-1,-1]
n = n_elements(f)
oid = lonarr(n)
onum = 0L
for i=0,n-1 do begin
	; GET OBS ID FROM PATH... Remove trailing separator if present
	this_dir = strtrim(f[i],2)
	sl = strlen(this_dir)
	if (strmid(this_dir,sl-1,1) eq path_sep()) then this_dir = strmid(this_dir,0,sl-1)
	; Extract TTTIIIIIIII from path...
	p = strpos(this_dir,path_sep(),/reverse_search)
	if (p gt 0) then this_dir = strmid(this_dir, p+1)
	if (strlen(this_dir) eq 11) then begin
		; has to be this length to be a valid data directory; get obs id
		reads, strmid(strtrim(file_basename(f[i]),2), 3,8), onum, format='(Z)'
	endif
	oid[i] = onum
endfor
y = max(oid,min=x)
return, [x,y]
end


;**********************************************************************
;
; function filesearch_obsid, trdrpath, obsid
;
; Search for  on a given date, find one that matches an input
; observation ID.
;
; INPUT
;	trdrpath  string    Path to /trdr/TRDR/ on pds archive tree, 
;	                    path_sep() terminated 
;	                    (like '/project/crism/flight/crism_pds_archive/trdr/TRDR/')
;	obsid    string     Hex target ID 
;
; RETURN
;	Path to directory containing the matching observation; '' if non-existant
;
;**********************************************************************

function filesearch_obsid, trdrpath, obsid

forward_function cat_term_path_sep

ps = path_sep()
;f = file_search(trdrpath+'*/*'+strtrim(obsid,2)+'/',/fold_case)
f = file_search(trdrpath+'*'+ps+'*'+strtrim(obsid,2)+ps,/fold_case)
if (f[0] ne '') then return,cat_term_path_sep(f[0])
return,''
end


;**********************************************************************
;
; function match_obsid, pdspath, mdate, oidtarg, try_file_search=try_file_search
;
; From the observations on a given date, find one that matches an input
; observation ID. Must have already estimated that the target obseration 
; is epected on that day.
;
; INPUT
;	pdspath  string    Path to PDS xDR archive (e.g. .../crism_pds_archive/trdr/TRDR/'
;	mdate    string    yyyy_ddd
;	oidtarg  long      Target observation ID
;
; RETURN
;	Path to directory containing the matching observation
;
; KEYWORD
;	If fail to find the target observation in the expected directory, 
;	and try_file_search is set, then search globally using file_search().
;	[Very  slow, but will find an obsid out of sequence.]
;
;**********************************************************************

function match_obsid, pdspath, mdate, oidtarg, try_file_search=try_file_search

forward_function cat_term_path_sep

ps = path_sep()
xdr_path = cat_term_path_sep(pdspath)
f = file_search(xdr_path + strtrim(mdate,2) + ps + '*',/fold_case)
n = n_elements(f)
oid = 0L
for i=0,n-1 do begin
	x = strtrim(f[i],2)
	if (strlen(x) gt 11) then reads, strmid(file_basename(x),3,8), oid, format='(Z)'
	if (oid eq oidtarg) then return, cat_term_path_sep(f[i])
endfor
if (keyword_set(try_file_search)) then begin
	oidstr = strtrim(string(oidtarg,format='(Z)'),2)
	f = filesearch_obsid(xdr_path, oidstr)
	if (f[0] ne '') then return,f[0]
endif
return,''
end


;*******************************************************************************
;
; function find_pds_dir, pds_xdr_path, obsid, try_file_search=try_file_search
;
; Find the dated PDS directory path containing a TRDR or DDR.
;
;
; INPUT
;	pds_xdr_path   string   Path to PDS archive directory containing dated 
;	                        subdirectories where the observation is stored.
;	                        choices like:
;	                           /project/crism/flight/crism_pds_archive/trdr/TRDR/)
;	                           /project/crism/flight/crism_pds_archive/ddr/DDR/)
;	obsid          string   Hex observation ID
;
; OUTPUT PARAMETERS   none
;
; RETURN VALUE
;	string = path to the PDS directory containing the target observation data files
;
; KEYWORD
;	try_file_search   If set, try file search for .../trdr/TRDR/*/TTTIIIIIIII 
;	                  if bisection fails.
;
; EXAMPLES:
;	ENVI> fp = find_pds_dir('/Volumes/crism/flight/crism_pds_archive/trdr/TRDR', '11B6D')
;	ENVI> print,fp
;	/Volumes/crism/flight/crism_pds_archive/trdr/TRDR/2009_082/FRT00011B6D/
;
;	ENVI> fp = find_pds_dir('/Volumes/crism/flight/crism_pds_archive/ddr/DDR','61C4')
;	ENVI> print,fp
;	/Volumes/crism/flight/crism_pds_archive/ddr/DDR/2007_158/FFC000061C4/
;
; NOTES
;	Could do ls .../trdr/TRDR/*/TTTIIIIIIII with TTT=obs type and IIIIIIII=hex ID,
; 	but that is very slow. Use bisection over dates instead.
; 	This assumes that observation IDs are sequential... that a larger observation ID
; 	never shows up with an earlier date in the path, or vice versa.
; 	If keyword try_file_search is set, will try the ls search if bisection fails.
;
; HISTORY
;	2009 Jun 12  Frank M. First version
;	2010 Mar 08  Frank M. Replaced '/' with path_sep() for Windows compatibility.
;
;*******************************************************************************

function find_pds_dir, pds_xdr_path, obsid, try_file_search=try_file_search


forward_function cat_term_path_sep, cat_strip_nondigit, missionday_from_date, $
                 obs_limits, match_obsid, date_from_missionday

if (~file_test(pds_xdr_path)) then return,''

xdr_path = cat_term_path_sep(pds_xdr_path)     ; add terminating / if absent

oid = cat_strip_nondigit(obsid,/allow_hex)            ; strip non-hex digit characters, including blanks
if (strlen(oid) eq 0) then return,''
oidnum = 0L
reads,oid,oidnum, format='(Z)'


; Earliest possible CRISM PDS archive date
date1 = '2006_270'

; Latest possible CRISM PDS archive date = today
caldat,systime(/julian),m,d,y
days_in_month = [31,28,31,30,31,30,31,31,30,31,30,31]
; leap year?
if ((((y mod 100) ne 0) and ((y mod 4) eq 0)) or ((y mod 400) eq 0)) then days_in_month[1]=29
doy = d
for i=1,m-1 do doy += days_in_month[i-1]
max_date = string(y,format='(I4.4)') + '_' + string(doy,format='(I3.3)')
; bracket mission day
dom_lo = 1
dom_hi = missionday_from_date(date1, max_date)


; Make sure data available on limit days and see if the target observation
; is on one of the limits. 
obslim = obs_limits(xdr_path+date_from_missionday(date1, dom_lo)+path_sep())
while (obslim[0] lt 0) do begin
	dom_lo++
	obslim = obs_limits(xdr_path+date_from_missionday(date1, dom_lo)+path_sep())
endwhile
if ((obslim[0] le oidnum) and (oidnum le obslim[1])) then begin
	; Obs on this day:
	return, match_obsid(pds_xdr_path, date_from_missionday(date1, dom_lo), oidnum, $
			try_file_search=try_file_search)		
endif

obslim = obs_limits(xdr_path+date_from_missionday(date1, dom_hi)+path_sep())
while (obslim[0] lt 0) do begin
	dom_hi--
	obslim = obs_limits(xdr_path+date_from_missionday(date1, dom_hi)+path_sep())
endwhile
if ((obslim[0] le oidnum) and (oidnum le obslim[1])) then begin
	; Obs on this day:
	return, match_obsid(pds_xdr_path, date_from_missionday(date1, dom_hi), oidnum, $
			try_file_search=try_file_search)
endif

; Now, dom_lo and dom_hi are mission day numbers at earliest and latest days 
; with data, and data on neither date matches the requested ID.

while (dom_hi-dom_lo gt 1) do begin
	dom_mid = fix(0.5*(dom_lo+dom_hi))
	mid_path = xdr_path + date_from_missionday(date1, dom_mid) + path_sep()
	obslim = obs_limits(mid_path)
	if (obslim[0] lt 0) then begin
		; no data on that day, bounce around till find a day with data
		bounce = 1
		hit_lo = 0
		hit_hi = 0
		while (obslim[0] lt 0) do begin
			dom_mid = dom_mid - bounce * (-1)^(bounce mod 2)
			if (dom_mid le dom_lo) then begin
				hit_lo = 1
				continue
			endif else if (dom_mid ge dom_hi) then begin
				hit_hi = 1
				continue
			endif
			if (hit_lo and hit_hi) then begin
				; Hit endpoints on both ends and never found a new midpoint day 
				; with data - target not found
				; (Eventually this will happen and terminate the while (1) loop 
				; if the target's not found somehow before that happens)
				; Bisection fails, optionally try file search
				if (keyword_set(try_file_search)) then begin
					; Try the file search, in case obs id is out of order or something
					f = filesearch_obsid(xdr_path, oid)
					if (f[0] ne '') then return,f[0]
				endif
				return,''   ; get here if file search refused
			endif
			mid_path = xdr_path + date_from_missionday(date1, dom_mid) + path_sep()
			obslim = obs_limits(mid_path)
			bounce++
		endwhile
	endif
	; If get here, have dom_lo < dom_mid < dom_hi, all are days with some data,
	; and target observaion would be somewhere between dom_lo and dom_hi, exclusive,
	; if it exists.
	; Is the data at dom_mid? (Last obs_limits call was at dom_mid, path = mid_path...)
	if ((obslim[0] le oidnum) and (oidnum le obslim[1])) then begin
		; Obs on this day:
		return, match_obsid(pds_xdr_path, date_from_missionday(date1, dom_mid), oidnum, $
				try_file_search=try_file_search)
	endif

	; If get here, data not at dom_mid; is it before dom_mid (reset dom_hi = dom_mid)
	; or after dom_mid (reset dom_lo = dom_mid)?
	if (oidnum lt obslim[0]) then begin
		; Target observation is before dom_mid...
		dom_hi = dom_mid
	endif else begin
		; Target observation is after dom_mid...
		dom_lo = dom_mid
	endelse
endwhile

; Get here if target obs "between directories"... not there
if (keyword_set(try_file_search)) then begin
	; Try the file search, in case obs id is out of order or something
	f = filesearch_obsid(xdr_path, oid)
	if (f[0] ne '') then return,f[0]
endif

; If get here, then bisection failed, AND file search came up empty too or was refused...
return,''
end
