;******************************************************************************
;
; function find_sw_cdr, fname, batch=batch, bw=bw
;
; Find a SW CDR for a TRR datafile given the label filename.
;
; INPUT:
;	fname         string          ENVI file ID for open data file
;
; OUTPUT PARAMETERS: None.
;
; RETURN VALUE:
;	Selected filename (with path), or empty string if no applicable CDR found
;
; KEYWORDS:
;	BATCH     No interactive searching if set.
;	BW        If set, find BW instead of SW
;	AUX_PATH  Path to CAT aux_files dir; use in DIL to prevent interactive dialog
;
; NOTES:
;	Selection procedure:
;		1. Find all the SW CDRs for the detector of interest (L or S)
;		2. From those, eliminate all with version < max of the set
;		3. From the remaining files, keep only those with the desired
;		   binning & wavelength filter
;		4. From the remaining files, select the file with the latest
;		   filename SCLK <= the observation SCLK
;		(These latter steps are implemented for non-WA files in select_dr.pro)
;
;	Need to use this routine in get_cat_header_data to avoid recursion. 
;	The standard find_cdr routines call get_cat_header_data, so if 
;	get_cat_header_data calls them, the universe explodes.
;
; USES:
;	list_all_dr
;	select_dr
;	catuser_file_search
;	find_lblfile
;	read_crism_lbl
;
; HISTORY:
;	2010 Nov 23 FM  First version
;	2013 Jun 10 FM  Added aux_path keyword
;
;******************************************************************************

function find_sw_cdr, fname, batch=batch, bw=bw, aux_path=aux_path

forward_function list_all_dr,select_dr,catuser_file_search

;===============================================
;  OBSERVATION INFO
;===============================================

; Use label to avoid recursion with get_cat_header_data
find_lblfile, fname, lblfile, batch=batch
read_crism_lbl, lblfile, ['SPACECRAFT_CLOCK_START_COUNT','MRO:SENSOR_ID'],val
sclk = cat_strip_quotes(val[0])
p = strpos(sclk,'.')
if (p gt 0) then sclk=strmid(sclk,0,p)
det = cat_strip_quotes(val[1])
if (sclk eq '') then return,''
if (det eq '') then return,''


;===============================================
;  FIND SW CDR
;===============================================
if (keyword_set(bw)) then begin
	cdrtype = 'BW'
endif else begin
	cdrtype = 'SW'
endelse
cdr_set = list_all_dr('6', cdrtype, aux_path=aux_path)
swcdr = (select_dr(cdr_set, '6', cdrtype, sclk, det, -1, -1, -1))[0]

;===============================================
;  USER SEARCH
;===============================================
if ((swcdr eq '') and (~keyword_set(batch))) then begin
	msg = 'Unable to find '+cdrtype+' CDR for ' + file_basename(fname) + string(10b) $
	    + 'Do you want to search for it?'
	srch_ttl = 'Find '+cdrtype+' CDR'
	cdrfilt='*.TAB'
	cdrfilt = strlowcase(cdrfilt) + ';' + cdrfilt
	return, catuser_file_search(msg, srch_ttl, cdrfilt)
endif

return,swcdr
end
