;*****************************************************************************
;
; function load_vs_data, det, bincode, wvfilt, obs_sclk, cal_version, $
;            fail_msg, loadall=loadall, batch=batch, aux_path=aux_path
;
; Load volcano scan data (transmission and artifact).
;
; INPUT:
;	det         string          detector code ('S', 'L', or 'J')  [expect 'L']
;	bincode     int             Binning code
;	wvfilt      int             Wavelength filter
;	obs_sclk	string or long	Observation SCLK (see note)
;	cal_version string          Observation calibration version (eg '2' for TRR2)
;
; OUTPUT PARAMETERS:
;	fail_msg    string          Error message; '' if OK
;
; RETURN:
;	vsdata = structarr; -1 if fail
;
; KEYWORDS:
;	loadall    If set, load all available volcano scans (by default, 
;	           skip icy and dusty VS)
;	aux_path   Path to CAT's aux_files directory (avoids cat_auxfile_path
;	           search)
;	Batch      Suppress user search if set
;
; USES:
;	volcano_scan_list
;	get_sw_waves
;	read_crism_lbl
;	find_lblfile
;	get_vs_artifact
;	vs_adr_filename
;
; NOTES:
;	Structure elements:
;		label        string            VS ADR label file
;		vsid         string            Observation ID
;		sclk         long              Observation SCLK
;		bench_temp   float             Optical bench temperature
;		spshift      float             Spectral shift
;		icy          int               Flag, 1=icy, 0=not
;		dusty        int               Flag, 1=dusty, 0=not
;		no_v2        int               Flag, 1=No vers 2, 0 = v2 exists
;		low_noise    int               Flag, 1=low noise, 0=noisy
;		vstrans      fltarr(ns,nb)     Transmission data
;		vsart        fltarr(ns,nb,2)   Artifact data, both scaling wave pairs
;		scale_waves  fltarr(2,2)       Scaling wavelengths
;		interp_waves fltarr(2,2)       Continuum interpolation wavelengths
;		vsvalid      int               1=VS transmission valid, 0=ninvalid
;		artvalid     intarr(2)         1 = artifact valid, 0=invalid
;
; HISTORY
;	2012 Jul 15   FM  First version
;	2013 Jun 11   FM  Add aux_path to get_sw_waves call
;
;*****************************************************************************


function load_vs_data, det, bincode, wvfilt, obs_sclk, cal_version, $
            fail_msg, loadall=loadall, batch=batch, aux_path=aux_path


scale_waves = [[2007.0,1980.0], [2011.0,1899.0]]    ; McGuire, Pelkey

; Get volcano scan list:
stat = volcano_scan_list(vslist, aux_path=aux_path)
if (stat ne 0) then begin
	fail_msg = 'Did not load volcano scan list.'
	return,-1
endif
if (size(vslist, /type) ne 8) then begin
	fail_msg = 'Did not return a struct for volcano scan list.'
	return,-1
endif

; If loadall not set, drop icy, dusty VS:
if (~keyword_set(loadall)) then begin
	k = where((~vslist.dusty and ~vslist.icy), nk)
	if (nk eq 0) then begin
		fail_msg = ' No volcano scans left in list after filtering.'
		return,-1
	endif
	vslist = vslist[k]
endif

if (cal_version eq '2') then begin
	k = where((~vslist.no_v2), nk)
	if (nk eq 0) then begin
		fail_msg = ' No volcano scans left in list after filtering.'
		return,-1
	endif
	vslist = vslist[k]
endif

nvs = n_elements(vslist)


; Get dimensions from first volcano scan label:
adr = vs_adr_filename(vslist[0].vsid, det, bincode, wvfilt, obs_sclk, $
            cal_version, batch=batch, aux_path=aux_path)
if (adr eq '') then begin
	fail_msg = 'Cant find ADR for '+vslist[0].vsid
	return,-1
endif
find_lblfile, adr, adrlbl, batch=batch
if (adrlbl eq '') then begin
	fail_msg = 'Cant find label for '+adr
	return,-1
endif
read_crism_lbl, adrlbl, ['LINE_SAMPLES','BANDS', 'LINES'], val
ns = fix(val[0])
nb = fix(val[1])
nl = fix(val[2])
if ((ns le 0) or (nb le 0) or (nl le 0)) then begin
	fail_msg = 'Incorrect dimensions in label '+adrlbl
	return,-1
endif

; Output structs:
vsstr = { label: '', $
          vsid: '',    $
          sclk: '',    $
          bench_temp: 999.0,  $
          spshift: 1.0e10, $
          icy: 0, $
          dusty: 0, $
          no_v2: 0, $
          low_noise: 0, $ 
          vstrans: fltarr(ns,nb), $
          scale_waves: fltarr(2,2), $
          vsart: fltarr(ns,nb,2), $
          vsvalid: 0, $
	      interp_waves: fltarr(2,2), $
          artvalid: intarr(2)   }
vsdata = replicate(vsstr, nvs)

tmp = fltarr(ns,nb)
for i=0,nvs-1 do begin
	; Copy list data...
	vsdata[i].label = vslist[i].label
	vsdata[i].vsid = vslist[i].vsid
	vsdata[i].sclk = vslist[i].sclk
	vsdata[i].bench_temp = vslist[i].bench_temp
	vsdata[i].spshift = vslist[i].spshift
	vsdata[i].icy = vslist[i].icy
	vsdata[i].dusty = vslist[i].dusty
	vsdata[i].no_v2 = vslist[i].no_v2
	vsdata[i].low_noise = vslist[i].low_noise

	adr = vs_adr_filename(vslist[i].vsid, det, bincode, wvfilt, obs_sclk, $
            cal_version, batch=batch, aux_path=aux_path)
	if (adr eq '') then begin
		vsdata[i].vsvalid = 0
		continue
	endif
	openr,u, adr, /get_lun
	readu,u, tmp
	vsdata[i].vstrans = tmp
	free_lun,u
	vsdata[i].scale_waves = scale_waves
	vsdata[i].vsvalid = 1	
	get_sw_waves,adr,swwv, batch=batch, aux_path=aux_path
	for j=0,1 do begin
		vsa = get_vs_artifact(adr, scale_waves[*,j], swwv, iwave, batch=batch)
		if ((n_elements(vsa) eq 1) && (vsa[0] eq -1)) then begin
			vsdata[i].artvalid[j] = 0
		endif else begin
			vsdata[i].vsart[*,*,j] = vsa
			vsdata[i].interp_waves[*,j] = iwave
			vsdata[i].artvalid[j] = 1
		endelse
	endfor
endfor

return, vsdata
end
