;******************************************************************************
;
; function fix_tile_mapping
;
; CAT versions <= 6.5 wrote tile headers with an undefined datum 
; specified in the projection and mapping data. When those tiles 
; are read by ENVI versions >= 4.7, the projection defaults to 
; arbitrary and invalid coordinates are reported. (Tile data 
; OK in other respects.)
;
; This routine fixes the tile header mapping info.
;
; Tile file must be specified by either an FID if it's already open,
; or by the filename if it's not. (Optionally can specify both.)
;
; INPUT PARAMETERS:
;	tile_fid     long    ENVI file ID for open tile file
; 	tile_fnm     string  Tile filename
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE: int status...
;	 0   success
;	-1   Input file not specified
;	-2   Label not found
;	-3   Failed to get required map info from label
;
; KEYWORDS:
;	BATCH      Run in batch mode, no interactive dialogs
;	R_FID      File ID for tile left open 
;
; NOTES:
;	Here we open the tile, then it gets closed and reopened 
;	when header is rewritten.
;
;	tile_fid is taken as valid if > 0. 
;	tile_fnm is valid if it's defined string type, and it references 
;   an existing file. 
;	If tile_fid and tile_fnm are both valid, they better both 
;	refer to the same file or inserted map info will be wrong.
;
; TYPICAL CALL:
;
; USES:
;	find_lblfile
;	get_cat_header_data
;	read_crism_lbl
;	tilemap_pds2envi
;	write_cat_header
;
; HISTORY:
;	2010 Feb 24 F. Morgan: First version.
;   2010 Nov 17 F. Morgan: add header parameter CAT_PRODUCT_VERSION
;
;******************************************************************************

function fix_tile_mapping, tile_fid, tile_fnm, BATCH=batch, R_FID=r_fid


forward_function get_cat_header_data, write_cat_header

; Make sure tile specification is OK and file's open.
; Need filename to find label, get it if not specified.

; If FID not valid, open from filename if possible or give up
fidtp = size(tile_fid,/type)
if ((fidtp eq 0) || (fidtp eq 7) || (fidtp eq 8) || (tile_fid[0] lt 0)) then begin
	if (size(tile_fnm,/type) ne 7) then return, -1
	envi_open_file, tile_fnm[0], /no_interactive_query, /no_realize, r_fid=tile_fid
	if (tile_fid lt 0) then return, -1
endif

; If tile_fnm undefined or doesn't exist, get it from FID 
; if possible or give up
if (   (size(tile_fnm,/type) ne 7) $
    || (file_search(tile_fnm, /fold_case) eq '')) then begin
	; Filename not valid on input. Get it from query on FID.
	envi_file_query, tile_fid, fname=tile_fnm
	; If that returns undefined tile_fnm, not a valid open file
	if (   (n_elements(tile_fnm) eq 0) $
	    || (file_search(tile_fnm, /fold_case) eq '')) then return, -2
endif

; =============    Get existing header data   ==========

stat = get_cat_header_data(tile_fid, NS=ns, NL=nl, NB=nb, WL=wl, $
	WAVELENGTH_UNITS=wu, BBL=bbl, FWHM=fwhm, BNAMES=bnames, FNAME=fname, $
	INTERLEAVE=interleave, OFFSET=offset, $
	BYTE_SWAP=byte_swap, BYTE_ORDER=byte_order, DIMS=dims, $
	DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DEF_ZRANGE=def_zrange, $
	DEF_STRETCH=def_stretch, DESCRIP=descrip, $
	XSTART=xstart, YSTART=ystart, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	BATCH=batch )

; Find label, use the label from header if can find it, 
; otherwise search for it
lblfile = ''
if (size(cat_pds_label_file,/type) eq 7) then begin
	lblfile = file_search(cat_pds_label_file, /fold_case)
endif
if (lblfile eq '') then begin
	; Need to find label. Need tile filename to do that. 
	; If tile_fnm undefined or doesn't exist, get it from FID 
	; if possible or give up
	if (   (size(tile_fnm,/type) ne 7) $
	    || (file_search(tile_fnm, /fold_case) eq '')) then begin
		; Filename not valid on input. Get it from query on FID.
		envi_file_query, tile_fid, fname=tile_fnm
		; If that returns undefined tile_fnm, not a valid open file
		if (   (n_elements(tile_fnm) eq 0) $
		    || (file_search(tile_fnm, /fold_case) eq '')) then return, -2
	endif
	; Find label (need PDS label to rebuild mapping data)
	find_lblfile, tile_fnm, lblfile, batch=batch
	if (lblfile eq '') then return, -2
endif

; ============    Make new mapping data structs   ==========

mapkeywords = ['MAXIMUM_LATITUDE', 'MINIMUM_LATITUDE', $
    'WESTERNMOST_LONGITUDE', 'MAP_PROJECTION_TYPE']
read_crism_lbl, lblfile, mapkeywords, maplblvals
maplblvals = strtrim(maplblvals,2)
w = where((maplblvals eq ''), nbad)
if (nbad gt 0) then begin
	if (keyword_set(batch)) then return, -3
	msg = 'Failed to get all required map info from label'+ string(10b) $
	    + file_basename(lblfile)
	ok = dialog_message(msg, title='Tile Mapping Correction')
	return, -3
endif

max_lat = double(maplblvals[0])
min_lat = double(maplblvals[1])
westmost_lon = double(maplblvals[2])
map_proj_type = maplblvals[3]
tilemap_pds2envi, westmost_lon, max_lat, min_lat, map_proj_type, $
					map_info, projection_info, /structures


; =============    Write new existing header   ==========

; Downshift def bands...
if (n_elements(def_bands) eq 3) then def_bands = def_bands-1

stat = write_cat_header(FNAME=tile_fnm, NS=ns, NL=nl, NB=nb, $                  ; required
	DATA_TYPE=data_type, INTERLEAVE=interleave, OFFSET=offset, $
	BBL=bbl, BNAMES=bnames, $   ; optional ENVI 
	DATA_IGNORE_VALUE=data_ignore_value, $
	DEF_BANDS=def_bands, DEF_STRETCH=def_stretch, ZRANGE=def_zrange, $
	DESCRIP=descrip, FWHM=fwhm, $
	MAP_INFO=map_info, $
	WAVELENGTH_UNIT=wu, WL=wl, XSTART=xstart, YSTART=ystart, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $                          ; CAT
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_PDS_LABEL_FILE      = lblfile, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	BATCH=batch, /OVERWRITE, R_FID=r_fid)

return,0
end
