;******************************************************************************
;
; pro mtrdrmap_pds2envi
;
; Given selected parameters from a CRISM MTRDR label IMAGE_MAP_PROJECTION 
; object, generate the "map info" and "projection info" structures for a 
; corresponding CAT/ENVI header entry.
;
;
; INPUT:  (parameters from PDS label associated with a MTR; object IMAGE_MAP_PROJECTION)
;	map_proj_type		string	Map projection type (PDS keyword MAP_PROJECTION_TYPE)
;	map_resolution      double  Map resolution, pixels/degree from label
;	center_lat          double  Center latitude, degrees, from label
;	center_lon          double  Center longitude, degrees, from label
;	line_offset         double  Label LINE_PROJECTION_OFFSET
;	sample_offset       double  Label SAMPLE_PROJECTION_OFFSET
;
; OUTPUT:
;	map_info			string	Map info string for envi header
;	projection_info		string	Projection info string for envi header
;
; KEYWORDS:
;
; NOTES:
;	Assumptions:
;		* Tile pixels are square with size PIXEL_SIZE at latitude of true scale
;
;	Map parameters:
;
; USES:
;	No external procedures.
;
; HISTORY:
;	2012 Mar 16 FM: Created
;	2012 Mar 17 FM: Projection checks out against TRR DDR for FRT C202.
;	                But - this routine differs from the tile version 
;	                of map2pds for reasons I don't understand yet.
;
;******************************************************************************

pro mtrdrmap_pds2envi, map_proj_type, axis_radius, map_resolution, $
				        center_lat, center_lon, line_offset, sample_offset, $
					    map_info, projection_info


map_info = ''
projection_info = ''

; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
  	cat_error_report, routine_name='tilemap_pds2envi'
	return
endif

; map scale km/pixel
map_scale = !DPI * axis_radius/180.0d0 / map_resolution

proj_param = dblarr(15)
proj_param[0] = axis_radius * 1000.0d0  ; meters
proj_param[1] = center_lat
proj_param[2] = center_lon
x_off = -1000.0d0 * map_scale * sample_offset
y_off = 1000.0d0 * map_scale * line_offset
pixel_size = 1000.0d0 * map_scale


; Projection type: "equirectangular" = 17.
if (strpos(strupcase(map_proj_type),'EQUIRECTANGULAR') ge 0) then begin
	proj_type = 17
endif else begin
	cat_error_report, routine_name='tilemap_pds2envi', /noidlmsg, $
		cat_msg='Unsupported MAP_PROJECTION_TYPE '+map_proj_type
endelse

; Projection name: like "CRISM MRDR LAT=45" or "CRISM MRDR LAT=05" or
; "CRISM MRDR LAT=00" or "CRISM MRDR LAT=-05" or "CRISM MRDR LAT=-45" 
proj_name = 'CRISM MRDR LAT=' + strtrim(string(center_lat,format='(I3.2)'),2)



projection_info = envi_proj_create(type=proj_type, name=proj_name, $
			params=proj_param)

; Note: MC pixel coordinates are at IDL zero-offset level.
map_info = envi_map_info_create(MC=[0.0d, 0.0d, x_off, y_off], $
      PROJ=projection_info, $
      PS=double([PIXEL_SIZE, PIXEL_SIZE]), units='Meters')


return
end
