;******************************************************************************
;
; pro tilemap_pds2envi
;
; Given selected parameters from a CRISM PDS tile label IMAGE_MAP_PROJECTION 
; object, generate the "map info" and "projection info" strings for a 
; corresponding CAT/ENVI header entry.
;
;
; INPUT:  (parameters from PDS label associated with a tile; object IMAGE_MAP_PROJECTION)
;	westmost_lon		float	Westernmost longitude (PDS keyword WESTERNMOST_LONGITUDE)
;	max_lat				float	Northmost latitude (PDS keyword MAXIMUM_LATITUDE)
;	min_lat				float	Southmost latitude (PDS keyword MINIMUM_LATITUDE)
;	map_proj_type		string	Map projection type (PDS keyword MAP_PROJECTION_TYPE)
;
; OUTPUT:
;	map_info			string	Map info string for envi header
;	projection_info		string	Projection info string for envi header
;
; KEYWORDS:
;	structures		If set, outputs ENVI map info and projection info structures
;	          		instead of strings.
;
; NOTES:
;	Assumptions:
;		* Longitudes from PDS label are positive east
;		* Reference surface = sphere with radius 3396.0000 km
;		* Tile pixels are square with size PIXEL_SIZE at latitude of true scale
;
;	Map parameters:
;		The map_info parameter here named y_off is the distance in meters
;		from the equator to the north edge of the tile.
;
;		The map_info parameter here named x_off is the distance in meters
;		from the prime meridian to the west edge of the tile, along 
;		the parallel of the equatorward edge of the tile (the "latitude of 
;		true scale). For tiles that straddle the equator, this distance is 
;		along the equator.
;
;	String output:
;		String outputs may not be compatible with ENVI version >= 4.7.
;
; USES:
;	No external procedures.
;
; HISTORY:
;	2007 Aug 03 F. Morgan: First version written and tested.
;	2007 Sep 07 FM: Made PIXEL_SIZE double precision
;	2010 Feb 24 FM: Made projection type integer instead of string.
;	2010 Feb 24 FM: Removed datum keyword from envi_proj_create and 
;               envi_map_info_create calls to avoid apparent undefined 
;               datum problems in ENVI 4.7.
;	2010 Feb 24 FM: Add string output warning message.
;
;******************************************************************************

pro tilemap_pds2envi, westmost_lon, max_lat, min_lat, map_proj_type, $
					map_info, projection_info, structures=structures


map_info = ''
projection_info = ''

; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
  	cat_error_report, routine_name='tilemap_pds2envi'
	return
endif

if (~keyword_set(structures)) then begin
	cr = string(10b)
	msg = '******************   WARNING!!!!!!!   ******************'+ cr $
	    + '  In tilemap_pds2envi: Calling for projection and '+ cr $
	    + '  map info strings. Strings may not be compatible with'+ cr $
	    + '  ENVI versions 4.7 or higher.'+ cr $
	    + '  Consider using structure output instead by setting'+ cr $
	    + '  setting /structures keyword.'+ cr $
	    + '********************************************************'
	ok = dialog_message( msg, title='Projection Info Message',/cancel)
	if (ok eq 'Cancel') then return
endif

; CONSTANTS:
REF_RADIUS = 3396000.0d      ; meters
PIXEL_SIZE = '2.3152883359d+02'  ; pixel size in meters
DEG2RAD = !DPI/180.d


; Find equatorward latitude (latitude of true scale)
if (min_lat gt 0.) then begin
	; southmost lat is north, whole tile must be north:
	eqwd_lat = min_lat
endif else begin
	; southmost lat is south
	if (max_lat gt 0.) then begin
		; straddles equator
		eqwd_lat = 0.
	endif else begin
		; whole tile is south
		eqwd_lat = max_lat
	endelse
endelse

; Get a longitude on a +/- 180 degree range:
pmlon = westmost_lon
if (pmlon gt 180.) then pmlon -= 360.d

; Calculate x and y offsets for map_info
x_off = REF_RADIUS * cos(eqwd_lat*DEG2RAD) * pmlon*DEG2RAD
y_off = REF_RADIUS * max_lat*DEG2RAD

; Find center latitude for projection name:
clat = round(0.5*(max_lat+min_lat))

; Projection type: "equirectangular" = 17.
if (strpos(strupcase(map_proj_type),'EQUIRECTANGULAR') ge 0) then begin
	proj_type = 17
endif else begin
	cat_error_report, routine_name='tilemap_pds2envi', /noidlmsg, $
		cat_msg='Unsupported MAP_PROJECTION_TYPE '+map_proj_type
endelse

; Projection name: like "CRISM MRDR LAT=45" or "CRISM MRDR LAT=05" or
; "CRISM MRDR LAT=00" or "CRISM MRDR LAT=-05" or "CRISM MRDR LAT=-45" 
proj_name = 'CRISM MRDR LAT=' + strtrim(string(clat,format='(I3.2)'),2)


if (keyword_set(structures)) then begin
	; Return ENVI structures.
	
	; Projection parameters: 
	proj_param = dblarr(15)
	proj_param[0] = REF_RADIUS
	proj_param[1] = eqwd_lat

	projection_info = envi_proj_create(type=proj_type, name=proj_name, $
				params=proj_param)

	; Note: MC pixel coordinates are at IDL zero-offset level.
	map_info = envi_map_info_create(MC=[0.0d, 0.0d, x_off, y_off], PROJ=projection_info, $
				PS=double([PIXEL_SIZE, PIXEL_SIZE]), units='Meters')

endif else begin
	; Return map_info and proj_info strings, not structures.

	; Build map_info string; 	
	map_info = '{' + proj_name + ', 1.0000, 1.0000, ' $
				+ strtrim(string(x_off,format='(F15.4)'),2) + ', '  $
				+ strtrim(string(y_off,format='(F15.4)'),2) + ', '  $
				+ PIXEL_SIZE + ', ' + PIXEL_SIZE + ', CRISM MRDR, units=Meters}'

	; Build projection info string:
	projection_info = '{'+strtrim(proj_type,2) + ', ' + strtrim(string(REF_RADIUS, format='(F12.1)'),2) + ', ' $
				+ strtrim(string(eqwd_lat,format='(F11.6)'),2) + ', ' $
				+ '0.000000, 0.0, 0.0, CRISM MRDR, ' + proj_name + ', units=Meters}'
endelse

return
end
