;****************************************************************************
;
; function measure_shift_artifact, wa, cube, contin
;
; For shift artifact quantification in hyperspectral data
;
; INPUT:
;	wa      fltarr(ns,nb)      Wavelengths (nm)
;	cube    fltarr(ns,nb,nl)   I/F data, VS corrected
;
; OUTPUT:
;	contin  float              Median of continuum level in artifact region
;
; RETURN
;	Median of shift artifact over the cube, columns either 32-628 
;   or sweetspot.
;
; NOTES
;	This routine derived from vscorr_shift_artifact. 
;
;	Allows non-hyperspectral data; and inputs may be spatially subsampled, 
;	but WA and CUBE must match. Computation is over entire cube.
;
;	Couple of different approaches were considered, both involving 
;	manipulation of corrected I/F within bands 275-295 (unreversed WA):
;	1) second highest - second lowest
;	2) 4-pixel running 2nd difference...
;	      y[0] + y[3] - y[1]-y[2]
;
;	These two give similar results, strongly correlated with 
;	slope near(ish) 1.
;	However, the 4-pixel diff is much less biased by real absorptions...
;	Testing with simulated data - straight diff ends up factor of 2 too large 
;	when there's a 5% absopriotn FWHM 5 pixels in vicinity of 
;	a 2% artifact.
;	-- Use the 4-pixel diff
;
;****************************************************************************


function measure_shift_artifact, wa, cube, contin

WAVE_RANGE = [1947.0, 2066.]    ; wavelength limits of artifact evaluation

s = size(cube)
if (s[0] eq 3) then begin
	nfrm = s[3]
endif else begin
	nfrm = 1
endelse
ns = s[1]
nb = s[2]

; Bands limiting the range of pixels to test for artifact; include bands within 
; WAVE_RANGE at the middle of the WA input (not exactly equal to SW, but 
; band results within one band and often identical):
smid = ns/2
dum = min(abs(wa[smid,*]-WAVE_RANGE[0]), j1)
dum = min(abs(wa[smid,*]-WAVE_RANGE[1]), j2)
b1 = min([j1,j2])    ; order b1,b2 regardless of input WA, WAVE_RANGE order
b2 = max([j1,j2])

nb = 1+b2-b1
xa = fltarr(nb)
pixart = fltarr(ns,nfrm) + 65535.0
for k=0,nfrm-1 do begin
	for i=0,ns-1 do begin
		if (median(cube[i,*,k]) gt 65534.0) then continue
		for j=b1,b2 do begin
			xa[j-b1] = cube[i,j-1,k] - cube[i,j,k] - cube[i,j+1,k] + cube[i,j+2,k]
		endfor
		pixart[i,k] = max(abs(xa))
	endfor
endfor

; continuum estimate over bands near artifact:
contin = 0.0
sub = cube[*,b1:b2,*]
kok = where(((sub lt 65534.0) and (finite(sub))), nok)
if (nok gt 0) then contin = median(sub[kok])

k = where(((pixart lt 65534.0) and (finite(pixart))), nk)
if (nk eq 0) then return,65535.0
return, median(pixart[k])
end
