;mgm4crism
;
; Takes a dataframe (spectral slice) as input to MGM code.
; Wavelength array passed in is from ENVI file.
;
; If code does not converge in 10 iterations, first three
; bands are filled with 65535.
;
; Created:
; October 20,2006 by Shannon Pelkey
; --Based on code by Lisa Kanner and Aline Gendrin
;
; Modification History:
; May 10,2007 S.Pelkey: 
; --Added option to subset hyperspectral data to MS wavelengths
; --Added cut off at 2.54 to avoid CRISM specific 2.6 feature.
;
; May 14,2007 S.Pelkey: 
; --Changed convergence criteria and output to use actual RMS and
; delta RMS; assumed observational error of ~1% for CRISM
;
; May 16,2007 S.Pelkey:
; --Eliminated RMS exit criteria; using detla RMS and niter only
;   based on assessment by Patrick Thollot
;
; Jun 28,2007 S.Pelkey:
; --Confirmed to work with tiles
;


pro mgm4crism, dataframe,inwaves,runtype, mgmout, errflag
;band_names

COMMON besoin2, atmosphere
errflag=0

forward_function mro_crism_lookupwv

; Define arrays manipulated in procedure from input arrays:
data=dataframe
waves=inwaves

dims = size(data)
ns = dims(1)
nb = dims(2)

; Since working with L detector data only, make sure that data has
;  increasing wavelength w/band #: (check 2nd to first and last to
;  avoid CRISM NAN)
; If the wavelength dim has not been flipped yet, do it now:
;  note: this flip only applies to the data used in the subroutine
;        and does not affect the original data.
if (waves(nb-2) lt waves(1)) then begin
  data = reverse(data,2)
  waves = reverse(waves)
endif

; Crop data from 1.1 to 2.54 microns to conserve memory
; (and avoid CRISM upturn near 2.6 due to atm correction):
; (waves array is in microns)
b1 = mro_crism_lookupwv(1.1,waves)
b2 = mro_crism_lookupwv(2.54,waves)
if (b1 eq -1) or (b2 eq -1) then begin & errflag=1 & return & endif
data=data(*,b1:b2)
waves = waves(b1:b2)

; If subsetting of hyperspectral has been requested, subset again to 
;  representative MS wavelengths (values used here are based on  SW_L_v3):
if (runtype eq '_mssub') then begin
  subwaves=[1.07996, 1.15206, 1.21109, 1.25045, 1.25701, 1.26357, 1.27670,$
            1.32921, 1.36861, 1.39489, 1.42773, 1.46716, 1.50003, 1.50661,$
            1.55921, 1.62500, 1.65791, 1.69082, 1.75009, 1.80939, 1.87530,$
            1.92806, 1.97424, 1.98084, 2.00723, 2.06664, 2.11948, 2.13930,$
            2.16572, 2.20538, 2.23182, 2.25165, 2.29133, 2.31779, 2.33102,$
            2.35087, 2.39058, 2.43030, 2.45679, 2.52951,2.54]
  subdata=fltarr(ns,n_elements(subwaves))
  for j=0,n_elements(subwaves)-1 do begin
    b1 = mro_crism_lookupwv(subwaves(j),waves)
    subdata(*,j)=data(*,b1)
  endfor
  data=subdata
  waves=subwaves
endif


; Create output array; keep as BIL for output:
mgmout = fltarr(ns,1,6)
;band_names=['BDLCP','BDHCP','NBDLCP:(BDLCP/BDLCP+BDHCP)','RMS','deltaRMS','NITER']

; Run loop over samples:
for a=0,ns-1 do begin

    ; Only run code for spectral subset if no NANs:
    w = where(data(a,*) eq 65535.)
    if (w(0) eq -1) then begin
      spectre = alog(reform(data(a,*)))
            
      ; Define parameters describing initial Gaussians:
      ; Position info: (should not need changing)
      lespos = [0.950,1.900,2.300]
      errpos = [0.000,0.000,0.000]
      ; FWHM info: (should not need changing)
      leslar = [0.400,0.500,0.560]
      errlar = [0.000,0.000,0.000]
      ; Band strength/depth info: 
      lespro = [0.080,0.080,0.080]     ; probably fine, but could be changed
      errpro = [1.500,1.500,1.500]
      ; Continuum info: (offset in ln(refl), slope, curvature):
      lecont = [-0.9,-0.02,0.000]      ; (could be changed if a better contin required)
      errcon = [1.00,0.5,0.000]        ; (should not need changing)

      lespro = sqrt(lespro)

      lesparams = [lespos,leslar,lespro,lecont]
      leserreurs = [errpos,errlar,errpro,errcon]
      indices = where(leserreurs ne 0)
      lesparamsvar = lesparams(indices)
      leserreursvar = leserreurs(indices)

      Cd0d0m1 = identity(n_elements(waves))*100.
      Cp0p0m1 = identity(n_elements(leserreursvar))

      ; Create covariance array: filled with variances of 
      ;   the parameters that are allowed to vary:
      for i=0,n_elements(leserreursvar)-1 do Cp0p0m1(i,i) = (1./leserreursvar(i))^2

      sortir2 = 0.
      pk = lesparams
      ;pk(n_elements(pk)-3) = max(spectre)

      rms_prev = 0                 ; inital rms value
      iteration = 0             ; initial iteration value
      cutoff = 10               ; iteration cut off
      while sortir2 eq 0 do begin
        ; Find derviatives of Gaussians:
        gk = foncmatparder(waves,pk,leserreurs)
	bout1 = invert((transpose(gk))##Cd0d0m1##gk+Cp0p0m1)

        ; Re-vamp Gaussians and compare to observed spectrum:
        gpk = lafonction(waves,pk)
	bout21 = reform((transpose(gk))##Cd0d0m1##(spectre-gpk))
        bout22 = reform(Cp0p0m1##(pk(indices)-lesparamsvar))

        ; Evaluate Gaussian fits:
        pkP1 = pk
        pkP1(indices) = pk(indices)+bout1##(bout21-bout22)

        rms = sqrt( total((exp(gpk)-exp(spectre))^2) / float(n_elements(spectre)) )
        iteration = iteration + 1

        ; This is where code checks to see if termination 
        ;  criteria are met.
        ; TWO ways to break out:
        ;  1) Delta RMS between last two runs < threshold
        ;  2) # iteration > cutoff
        ;  3rd way, NO LONGER USED:
        ;    --RMS error between observation and fit < observational error

        if (abs(rms-rms_prev) lt 1.e-5) or (iteration gt cutoff) then begin
          ; If any criteria are met, fill output array:
	  sortir2 = 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;STUFF FOR PLOTTING:
;         !P.Charsize = 2.5
;         !P.Multi = [0,1,3,0,1]
;         window,4,  XSize=500,YSize=500
;         plot,waves,gpk-spectre,Position=[0.15,0.85,0.85,0.95],xr = [0.9,2.7], yr=[-0.1,0.1],YTicks=2,YMinor=4
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	        	    	

          facteur = 2*sqrt(2*alog(2))
          ledelta = n_elements(pk)/3-1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;STUFF FOR PLOTTING:
;         plot,waves,[0],xr = [0.9,2.7], yr = [-0.3,0.],Position=[0.15,0.55,0.85,0.80]
;         for m=0,ledelta-1 do oplot,waves,-(pk(m+ledelta*2))^2*exp(-0.5*(waves-pk(m))^2/(pk(m+ledelta)/facteur)^2)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;    
        			
          continuum = pk(n_elements(pk)-3)+pk(n_elements(pk)-2)*waves+pk(n_elements(pk)-1)/waves
          tab = exp([gpk,spectre,continuum])

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;STUFF FOR PLOTTING:
;         plot,waves,exp(gpk),xr = [0.9,2.7], yr = [min(tab)-0.005,max(tab)+0.005],YTitle='Natural Log Reflectance',Position=[0.15,0.10,0.85,0.50],XTitle='Wavelength (microns)'
;         oplot,waves,exp(spectre),linestyle=2
;         oplot,waves,exp(continuum)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

          bd950 = pk(6)^2		           	
          lcppro = pk(7)^2
          hcppro = pk(8)^2
          nbsr = lcppro/(lcppro+hcppro)

          ; Fill output array:
          if iteration gt cutoff then mgmout(a,0,0:2) = 65535. else begin    		
            mgmout(a,0,0) = lcppro
            mgmout(a,0,1) = hcppro
	    mgmout(a,0,2) = nbsr
          endelse
  	  mgmout(a,0,3) = rms
          mgmout(a,0,4) = abs(rms - rms_prev)
          mgmout(a,0,5) = iteration

      endif

        rms_prev = rms
        pk=pkP1

      endwhile
		     
    endif else mgmout(a,0,*) = 65535.  ; if CRISM NAN in spectral range

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;STUFF FOR PLOTTING:
; wait, 3
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		     
endfor


end
