;*****************************************************************************
;
; function pick_vs_by_tshift, obs_tbench, obs_sclk, vsdata, $
;            allow_dust=allow_dust, allow_ice=allow_ice
;
; Choose a volcano scan that's best match to the spectral shift 
; of an observation based on spectral shift derived from optical 
; bench temperature.
;
; INPUT:
;	obs_sclk    string or long  Observation time
;	obs_tbench  float           Optical bench temperature
;	vsdata      struct          Volcano scan data
;
; OUTPUT:
;	obs_shift   float           Observation spectral shift
;
; RETURN:
;	VSID of the matching volcano scan; '' if fail
;
; KEYWORDS:
;	allow_dust    If set, allow dusty volcano scans (default: Don't)
;	allow_ice     If set, allow icy volcano scans (default: Don't)
;	require_v2    If set, don't allow volcano scans without TRR2 versions.
;	aux_path      Path to CAT aux_files; prevents cat_auxfile_path() call
;
; NOTES:
;	vsdata is a structure like that returned by volcano_scan_list()
;	with members including:
;	      label        VS label filename
;	      vsid         5 digit hex ID
;	      spshift      Spectral shift (nm)
;	      icy          0 = not, 1=yes
;	      dusty        0 = not, 1=yes
;
; HISTORY:
;	22 Mar 2012   FM   Created
;	16 Apr 2012   FM   Added aux_path keyword
;
;*****************************************************************************


function pick_vs_by_tshift, obs_tbench, obs_sclk, vsdata, obs_shift, $
            allow_dust=allow_dust, allow_ice=allow_ice, $
            require_v2=require_v2, aux_path=aux_path


if (n_elements(vsdata) eq 0) then return,''


; Set of volcano scans under consideration:
if (keyword_set(allow_dust) and keyword_set(allow_dust)) then begin
	k = indgen(n_elements(vsdata))
endif else if (keyword_set(allow_dust)) then begin
	k = where((~vsdata.icy), nk)
endif else if (keyword_set(allow_ice)) then begin
	k = where((~vsdata.dusty), nk)
endif else begin
	k = where((~vsdata.icy and ~vsdata.dusty), nk)
endelse
if (nk eq 0) then return,''
vs = vsdata[k]

if (keyword_set(require_v2)) then begin
	k = where((~vs.no_v2), nk)
	if (nk eq 0) then return,''
	vs = vs[k]
endif


; =============    Observation Spectral Shift    ===============

; Get the WC coefficients
wc = get_wc_coeff(obs_sclk, aux_path=aux_path)
if (max(wc) gt 65534.0) then begin
	; invalid WC coefficients
	errmsg = 'Invalid WC coefficients in for SCLK '+obs_sclk
	return, ''
endif
; Compute shift:
obs_shift = crism_wavelength_shift(wc, obs_tbench)


; =============    Select volcano scan    ===============

dum = min(abs(vs.spshift - obs_shift), j) 
if (vs[j].spshift gt 1.0e9) then return,''
vsid = vs[j].vsid
select_lbl = vs[j].label

return, vsid
end
