;*****************************************************************************
;
; function pick_vs_for_obs, obs_label, bandsetid, fail_msg, batch=batch, $
;          aux_path=aux_path, noart=noart
;
; Choose a volcano scan that's best match to an observation by trying all 
; those in an input set of VS data arrays and picking the one that minimizes 
; the shift artifact. But first check the obs info list to see if the best 
; volcano scan is already selected for us.
;
; INPUT:
;	obs_label      string            Observation PDS label path+filename
;	bandsetid      int               Scaling wave code: 0=McGuire, 1=Pelkey
;
; OUTPUT PARAMETERS:
;	fail_msg       string             Error message; '' if succeeds
;
; KEYWORD:
;	NOISYBEST:  Returns target ID of the best VS before substitution.
;	NOART:      If set, do not apply artifact correction when testing volcano scans
;	Batch       Suppress user search if set
;
; RETURN:
;	VSID of the matching volcano scan; '' if fail
;
; USES:
;	pick_vs_by_trial
;
; NOTES:
;
; HISTORY:
;	16 Jul 2012   FM   Created 
;	21 Aug 2012   FM   Try reading pick from obs info file before computing trials
;	11 Jun 2013   FM   Add aux_path to find_wavelength_file call 
;
;*****************************************************************************


function pick_vs_for_obs, obs_label, bandsetid, fail_msg, batch=batch, $
         aux_path=aux_path, noart=noart


fail_msg = ''

; ============================================
;    Try obs info file
; ============================================

; Need obs ID to select the obs info record; assume PDS filename 
; with ID in characters 4-11
fb = file_basename(obs_label)
p = strpos(fb, '.', /reverse_search)
if (p gt 0) then fb=strmid(fb,0,p)
if (strlen(fb) gt 11) then begin
	idstr = strmid(fb,3,8)
	isdig = cat_is_digit(idstr, /allow_hex)
	if (min(isdig) eq 1) then begin
		; All characters are hex digits as expected, proceed...
		idnum = 0L
		reads, idstr, idnum, format='(Z)'
		
		; Now load obs info...
		stat = load_crismobs_info(obsinfo, aux_path=aux_path)
		if (stat eq 0) then begin
			k = where((obsinfo.id eq idnum), nk)
			if (nk eq 1) then begin
				; Found it. If it's not '00000' which is flag for 
				; obs list failure to select VS, or old list without 
				; VS selection in it, then we're done:
				vs = strtrim(obsinfo[k[0]].vspick[bandsetid],2)
				if (strmid(vs,0,5) ne '00000') then return, vs
			endif
		endif
	endif
endif

; ============================================
;   No obs info selection; try all VS
; ============================================

; Load volcano scan data for this observation:
vsdata = load_vs_for_obs(obs_label, fail_msg, batch=batch, aux_path=aux_path)
if (size(vsdata,/type) ne 8) then begin
	fail_msg = 'Failed to load volcano scan data.'
	return,''
endif

; Drop any invalid transmission data:
k = where((vsdata.vsvalid), nk)
if (nk eq 0) then begin
	fail_msg = 'No valid volcano scan data loaded.'
	return,''
endif
vsdata = vsdata[k]

; Reformat matrices for input to pick_vs_by_trial:
vst = vsdata[*].vstrans                            ; ns x nb x nv
vsart = reform(vsdata[*].vsart[*,*,bandsetid])     ; ns x nb x nv
interp_waves = reform(vsdata[*].interp_waves[*,bandsetid])   ; 2 x nv


; Load cube sample:
read_crism_lbl, obs_label, ['LINE_SAMPLES', 'BANDS', 'LINES'], val
ns = fix(val[0])
nb = fix(val[1])
nl = fix(val[2])
; Line range for subsample, never closer than 2 lines to edge:
l1 = (nl/2-5) > 1
l2 = (nl/2+5) < (nl-2)
nsub = 1+l2-l1
irfnm = obs_label
p = strpos(irfnm,'.',/reverse_search)
irfnm = strmid(irfnm,0,p) + '.IMG'
cube = fltarr(ns,nb,nsub)
openr,udata, irfnm, /get_lun
aa = assoc(udata,fltarr(ns,nb))
for i=0,nsub-1 do cube[*,*,i] = aa[i+l1]
free_lun,udata

; Get WA CDR:
wafnm = find_wavelength_file(irfnm, aux_path=aux_path)
openr,u, wafnm, /get_lun
wa = fltarr(ns,nb)
readu,u, wa
free_lun,u

; If some artifacts not valid, skip artifact patch:
if (min(vsdata.artvalid[bandsetid]) eq 0) then noart = 1

vsid = pick_vs_by_trial(cube, wa, vst, vsart, interp_waves, bandsetid, $
	vsdata.vsid, vsdata.low_noise, fail_msg, noisybest=noisybest, noart=noart)

return, vsid
end
