;******************************************************************************
;
; function reconstruct_pds_path, pds_basename
;
; Given just the basename of a PDS file, return its path under a PDS archive
; tree. (No guarantee that the file with that basename actually exists at 
; the path returned.)
;
; INPUT:
;	pds_basename        string       Basename of a CRISM PDS data file 
;
; OUTPUT PARAMETERS: none
;
; RETURN VALUE:
;	Path to the PDS file ('' if unable to determine a path)
;
; KEYWORDS: none
;
; NOTES:
;	The input basename may contain a path, which is ignored.
;
;	The input basename may contain appended text between the PDS filename 
;	root and its extension. 
;
;	For example, you may have:
;		pds_basename = '/some/processing/outputdir/FRT0000ABCD_07_IF165L_TRR1_CAT_corr.IMG'
;
;	cat_pds_path()  **MUST**  be able to find the PDS root, or this
;	routine will not find a PDS path. [Best way to ensure this is to set
;	CRISM_PDS_PATH in the CAT config file.]
;	(Could try to find PDS root from the input filename, by supplying as argument 
;	to cat_pds_path. But: 1) input might be basename only, and 2) if it's there, 
;	you already have the PDS path and maybe there's no point calling this routine.)
;
;	FILENAME REQUIREMENTS...   to be properly recognized:
;		Tile: '_MRR' in name to recognize as tile,
;		      first contiguous string of numerical digits = CRISM tile number
;		      for determining Mars chart number (e.g. T0526_...)
;		DDR:  '_DDR' and '_DE' in filename, starts with 'tttnnnnnnnn' 
;		      where ttt=obs mode, nnnnnnnn = obs ID
;		LDR:  '_LDR' and '_DE' in filename, starts with 'tttnnnnnnnn' 
;		      where ttt=obs mode (LMB), nnnnnnnn = obs ID
;		TRDR: '_TRR' in filename, starts with 'tttnnnnnnnn' 
;		      where ttt=obs mode, nnnnnnnn = obs ID
;		CDR:  starts with either 'CDR4' or 'CDR6'
;		      If CDR4, first '_' followed by 2-letter type
;		      If CDR6, third '_' followed by 2-letter type
;
;	Does not guarantee that a file with the input basename actually exists at 
;	the returned path.
;
; USES:
;	cat_pds_path
;	crism_tile_chart
;	find_pds_dir
;	cat_term_path_sep
;	pdsfile_is_tile
;
; HISTORY:
;	2009 Jun 12  F. Morgan: First version written and tested.
;	2009 Jul 17  F. Morgan: Added LDR filetype (limb scan DDRs)
;	2012 Nov 26  FM: Bug fix; return, not reurn
;
;******************************************************************************

function reconstruct_pds_path, pds_basename


forward_function cat_term_path_sep, cat_pds_path, crism_tile_chart, $
       pdsfile_is_tile, find_pds_dir

; Find root to PDS tree; no arguments - config only, don't try from input path, 
; see note above:
pds_path_root = cat_pds_path()
if (~file_test(pds_path_root)) then return,''

pdsbase = strupcase(strtrim(file_basename(pds_basename),2))    ; case insenitive
psep = path_sep()
if (pdsfile_is_tile(pdsbase)) then begin
	; Path is mrdr/MRDR/MCxx where xx is the Mars chart number, 01-30.
	; Need to deduce mars chart from the filename
	mcnum = crism_tile_chart(pdsbase)
	if (mcnum lt 1) then return,''
	pds_path = pds_path_root + 'mrdr'+psep+'MRDR'+psep+'MC' $
			+ string(mcnum, format='(I2.2)') + path_sep()
	if (~file_test(pds_path)) then return,''
endif else if ((strpos(pdsbase,'_DDR') gt 0) and (strpos(pdsbase,'_DE') gt 0)) then begin
	; DDR are in ddr/DDR/yyyy_ddd/tttnnnnnnnn/
	; where yyyy_ddd = date, ttt = observation type (FRT, MSP, etc) 
	; nnnnnnnn = observation ID, hex.
	; Assume first 11 characters of pdsbase are the type/ID string from the 
	; original filename... like FRT000028BA_07_IF165S_TRR2.IMG
	p = strpos(pdsbase,'_')
	if (p ne 11) then return,''
	obsid = strmid(pdsbase,p-8,8)
	xdr_path = pds_path_root + 'ddr'+psep+'DDR'
	pds_path = cat_term_path_sep(find_pds_dir(xdr_path, obsid, /try_file_search))
	if (pds_path eq '') then return,''
endif else if ((strpos(pdsbase,'_LDR') gt 0) and (strpos(pdsbase,'_DE') gt 0)) then begin
	; LDR (limb scan DDRs) are in ldr/LDR/yyyy_ddd/tttnnnnnnnn/
	; where yyyy_ddd = date, ttt = observation type (LMB) 
	; nnnnnnnn = observation ID, hex.
	; Assume first 11 characters of pdsbase are the type/ID string from the 
	; original filename... like LMB00002B5C_07_DE227S_LDR0.IMG
	p = strpos(pdsbase,'_')
	if (p ne 11) then return,''
	obsid = strmid(pdsbase,p-8,8)
	xdr_path = pds_path_root + 'ldr'+psep+'LDR'
	pds_path = cat_term_path_sep(find_pds_dir(xdr_path, obsid, /try_file_search))
	if (pds_path eq '') then return,''
endif else if (strpos(pdsbase,'_TRR') gt 0) then begin
	; TRDR are in trdr/TRDR/yyyy_ddd/tttnnnnnnnn/
	; where yyyy_ddd = date, ttt = observation type (FRT, MSP, etc) 
	; nnnnnnnn = observation ID, hex.
	; Assume first 11 characters of pdsbase are the type/ID string from the 
	; original filename... like FRT000028BA_07_IF165S_TRR2.IMG
	p = strpos(pdsbase,'_')
	if (p ne 11) then return,''
	obsid = strmid(pdsbase,p-8,8)
	xdr_path = pds_path_root + 'trdr'+psep+'TRDR'
	pds_path = cat_term_path_sep(find_pds_dir(xdr_path, obsid, /try_file_search))
	if (pds_path eq '') then return,''
endif else if (strmid(pdsbase,0,4) eq 'CDR4') then begin
	p = strpos(pdsbase,'_')
	cdr_type = strmid(pdsbase,p+1,2)
	; CDR4 may be in edr/CDR/tt/ or in edr/CDR/yyyy_ddd/tt where tt=cdr_type
	cdr_name_start = strmid(pdsbase,0,p+3) + '*'
	; Look in edr/CDR/tt/
	p_filter = pds_path_root + 'edr'+psep+'CDR'+psep+cdr_type+psep+cdr_name_start
	pds_path_all = file_search(p_filter, /fold_case, /fully_qualify_path)
	if (pds_path_all[0] ne '') then begin
		pds_path = strtrim(file_dirname(pds_path_all[0], /mark_directory),2)
	endif else begin
		; Not in edr/CDR/tt/, look in edr/CDR/yyyy_ddd/tt/... assume year is 2xxx
		p_filter = pds_path_root + 'edr'+psep+'CDR'+psep+'2*'+psep+cdr_type+psep+cdr_name_start
		pds_path_all = file_search(p_filter, /fold_case, /fully_qualify_path)
		if (pds_path_all[0] ne '') then begin
			pds_path = strtrim(file_dirname(pds_path_all[0], /mark_directory),2)
		endif else begin
			pds_path = ''
		endelse
	endelse
endif else if (strmid(pdsbase,0,4) eq 'CDR6') then begin
	cdrseg = strsplit(pdsbase,'_',/extract)
	cdr_type = cdrseg[3]
	p = strpos(pdsbase, '_'+cdr_type)
	cdr_name_start = strmid(pdsbase,0,p+3) + '*'
	p_filter = pds_path_root + 'edr'+psep+'CDR'+psep+cdr_type+psep+cdr_name_start
	pds_path_all = file_search(p_filter, /fold_case, /fully_qualify_path)
	if (pds_path_all[0] ne '') then begin
		pds_path = strtrim(file_dirname(pds_path_all[0], /mark_directory),2)
	endif else begin
		pds_path = ''
	endelse
endif else begin
	; No valid file type identified, can't interpret...
	pds_path = ''
endelse

return,pds_path
end
