;******************************************************************************
;
; pro select_atmt_source, atmt_src, vsid, nouser=nouser, noauto=noauto
;
; Have user a source for atmospheric transmission spectra. Choices are 
;   auto    - AT CDR pre-selected for optimum wavelength shift match, SCLK based
;	user    - User selects a volcano scan from a list of all of them
;	default - Default volcano scan
;
; INPUT: no parameteers
;
; OUTPUT:
;	atmt_src    string   Selected source for atm transmission; '' if fail
;	vsid        string   Selected volcano scan observation ID (valid only 
;	                     for atmt_src = 'user' or 'default'
;
; KEYWORDS: 
;	NOUSER:   Disable user selection from list
;	NOAUTO:   Disable SCLK-based auto-select
;	noTbench: Disable bench temperature-based shift match
;	notrial:  Disable empirically optimized choice for observation
;
; NOTES:
;
; USES:
;	pick1_from_list.pro
;	list_vsat
;	select_atmt_source   (recursive)
;	get_catcfg_param
;
; HISTORY:
;	2009 Jan 02   F. Morgan: First version
;	2012 Mar 07   FM:  Annotate VSID list with icy, dusty
;	2012 Mar 20   FM:  Bug fix: Cut appendage like "ICY" off vsid
;	2012 Aug 21   FM:  Add empirically optimized for observation selection
;
;******************************************************************************

pro select_atmt_source, atmt_src, vsid, nouser=nouser, $
     noauto=noauto, noTbench=noTbench, notrial=notrial

forward_function pick1_from_list, list_vsat, get_catcfg_param


ICYVS = ['9e04', '94b5', 'a3f6']
DUSTYVS = ['6822']

;=============================================
; IDENTIFY DEFAULT VOLCANO SCAN
;=============================================

; Look first in the config file:
vsid_default = get_catcfg_param('CAT_DFLT_VOLCANOSCAN')
; If not specified, use the classic
if (vsid_default eq '') then vsid_default = '61C4'
vsid_default = strtrim(vsid_default,2)


;=============================================
; SELECT SOURCE
;=============================================

if (keyword_set(nouser) && keyword_set(noauto) && (noTbench)) then begin
	; Only choice left:
	atmt_src = 'default'
	vsid = vsid_default
	return
endif

; List:
atmt_src_ops = ['Default volcano scan '+vsid_default]
if (~keyword_set(nouser)) then begin
	atmt_src_ops = ['User-selected volcano scan', atmt_src_ops]
endif
if (~keyword_set(noauto)) then begin
	atmt_src_ops = ['Pre-selected volcano scan optimum for thermal period', $
	             atmt_src_ops]
endif
if (~keyword_set(noTbench)) then begin
	atmt_src_ops = ['Best match to bench temperature-derived spectral shift', $
	             atmt_src_ops]
endif
if (~keyword_set(notrial)) then begin
	atmt_src_ops = ['Empirically optimized for this observation', $
	             atmt_src_ops]
endif


; Pick from the list and decode the selection:
selected_option_str = pick1_from_list(atmt_src_ops, $
                              'Choose how CAT picks atm transmission reference', $
                              user_cancelled=user_cancelled)
if (user_cancelled) then retall
selected_option_str = strupcase(selected_option_str)
if (strpos(selected_option_str, 'DEFAULT') ge 0) then begin
	; For default, we're done...
	atmt_src = 'default'
	vsid = vsid_default
	return
endif else if (strpos(selected_option_str, 'EMPIRICALLY') ge 0) then begin
	; Selection by trial... done 
	atmt_src = 'trial'
	return
endif else if (strpos(selected_option_str, 'PRE-SELECTED') ge 0) then begin
	; Pre-selected... done all we can here too
	atmt_src = 'auto'
	return
endif else if (strpos(selected_option_str, 'BENCH') ge 0) then begin
	; Bench temp... done all we can here
	atmt_src = 'tbench'
	return
endif else if (strpos(selected_option_str, 'USER') ge 0) then begin
	; Can't return from here yet, still need VSID choice
	atmt_src = 'user'
endif else begin
	; Selection failed, fall back to default
	atmt_src = 'default'
	vsid = vsid_default
	return
endelse


; If you get here, user selected 'user'. Need to pick v.s. ID

; User needs to select a volcano scan observation ID
; Get a list of volcano scan IDs pulled from ADR files.
stat = list_vsat(vsid_list)
if (stat ne 0) then begin
	; User selection failed, try again, disable user selection
	snl = string([10b])
	ok = dialog_message('User selection FAILED. Couldnt get VSID list.' + snl $
	        + 'You may not have required VS ADRs in aux_files path.' + snl $
			+ 'Retry selecting atmospheric transmission source,' + snl $
			+ 'user selection will be disabled.', $
			title='Retry atm transmission' )
	select_atmt_source, atmt_src, vsid, /nouser, noauto=noauto
	return
endif

; Annotate the volcano scan list - tell which are icy or dusty...
vsid_list = strupcase(vsid_list)
nvs = n_elements(vsid_list)
for i=0,n_elements(ICYVS)-1 do begin
	k = where((strpos(vsid_list,strupcase(ICYVS[i])) ge 0), nk)
	if (nk gt 0) then vsid_list[k[0]] += '  (Icy)'
endfor
for i=0,n_elements(DUSTYVS)-1 do begin
	k = where((strpos(vsid_list,strupcase(DUSTYVS[i])) ge 0), nk)
	if (nk gt 0) then vsid_list[k[0]] += '  (Dusty)'
endfor

; Pick obs ID from the list:
vsid = strupcase(pick1_from_list(vsid_list, $
                         'Choose a volcano scan observation ID', $
                         user_cancelled=user_cancelled))
if (user_cancelled) then retall
if (vsid eq '') then begin
	; Failed to select a volcano scan observation
	snl = string([10b])
	ok = dialog_message('User selection FAILED. ' + snl $
			+ 'Retry selecting atmospheric transmission source,' + snl $
			+ 'user selection will be disabled.', $
			title='Retry atm transmission' )
	select_atmt_source, atmt_src, vsid, /nouser, noauto=noauto
	return
endif

; Cut off any appendage like "ICY"
vsid = strtrim(vsid,2)
p = strpos(vsid, ' ')
if (p gt 0) then vsid=strmid(vsid,0,p)

return
end
