;******************************************************************************
;
; function select_dr, candidate_drs, dr_level, dr_type, $
;           obs_sclk, obs_det, obs_bin, obs_wvfilt, obs_cal_version
;
; Select a data record (CDR or ADR) from a list of candidates.
;
; INPUT:
;	candidate_drs   strarr          List of CDR filenames expected to include the needed CDR
;	dr_level        string or int   CDR level (4 or 6) or 'ADR' for ADR
;	dr_type         string          CDR type (e.g. WA, SW, AT)
;	obs_sclk        string or long  Observation SCLK
;	obs_det         string          Observation detector code ('S', 'L', or 'J')
;	obs_bin         int             Observation binmode code (not used for CDR6)
;	obs_wvfilt      int             Observation wavelength filter code (not used for CDR6)
;	obs_cal_version string          Observation calibration version (required 
;	                                only for VS, AT)
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE:
;	Selected filename, or empty string if no applicable data record found
;
; KEYWORDS:
;	BATCH     If set, no interactive searching.
;
; NOTES:
;	SCLK string format = 'P/SSSSSSSSSS' or just 'SSSSSSSSSS'
;	                      (where P = sclk partition, SSSSSSSSSS=ticks; 
;	                      full length of tick string not required)
;
;   Expects:
;       detector:           one letter
;		bin mode:           one digit      (not used for CDR6)
;		wavelength filter:  one digit      (not used for CDR6)
;       If you want to default on any of these,, use '?', empty string, or 
;		string > 1 character long; anything strtrim(variable) != 1 will 
;       default to a single character wildcard (?).
;
;	Selection procedure:
;		1. Find all the CDRs of a given type for the detector of interest (L or S)
;		2. From those, eliminate all with version < max of the set
;		3. From the remaining files, keep only those with the desired
;		   binning & wavelength filter
;		4. From the remaining files, select the file with the latest
;		   filename SCLK <= the observation SCLK
;
;	For now, SCLK partition is ignored; at each partition, MRO has reset 
;	SCLK such that the SCLK value is very nearly, if not exactly, continuous.
;	For CRISM purposes, continuity is certainly sufficient to guarantee 
;	preservation of order.
;
;   Filenames:
;		CDR6:
;		(ProductType)(Level)_(Partition)_(Time)_(Product)_(SensorID)_version.(Ext)
;
;		CDR4:
;		(ProductType)(Level)(Partition)(Time)_
;		(Product)(FrameRate)(Binning)(ExposureParameter)(WavelengthFilter)(Side)(SensorID)_
;		version.(Ext)
;
; USES:
;	get_pds_version
;	extract_dr_sclk
;	select_sclk
;	find_lblfile
;	read_crism_lbl
;	cat_is_digit
;
; HISTORY:
;	2009 Sep 16 FM  First version
;	2010 Nov 22 FM  Add obs_cal_version argument. For VS and AT, 
;	                match CDR source data version to data cal version.
;	2012 Aug 07 FM  Add [0] subscript on sclk_idx checking select_sclk 
;	                output so don't get "Expression must be a scalar..." error
;
;******************************************************************************

function select_dr, candidate_drs, dr_level, dr_type, $
			obs_sclk, obs_det, obs_bin, obs_wvfilt, obs_cal_version, $
			batch=batch

forward_function get_pds_version, extract_dr_sclk, select_sclk, cat_is_digit

;===================================================
;         VALIDATE INPUT
;===================================================

if (n_elements(candidate_drs) eq 0) then return,''
up_candidate_list = strupcase(strtrim(candidate_drs,2))

; Check data record type validity
stype = strupcase(strtrim(dr_type,2))
if (strlen(stype) ne 2) then return,''

; Convert obs stuff to string; if detector invalid, fail; if bin mode 
; or wavelength filter invalid, ignore - set to '[0-9]' = regex default single digit
sdet = ''
if (n_elements(obs_det) eq 1) then sdet=strupcase(strtrim(obs_det[0],2))
if ((sdet ne 'L') && (sdet ne 'S') && (sdet ne 'J')) then return,''

sbin = ''
if (n_elements(obs_bin) eq 1) then sbin=strtrim(obs_bin[0],2)
if (strlen(sbin) ne 1) then sbin = '[0-9]'

swf = ''
if (n_elements(obs_wvfilt) eq 1) then swf=strtrim(obs_wvfilt[0],2)
if (strlen(swf) ne 1) then swf = '[0-9]'

obs_sclk_long = long(obs_sclk)   ; in case input is string
if (size(obs_sclk,/type) eq 7) then begin
	; String input; need to convert without partition
	p = strpos(obs_sclk,'/')
	obs_sclk_long = long(strmid(obs_sclk, p+1,10))
endif

up_level = strupcase(strtrim(dr_level[0],2))
if (n_elements(obs_cal_version) gt 0) then begin
	upobsver = strtrim(strupcase(obs_cal_version),2)
endif else begin
	upobsver = 'x'
endelse

; ===========================================================
;      DOWNSELECTION - DETECTOR, TYPE, BIN, WAVELENGTH
; ===========================================================

; Generate regular expressions for matching the filenames for 2 cuts,
; 1) data record type and detector (rg_td), 2) bin and wavelength filter codes (rg_bw),
; dependent on CDR/ADR type
; rg_td defaults SCLK, binning, wavelength filter, and version.
; Then eliminate lower version among the rg_td results.
; rg_bw sets binning and wavelength filter and leaves SCLK and version defaulted, 
; but because of the previous downselect, only the most recent version will be 
; in play any way.
; Then, select a final single file based on SCLK.
;
; Filename pattern elements:
;	p = SCLK partition
;	cccccccccc = SCLK tick
;	b = binning code (decimal digit)
;	w = wavelength filter code (decimal digit)
;	? = characters we don't care about here
;	tt = 2-character CDR acronym
;	d = detector code
;	v = version

if ((up_level eq 'ADR') && (stype eq 'VS')) then begin
	; VS ADR filename = ADRpcccccccccc_xxxxx_
	rg_td = 'ADR[0-9]{11}_[A-F0-9]{5}_VS[0-9]{2}' + sdet + '_.\.IMG'
	rg_bw = 'ADR[0-9]{11}_[A-F0-9]{5}_VS' + sbin + swf + sdet + '_.\.IMG'
endif else if (fix(up_level) eq 4) then begin
	; CDR4 filename like CDR4pcccccccccc_tt?b???w?d_v.IMG with
	rg_td = 'CDR4[0-9]{11}_' + stype + '[0-9]{7}' + sdet + '_.\.IMG'
	rg_bw = 'CDR4[0-9]{11}_' + stype + '[0-9]' + sbin + '[0-9]{3}' + swf + '[0-9]' + sdet + '_.\.IMG'
endif else if (fix(up_level) eq 6) then begin
	; CDR6 filename like CDR6_p_cccccccccc_tt_d_v.TAB; no bin/wave filter parameters
	rg_td = 'CDR6_[0-9]_[0-9]{10}_' + stype + '_' + sdet + '_.\.TAB'
	rg_bw = rg_td
endif else begin
	; Don't know what to do with this level...
	return,''
endelse

; Regex match for first cut - detector and CDR type
dr_indices = lindgen(n_elements(up_candidate_list))
dr_list = file_basename(up_candidate_list)
mtch = stregex(dr_list, rg_td, /boolean)
k = where(mtch, nk)
if (nk eq 0) then return,''
dr_list = dr_list[k]
dr_indices = dr_indices[k]

; Get the PDS version numbers; dump all files that aren't at the 
; most recent version (for AT and VS, also require that the CDR 
; source cal version matches the data)
pds_vers = get_pds_version(dr_list, latest_vers)
use_vers = latest_vers
if ( ((up_level eq '4') && (stype eq 'AT')) ||  $
     ((up_level eq 'ADR') && (stype eq 'VS')) ) then begin
	; Seeking AT or VS; need to match cal versions. 
	; Start with latest CDR version. Find the source data 
	; version for one of those CDRs. If it matches observation 
	; use the leatest. If not, check the next lowest. Keep 
	; working down till you find the highest version that 
	; matches the observation cal. 
	v0 = -1
	if ((cat_is_digit(latest_vers,/allow_hex))[0]) then begin
		reads,latest_vers,v0,format='(Z)'
	endif
	for iuse_vers=v0,-1,-1 do begin
		k = where((pds_vers eq string(iuse_vers,format='(Z1)')), nk)
		if (nk eq 0) then continue
		; Find label for one of the CDRs at use_vers...
		lbl = ''
		p = strpos(dr_list[k[0]],'.',/reverse_search)
		if (p gt 0) then begin
			lbl = file_dirname(candidate_drs[dr_indices[k[0]]], /mark_directory) $
			        + strmid(dr_list[k[0]],0,p)+'.LBL'
			lbl = (file_search(lbl, /fold_case))[0]
		endif
		if (lbl eq '') then find_lblfile, dr_list[k[0]], lbl, batch=batch
		if (lbl eq '') then continue
		read_crism_lbl,lbl,['SOURCE_PRODUCT_ID'], val
		p = strpos(strupcase(val[0]),'_TRR')+4
		if ((p gt 4) and (p lt strlen(val[0]))) then begin
			cdr_source_ver = strupcase(strmid(val[0],p,1))
			if (cdr_source_ver eq upobsver[0]) then break   ; normal exit
		endif
	endfor
	if (iuse_vers lt 0) then begin
		if (keyword_set(batch)) then begin
			return,''
		endif else begin
			msg = 'Cant find '+stype+' CDR from calibration version '+upobsver+'.' $
				    + string(10b) + 'Continue anyway with latest CDR version?'
			ok = dialog_message(msg,title='CDR search',/question,/cancel)
			if ((ok eq 'No') || (ok eq 'Cancel')) then begin
				return,''
			endif else begin
				use_vers = latest_vers
			endelse
		endelse
	endif else begin
		use_vers = string(iuse_vers,format='(Z1)')
	endelse
endif

; Downselect to those CDRs at version = use_vers. (use_vers=latest_vers
; except possibly for AT and VS searhes.)
k = where((pds_vers eq use_vers), nk)
if (nk eq 0) then return,''      ; should never happen
dr_list = dr_list[k]
dr_indices = dr_indices[k]

; Regex match for second cut - bin and wavelength filter
mtch = stregex(dr_list, rg_bw, /boolean)
k = where(mtch, nk)
if (nk eq 0) then return,''
dr_list = dr_list[k]         ; downselection
dr_indices = dr_indices[k]


; Now it should be down to a set of files differing only in valid SCLK.
; Pick the one with max SCLK <= observation SCLK.
file_start_sclk = extract_dr_sclk(dr_list)

; Select data records with the latest valid SCLK range
sclk_idx = select_sclk(obs_sclk, file_start_sclk.tick)
if (sclk_idx[0] lt 0) then return,''
dr_indices = dr_indices[sclk_idx]
return, candidate_drs[dr_indices]
end
