;******************************************************************************
;
; function get_tes_albedo, lat, lon, alb
;
; Return TES albedo at specified coordinates
;
; INPUT:
;	lat		float or fltarr		Latitude[s] (degrees)
;	lon		float or fltarr		Latitude[s] (degrees east)
;
; OUTPUT PARAMETERS:
;	alb		float or fltarr		TES albedo
;
; RETURN VALUE:
;	Status
;		0    Success
;		-1   IO failure reading TES data
;
; KEYWORDS: none
;
; NOTES:
;	Inputs lat and lon must both be either scalars, or arrays with matching 
;	dimensions. 
;
; USES:
;	No external procedures.
;
; TES ALBEDO DATA SOURCE:
;	http://jmars.asu.edu/data/tes_albedo/
;
;	Christensen et al., The Mars Global Surveyor Thermal Emission 
;	Spectrometer experiment: Investigation description and surface science 
;	results, J. Geophys. Res., 106, 23,823-23,871, 2001..
;
; HISTORY:
;	2008 Mar 30  F. Morgan: First version written.
;   2008 Sep 01  FM    Use cat_auxfile_path() to get config-defined path
;	2009 Aug 18  Replace on_ioerror with catch because on_ioerror might cause
;	             problems executing from save set 
;******************************************************************************

function get_tes_albedo, lat, lon, alb


catch, error_status
if (error_status ne 0) then begin
	if (n_elements(u) eq 1) then free_lun,u
	return, -1
endif

; TES albedo file path
pathsep = strtrim(path_sep(),2)
albedo_file = cat_auxfile_path() + pathsep + 'TESmaps' $
		+ pathsep + 'albedo' + pathsep + 'alb32'

; Read TES albedo data
nlon = 2880
nlat = 1440
openr,u, albedo_file, /get_lun, /swap_if_big_endian
h = bytarr(11520)
readu,u, h                  ; skip header
global_alb = fltarr(nlon, nlat)
readu,u, global_alb
free_lun,u

; Fix an obvious (?) glitch at elements 1440:1441 in longitude
global_alb[1440,*] = global_alb[1439,*]
global_alb[1441,*] = global_alb[1442,*]

; Bilinear interpolation to input lat/lon
lonx = lon mod 360
k = where((lonx lt 0.), nk)
if (nk gt 0) then lonx[k] += 360.
x = lonx * float(nlon+1)/360.
y = (90.0-lat) * float(nlat+1)/180.
alb = bilinear(global_alb, x, y)

return,0
end
