;===============================================================================
;LUT	Mars surface thermal model lookup table code for MRO CRISM.
;	Generates a subset of the large thermal lookup table that is
;	appropriate for a particular image fragment or strip segment
;	characterized by a single value of LS, latitude, and local time.
;	This allows lookup operations to be done faster and with smaller
;	arrays in hand - important for routine processing.
;
;	Assumptions:
;		This segment can be treated with a single value
;		of latitude, Ls, and local time. That means that for
;               multispectral strips, they must be broken into segments
;               of a maximum latitudinal extent (size is TBD).
;		The output lookup table from this routine is
;		called sublut and is made available to the
;		routine 'Fast', which operates on a pixel-by-pixel
;		basis to do the final processing.
;
;	Original version for small LUT supplied in 2003 by HHKieffer

;TZ Martin	Oct 21 2004
; Revisions commencing Sept 2005 for full LUT from T. Titus
;Action:	add tests for min values in arrays

;PC McGuire Feb 23 2008
; Added code to input filename and path as input parameters;
;      needed in order to have all installation parameters inside luttest.pro
;      added verbose_bin6r input argument

;Input parameters

;	ls	Ls = areocentric solar lat (season) of observation
;	ltim	local time (near 3 PM for MRO except near poles)
;	lat	latitude (use mean lat for this image segment)
;       tau	dust opacity in visible.  NOTE! This is not used,
;		due to elevation and tau being entwined. Done in fast_3
;		instead...
;	flag	First time flag. Do not read the big array more than once.
;		Set ge 0 for subsequent calls.
;   path_ADR_TE   Path for     ADR_TE LUT
;   fname         Filename for ADR_TE LUT

;Output parameters

;	sublut	Subset of lookup table - fltarr(5,33,3,6)
;       for inertia, slope/az, albedo, and opacity/elevation

;Data are available for the 2005 version:

; 24 times of day 	1-24H
; 18 latitudes 		-85 to +85 by 10 deg
; 40 Ls  			0-351.33
; [note: evenly spaced in TIME rather than Ls]
; 5 inertias		50,100,200,400,800
; 33 slope/az combos  	0,10,20,30,40 deg slope
; and		0,45,90 ... 315 azimuth
; 3 albedos   		0.10, 0.20, 0.30
; 6 opacity/elevations 	0.15 and 0.7 tau
; -4, 0, +4 km elevation
;===============================================================================

;===============================================================================
;
;
; lut
;
;
;===============================================================================
PRO lut_3,ls,ltim,lat,sublut,flag,path_ADR_TE,fname,types_instring,verbose_bin6r
COMMON sharelut,sublut0		;to allow fast repeated access to this once derived

   ;============================================================================
   ;Open full lookup table. Done once per image segment
   ;============================================================================
if flag ge 1 then goto, pass	;skip upon re-entry
   ;============================================================================
   ;Set path as needed for your own case
   ;============================================================================
   ;path   ='W:\marsdata\software\act\react_crism_cal\crism_atmcor\v5_00\ADR\LUTS_ADR\TE\'
    path=path_ADR_TE
   ;============================================================================
   ;This is the name of the large lookup table

   ;============================================================================
   ;filnam = path+'crism_ti_krc_mods.bin6'
   ;filnam = path+'ADR_TE_0000_0_0000000000_0L.DAT'
    filnam = path+fname



    ;PCM these 2 lines of code do not work; 2/23/2008
     ;str    = bin6r(filnam,head=descriptive_test_in_file) ;this routine is USGS
     ;sublut0 = str.tsrf

    ;PCM so I changed the 2 lines above instead to be the following 2 lines; 2/23/2008
    tsrf    = bin6r(filnam,head=descriptive_test_in_file,quiet=0,types_instring,verbose_bin6r=verbose_bin6r) ;this routine is USGS
	sublut0 = tsrf



   ;============================================================================
   ;Proceed first by Ls to achieve greatest volume reduction: factor of 40
   ;These are the calculation points in Ls space for the table, with one added
   ;at the end for interpolation purposes:
   ;============================================================================
pass:
   lsi    = [  0.0111,   8.465,  16.709,  24.768,  32.667,  40.434,  48.093, $
              55.673,   63.200,  70.7,    78.199,  85.725,  93.303, 100.961, $
             108.724,  116.620, 124.675, 132.915, 141.364, 150.045, 158.979, $
             168.180,  177.660, 187.420, 197.454, 207.742, 218.255, 228.950, $
             239.773,  250.664, 261.555, 272.381, 283.080, 293.597, 303.891, $
             313.930,  323.697, 333.183, 342.391, 351.330, 360.011 ]

   ;============================================================================
   ; find relevant index in lsi array
   ;============================================================================
   min_idx= MIN(WHERE(lsi GE ls))
   lo_idx = min_idx EQ 0 ? min_idx : min_idx - 1
   hi_idx = min_idx EQ 0 ? min_idx : min_idx
   frac   = min_idx EQ 0 ? min_idx : (ls-lsi[lo_idx])/(lsi[hi_idx]-lsi[lo_idx])
   if hi_idx eq 40 then hi_idx = 0	;handle 355-360 range of Ls
   sublut = REFORM(sublut0[*,*,lo_idx,*,*,*,*]) +      $
            frac *                                    $
               ((REFORM(sublut0[*,*,hi_idx,*,*,*,*]) - $
                 REFORM(sublut0[*,*,lo_idx,*,*,*,*])))

   ;============================================================================
   ; sublut is now an array of size (24,18,5,33,3,6) = 1283040 elements
   ;============================================================================

   ;============================================================================
   ; array of times, 1 to 24 H
   ; find relevant index in lti array
   ; remainder will do, since data are hourly
   ;============================================================================
   lti    = FINDGEN(24)+1.
   min_idx= MIN(WHERE(lti GE ltim))
   lo_idx = min_idx EQ 0 ? 23      : min_idx - 1
   hi_idx = min_idx EQ 0 ? min_idx : min_idx
   ;============================================================================
   ;Now do interpolation in time of day
   ;Presume local time input is of the form "15.23"
   ;============================================================================
   sublut = REFORM(sublut[lo_idx,*,*,*,*,*]) +             $
               (ltim MOD 1) *                              $
                  ((REFORM(sublut[hi_idx,*,*,*,*,*]) -     $
                    REFORM(sublut[lo_idx,*,*,*,*,*])))

   ;============================================================================
   ; sublut is now an array of size (18,5,33,3,6) = 53460 elements
   ;============================================================================

   ;============================================================================
   ;Array of lats
   ;Find relevant index
   ;============================================================================
   lati   = -85.+findgen(18)*10.
   min_idx= MIN(WHERE(lati GE lat))
   lo_idx = min_idx EQ 0 ? min_idx : min_idx - 1
   hi_idx = min_idx EQ 0 ? min_idx : min_idx
   frac   = min_idx EQ 0 ? min_idx : (lat-lati[lo_idx])/10.
   ;============================================================================
   ; next is interpolation in latitude
   ; note delta lat is 10. deg
   ;============================================================================
   sublut = REFORM(sublut[lo_idx,*,*,*,*]) +               $
               frac *                                      $
                  ((REFORM(sublut[hi_idx,*,*,*,*]) -       $
                    REFORM(sublut[lo_idx,*,*,*,*])))

   ;============================================================================
   ;sublut is an array of size (5,33,3,6) = 2970 elements
   ;Lookup table is now reduced in size by 18*24*40 = factor of 17280
   ;============================================================================

END
; lut


