;******************************************************************************
;
; function tile_support_fnm, img_fnm, othertype
;
; Given a filename for a tile-related file, make a filename for a different 
; tile file type for the same tile. (e.g., generate the filename for the MRWV 
; file associated with T0934_MRRIF_05S298_0256_1.IMG)
;
; INPUT:
;	img_fnm     string     Image tile filename 
; 	othertype   string     Supporting file type ('DE', 'WV')
;
; OUTPUT: no parameters
;
; RETURN VALUE:
;	Supporting file filename, '' if fail.
;
; KEYWRODS:
;	BATCH    If set, do not ask for user search.
;
; NOTES:
;	Image filename is like T0934_MRRIF_05S298_0256_1.IMG. Might also have some 
;	post-processing tags like T0934_MRRIF_05S298_0256_1_cat.IMG. Here, 
;	rely on sequence of underscores to place 'MRRDE' or 'MRRWV' in correct 
;	place, and drop any added tags.
;
;	This will drop any CAT-appended text in the input filename.
;
; HISTORY:
;	2008 Dec 31   First version, F.Morgan
;	2009 Jul 08   Test input is tile. Add wildcard for version number.
;	2009 Jul 27   Check for file on PDS archive if not found in datafile 
;                 directory. Add user search and BATCH keyword.
;	2009 Sep 16   Add label-specified wavelength file search for WV.
;	2009 Sep 16   Reorganize version selection if multiple copies found.
;
;******************************************************************************

function tile_support_fnm, img_fnm, othertype, batch=batch


forward_function find_wavelength_file, reconstruct_pds_path, get_pds_version, $
          catuser_file_search

; Fail if not tile:
if (~pdsfile_is_tile(img_fnm)) then return,''

otup = strupcase(strtrim(othertype,2))


;===========================================================================
;  SPECIAL CASE: WV FILE
;===========================================================================

; For WV file, look first for the wavelength file called out in label
if (otup eq 'WV') then begin
	tile_fnm = find_wavelength_file(img_fnm)
	if (tile_fnm ne '') then return,tile_fnm
endif
		

;===========================================================================
;  FIND A BASENAME FOR THE SUPPORT FILE
;===========================================================================

; Separate directory and filename:
img_dir = file_dirname(img_fnm,/mark_directory)
img_base = file_basename(img_fnm)

; Split filename into sections at underscores:
tf = strsplit(img_base, '_', /extract)

; Have 5 parts of interest; last is a version number, might be like 
; '1.IMG' or just '1' if there was an extra tag inserted. Check that:
p = strpos(tf[4],'.')
if (p gt 0) then tf[4] = strmid(tf[4],0,p)

; Generate new 2nd section according to requested type:
otup = strupcase(othertype)
if (strpos(otup,'AL') ge 0) then begin
	sec2 = 'MRRAL'
	ext = '.IMG'
endif else if (strpos(otup,'DE') ge 0) then begin
	sec2 = 'MRRDE'
	ext = '.IMG'
endif else if (strpos(otup,'DL') ge 0) then begin
	sec2 = 'MRRDL'
	ext = '.IMG'
endif else if (strpos(otup,'IF') ge 0) then begin
	sec2 = 'MRRIF'
	ext = '.IMG'
endif else if (strpos(otup,'SU') ge 0) then begin
	sec2 = 'MRRSU'
	ext = '.IMG'
endif else if (strpos(otup,'WV') ge 0) then begin
	sec2 = 'MRRWV'
	ext = '.TAB'
endif else begin
	return,''
endelse

new_fnm = tf[0] + '_' + sec2 
for i=2,3 do new_fnm += ('_' + tf[i])
new_fnm += '_?'      ; version = wildcard
new_fnm += ext


;=====================================================================
;  FIND A PATH FOR THE SUPPORT FILE
;=====================================================================

; First, try the same directory where the original file was...
try_file = img_dir + new_fnm
fnd_file = file_search(try_file, /fold_case)
if (fnd_file[0] eq '') then begin
	; local path didn't work; try PDS path...
	try_file = reconstruct_pds_path(img_base) + new_fnm
	fnd_file = file_search(try_file, /fold_case)
endif

; Now pick a version if found more than one file:
if (n_elements(fnd_file) gt 1) then begin
	; Try to get a version same as input...
	fnd_vers = get_pds_version(fnd_file, latest_vers)
	img_vers = (get_pds_version(img_base))[0]
	k = where((fnd_vers eq img_vers),nk)
	if (nk gt 0) then begin
		tile_fnm = fnd_file[k[0]]
	endif else begin
		; Nope; use highest version found...
		k = where((fnd_vers eq latest_vers),nk)
		if (nk gt 0) then tile_fnm = fnd_file[k[0]]  ; should always be true
	endelse
endif else begin
	tile_fnm = fnd_file[0]
endelse
if (tile_fnm ne '') then return, tile_fnm

;====================================================================
;  STILL NO LUCK - USER SEARCH
;====================================================================
if (~keyword_set(batch)) then begin
	msg = othertype +' file not found for ' + string(10B) $
	        + strtrim(img_base,2) + string(10B) $
			+ 'Would you like to locate it?'
	srch_ttl = 'Choose ' + othertype + ' file'
	file_filt = '*.*'
	user_fnm = catuser_file_search(msg, srch_ttl, file_filt, user_cancel)
	if (user_cancel) then begin
		tile_fnm = ''
	endif else begin
		if (file_test(user_fnm)) then tile_fnm = user_fnm
	endelse
endif

return, tile_fnm
end
