;******************************************************************************
;
; function vs_adr_filename(vsid, det, bincode, wvfilt, obs_sclk)
;
; Return the path for a VS ADR given its observation ID
;
; INPUT:
;	vsid        any scalar      Volcano scan observation ID
;	det         string          detector code ('S', 'L', or 'J')  [expect 'L']
;	bincode     int             Binning code
;	wvfilt      int             Wavelength filter
;	obs_sclk	string or long	Observation SCLK (see note)
;	cal_version string          Observation calibration version (eg '2' for TRR2)
;
; OUTPUT: no parameters
;
; KEYWORDS: Batch (suppress user search if set)
;
; RETURN VALUE:  Selected filename, '' if fail.
;
; NOTES:
;	If vsid is input as a numeric scalar, it is converted to a hex string 
;	for inclusion in filename.
;
;	SCLK string format = 'P/SSSSSSSSSS' or just 'SSSSSSSSSS'
;	                      (where P = sclk partition, SSSSSSSSSS=ticks; 
;	                      full length of tick string not required)
;
;	VS ADR filename as of 8/25/2009 is ADRPTTTTTTTTTT_IIIII_AABWS_V
;     P = sclk partition
;     TTTTTTTTTT  = sclk tick
;     IIIII = volcano scan observation ID (hex)
;     AA = product acronym = VS
;     B = binning code
;     W = wavelength table code
;     S = sensor ID (always L)
;     V = version
;
; USES:
;	cat_auxfile_path
;	select_dr
;
; HISTORY:
;	2009 Jan 09  F.Morgan  First version. 
;	2009 Jan 16  F.Morgan  Allow string or numeric input for vsid
;	                       and add IO error trap.
;	2009 Aug 18  Replace on_ioerror with catch because on_ioerror might cause
;	             problems executing from save set 
;	2009 Aug 25  F. Morgan. Update for VS ADR filename change.
;	2010 Mar 08  F. Morgan. Replace '/' with path_sep() in VS path.
;	2010 Dec 19  F. Morgan. Use select_dr to downselect from candidates.
;	                        Add cal_version input and batch keyword.
;	2012 Apr 16  F. Morgan. Add aux_path keyword
;
;******************************************************************************

function vs_adr_filename, vsid, det, bincode, wvfilt, obs_sclk, $
            cal_version, batch=batch, aux_path=aux_path


forward_function cat_auxfile_path, select_dr

catch, error_status
if (error_status ne 0) then return, ''


if (size(vsid,/type) eq 7) then begin
	vsid_str = strtrim(vsid,2)
	while (strlen(vsid_str) lt 5) do vsid_str = '0' + vsid_str
endif else begin
	vsid_str = string(round(vsid), format='(Z5.5)')
endelse

auxpath = ''
if (size(aux_path,/type) eq 7) then begin
	if (file_test(aux_path[0])) then begin
		auxpath = (file_search(aux_path[0], /mark_directory))[0]
	endif
endif
if (auxpath eq '') then begin
	auxpath = cat_auxfile_path()
endif
vsadr_path = auxpath + 'ADR' + path_sep() + 'VS' + path_sep()
if (~file_test(vsadr_path)) then return,''

fnm = vsadr_path + 'ADR*' + vsid_str + '*VS' + strtrim(bincode,2) $
             + strtrim(wvfilt,2) + strtrim(det,2) +'_?.IMG'
found_vs = file_search(fnm, /fold_case)
if (found_vs[0] eq '') then return,''     ; no matching files t all found

; Select correct ADR:
valid_vs = select_dr(found_vs, 'ADR', 'VS', obs_sclk, det, bincode, $
   wvfilt, cal_version, batch=batch)

return,valid_vs[0]
end
