;******************************************************************************
;
; function write_cat_header
;
; Write an ENVI header for a new CAT disk file. 
;
; INPUT PARAMETERS: None.
;
; OUTPUT: No parameters
;
; RETURN VALUE: int status...
;	 0   success
;	-1   missing required keyword
;	-2   Not overwriting existing header
;	-3   Unable to open for CAT keyword output
;
; KEYWORDS, INPUT:
;	Most of the keywords accepted by envi_setup_head, plus "operational" keywords:
;	   CLOSEFILE  If set, removes the new instnce of FNAME from the 
;		          available bands list when done.
;	   BATCH      No interactive dialogs
;	   OVERWRITE  Overwrite existing header if present.
;	   R_FID      ENVI file ID of the newly opened file; -1 if fails, or file is 
;	              closed on /CLOSEILE keyword.
;
;    plus CAT keywords:
;	   CAT_START_TIME            Observation start date 'yyyy-mm-ddThh:mm:ss.fff'
;	   CAT_SCLK_START            Observation start SCLK 'p/tttttttttt'
;	   CAT_CRISM_OBSID           CRISM observation ID string
;	   CAT_OBS_TYPE              CRISM obs type (string = FRT, MSP, etc)
;	   CAT_PRODUCT_VERSION       Cal version; from PDS PRODUCT_VERSION_ID
;	   CAT_CRISM_DETECTOR_ID     CRISM detector ('L', 'S', 'J')
;	   CAT_BIN_MODE              Detector bin mode (numeric code from PDS label)
;	   CAT_WAVELENGTH_FILTER     Wavelength filter (numeric code from PDS label)
;	   CAT_CRISM_DETECTOR_TEMP   CRISM detector temperature (C)
;	   CAT_CRISM_BENCH_TEMP      CRISM optical bench temperature (C)
;	   CAT_CRISM_HOUSING_TEMP    CRISM spectrometer housing temperature (C)
;	   CAT_SOLAR_LONGITUDE       Observation solar longitude (L_S, degrees)
;	   CAT_PDS_LABEL_FILE        Label file of initial PDS file, with path
;	                             (could be multiple files after merging)
;	   CAT_SPECTRUM_RESAMPLED    Flag whether data are resampled (1) or not (0)
;	   CAT_TILE_WAVE_FILE        For tiles - wavelength file
;	   CAT_SWEETSPOT_WAVE_FILE   For non-tiles - sweetspot wavelength file
;	   CAT_WA_WAVE_FILE          For non-tiles - WA wavelength file (full array)
;	   CAT_IR_WAVES_REVERSED     'YES', IR wavelength array reversed so low 
;		                         wavelengths first; 'NO', not reversed.
;	   CAT_HISTORY               CAT processing steps
;	   CAT_INPUT_FILES           containing input files for each CAT 
;	 	                         processing step
;
;	Required ENVI standard keywords:
;		FNAME          This is the path+name of the data file the new header
;		               will be associated with
;		NS             Number of samples
;		NL             Number of lines
;		NB             Number of bands
;		DATA_TYPE      IDL type code; 1=byte, 2=int (16 bit), 
;		               3=long int (32 bit), 4=float, etc
;		INTERLEAVE     ENVI Interleave code (0=BSQ, 1=BIL, 2=BIP)
;		OFFSET         Offset (in bytes) to the start of the data
;
; KEYWORDS, OUTPUT:
;	R_FID      ENVI file ID of the newly opened file; -1 if fails, or file is 
;	           closed on /CLOSEILE keyword.
;
; NOTES:
;	Input CAT parameters can be string or, for applicable parameters, integers.
;
;	Keywords not specified on input are not written to the output header. 
;
;	CAT keywords are written as strings, as input, with the following exceptions, 
;	where if input is string conversion to the expected numerical type is 
;	attempted and, if it fails, the value is not written:
;	      Keyword                Type 
;	   CAT_BIN_MODE               int 
;	   CAT_WAVELENGTH_FILTER      int  
;	   CAT_CRISM_DETECTOR_TEMP   float
;	   CAT_CRISM_BENCH_TEMP      float
;	   CAT_CRISM_HOUSING_TEMP    float
;	   CAT_SOLAR_LONGITUDE       float
;	   CAT_SPECTRUM_RESAMPLED     int  
;
;	********    DECREMENTING DEF_BANDS    *********
;	This routine passes def_bands directly through to envi_setup_head, 
;	which treats input as zero-offset indices into the wavelength array,
;	actually writes values incremented by one to match ENVI's 
;	unity-offset band numbering scheme.
;
;	envi_file_query or (for consistency with that routine) 
;	get_cat_header_data will return exactly what's in the header, 
;	which is ALREADY BE IN UNITY-OFFSET BAND NUMNER TERMS.
;	Therefore, you should DECREMENT THAT RESULT BY 1 for input 
;	to this routine in order to preserve the same default bands.
;
; TYPICAL CALL:
;
; USES:
;
; HISTORY:
;	2009 Nov 02 F. Morgan: First version written
;	2009 Nov 05 FM. Debugs.
;	2009 Nov 26 FM. Eliminated byte_order keyword. Do the test here.
;	2009 Nov 27 FM. Documentation edit (add def_bands note)
;	2009 Dec 08 FM. Close displays associated with file, and remoe from 
;	                available bands, to keep ENVI from reverting header data.
;	2010 Feb 25 FM. Fix message typo.
;	2010 Nov 17 FM. New parameter... cat_product_version = cal version
;	2010 Nov 18 FM. New parameters: cat_start_time, cat_sclk_start, 
;	                cat_crism_obsid, cat_obs_type
;	2015 Oct 07 FM. Write MTRDR filtering flags to header
;
;******************************************************************************

function write_cat_header, FNAME=fname, NS=ns, NL=nl, NB=nb, $                  ; required
	DATA_TYPE=data_type, INTERLEAVE=interleave, OFFSET=offset, $
	CLOSEFILE=closefile, BATCH=batch, OVERWRITE=overwrite, R_FID=fid,  $        ; operational
	BBL=bbl, BNAMES=bnames, CLASS_NAMES=class_names, $   ; optional ENVI                   ; ENVI
	DATA_GAINS=data_gains, DATA_IGNORE_VALUE=data_ignore_value, $
	DATA_OFFSETS=data_offsets, DEF_BANDS=def_bands, DEF_STRETCH=def_stretch, $
	DESCRIP=descrip, FILE_TYPE=file_type, FWHM=fwhm, GEO_POINTS=geo_points, $
	LOOKUP=lookup, MAP_INFO=map_info, NUM_CLASSES=num_classes, $
	PIXEL_SIZE=pixel_size, REFLECTANCE_SCALE_FACTOR=reflectance_scale_factor, $
	SENSOR_TYPE=sensor_type, SPEC_NAMES=spec_names, UNITS=units, $
	WAVELENGTH_UNIT=wavelength_unit, WL=wl, XSTART=xstart, YSTART=ystart, $
	ZPLOT_AVERAGE=zplot_average, ZPLOT_TITLES=zplot_titles, ZRANGE=zrange, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $                          ; CAT
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	CAT_PHOTOMETRIC_CORR_FLAG        = cat_photometric_corr_flag, $
	CAT_ATMOSPHERIC_CORR_FLAG        = cat_atmospheric_corr_flag, $
	CAT_RATIO_SHIFT_CORR_FLAG        = cat_ratio_shift_corr_flag, $
	CAT_EMPIRICAL_GEOM_NORM_FLAG     = cat_empirical_geom_norm_flag, $
	CAT_EMPIRICAL_SMILE_CORR_FLAG    = cat_empirical_smile_corr_flag, $
	CAT_SENSOR_SPACE_TRANSFORM_FLAG  = cat_sensor_space_transform_flag



; Set fid to invalid flag in case of early return, or open not set:
fid = -1

; Check required keywords:
reqkw = n_elements(fname) < n_elements(ns) < n_elements(nl) < n_elements(nb) $
          < n_elements(data_type) < n_elements(interleave) < n_elements(offset)
if (reqkw eq 0) then begin
	; Not all required keywords present.
	return, -1
endif

; Does an ENVI header already exist? Might be file.img.hdr or file.hdr...
hdr_cand1 = strtrim(fname,2)+'.hdr'
p = strpos(fname,'.',/reverse_search)
hdr_cand2 = strmid(fname,0,p)+'.hdr'
fhdr = [file_search(hdr_cand1,/fold_case), file_search(hdr_cand2,/fold_case)]
k = where((fhdr ne ''),nk)
delete_header = 0
if (nk gt 0) then begin
	fhdr = fhdr[k]
	if (keyword_set(overwrite)) then begin
		delete_header = 1
	endif else begin
		if (~keyword_set(batch)) then begin
			; Interactive... ask about overwriting:
			msg = 'CAT is attempting to overwrite header for ' + string(10b) $
		    	   + fname + '. Proceed?'
			ok = dialog_message(msg, title='CRISM Header', /question, /cancel)
			if ((ok eq 'No') or (ok eq 'Cancel')) then return,-2
			; If didn't return, delete it:
			delete_header = 1
		endif else begin
			; In batch mode, just give up; overwrite not set:
			return,-2
		endelse
	endelse
	if (delete_header) then begin
		; File has to be closed or else ENVI might force reversion to
		; existing header values if one was present. But ENVI won't close 
		; a file without throwing an 'OK?' prompt if there are any open 
		; associated displays, so close any displays associated with 
		; this file first. The file will be reopened later.
		if (catfile_isopen(fname, r_fid)) then begin
			; Find and close any displays:
			cat_dn = cat_envi_find_display(r_fid)
			if (cat_dn[0] ge 0) then begin
				for i=0,n_elements(cat_dn)-1 do envi_close_display,cat_dn[i]
			endif
			; And close the file:
			envi_file_mng, id=r_fid, /remove
		endif
		; And delete the old header file.
		file_delete, fhdr, /quiet
	endif
endif


;=====================================
;   ENVI STANDARD PARAMETERS
;=====================================

; Determine byte order of current machine:
enda = 5
endb = swap_endian(enda,/swap_if_big_endian)
if (enda ne endb) then byte_order=1 else byte_order=0

; Write the standard ENVI parameters - don't use the open keyword here, 
; that sometimes causes a crash (F.Seelos tip, 11/5/09)
envi_setup_head, FNAME=fname, NS=ns, NL=nl, NB=nb, DATA_TYPE=data_type, $
	INTERLEAVE=interleave, $
	BBL=bbl, BNAMES=bnames, BYTE_ORDER=byte_order, CLASS_NAMES=class_names, $
	DATA_GAINS=data_gains, DATA_IGNORE_VALUE=data_ignore_value, $
	DATA_OFFSETS=data_offsets, DEF_BANDS=def_bands, DEF_STRETCH=def_stretch, $
	DESCRIP=descrip, FILE_TYPE=file_type, FWHM=fwhm, GEO_POINTS=geo_points, $
	LOOKUP=lookup, MAP_INFO=map_info, NUM_CLASSES=num_classes, OFFSET=offset, $
	PIXEL_SIZE=pixel_size, REFLECTANCE_SCALE_FACTOR=reflectance_scale_factor, $
	SENSOR_TYPE=sensor_type, SPEC_NAMES=spec_names, UNITS=units, $
	WAVELENGTH_UNIT=wavelength_unit, WL=wl, XSTART=xstart, YSTART=ystart, $
	ZPLOT_AVERAGE=zplot_average, ZPLOT_TITLES=zplot_titles, $
	ZRANGE=zrange, /write

; File is left unopened at this point...


;=====================================
;   CAT PARAMETERS
;=====================================

; Open for CAT parameter output (it HAS to be opened to get a FID for the 
; custom keyword insertion commands. No way to silently open if the available 
; bands window is open, which it might be. So, open it now, and leave it 
; open on return unless CLOSEFILE keyword is set.)
envi_open_file, fname, /no_interactive_query, r_fid=fid
if (fid lt 0) then begin
	; Open failed
	if (~keyword_set(batch)) then begin
		msg = 'Failed to open '+strtrim(file_basename(fname),2) + $
		      ' for CAT header output.' $
		      + string(10b) + 'Header contains only ENVI standard parameters.'
		ok = dialog_message(msg, title='CRISM Header Fault')
	endif
	return,-3
endif

ncat_kw = 0
if (n_elements(cat_start_time) eq 1) then begin
	t = strtrim(cat_start_time[0],2)
	if ((t ne '-1') and (t ne '')) then begin
		envi_assign_header_value, fid=fid, keyword='cat start time', value=t
		ncat_kw++
	endif
endif
if (n_elements(cat_sclk_start) eq 1) then begin
	sclk = strtrim(cat_sclk_start[0],2)
	if ((sclk ne '-1') and (sclk ne '')) then begin
		envi_assign_header_value, fid=fid, keyword='cat sclk start', value=sclk
		ncat_kw++
	endif
endif
if (n_elements(cat_crism_obsid) eq 1) then begin
	obsid = strtrim(cat_crism_obsid[0],2)
	if ((obsid ne '-1') and (obsid ne '')) then begin
		envi_assign_header_value, fid=fid, keyword='cat crism obsid',value=obsid
		ncat_kw++
	endif
endif
if (n_elements(cat_obs_type) eq 1) then begin
	obstyp = strtrim(cat_obs_type[0],2)
	if ((obstyp ne '-1') and (obstyp ne '')) then begin
		envi_assign_header_value, fid=fid, keyword='cat obs type',value=obstyp
		ncat_kw++
	endif
endif
if ((n_elements(cat_product_version) eq 1) && (strtrim(cat_product_version) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat product version', value=cat_product_version
	ncat_kw++
endif
if ((n_elements(cat_crism_detector_id) eq 1) && (strtrim(cat_crism_detector_id) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat crism detector id', value=cat_crism_detector_id
	ncat_kw++
endif
if (n_elements(cat_bin_mode) eq 1) then begin
	cv = cat_strip_nondigit(string(cat_bin_mode),/allow_minus)
	if ((cv ne '') && (fix(cv) ge 0)) then begin
		envi_assign_header_value, fid=fid, keyword='cat bin mode', value=fix(cv)
		ncat_kw++
	endif
endif
if (n_elements(cat_wavelength_filter) eq 1) then begin
	cv = cat_strip_nondigit(string(cat_wavelength_filter),/allow_minus)
	if ((cv ne '') && (fix(cv) ge 0)) then begin
		envi_assign_header_value, fid=fid, keyword='cat wavelength filter', value=fix(cv)
		ncat_kw++
	endif
endif
if (n_elements(cat_crism_detector_temp) eq 1) then begin
	fstrval = cat_strip_nondigit(cat_crism_detector_temp, /allow_float)
	if (strlen(strtrim(fstrval)) gt 0) then begin
		envi_assign_header_value, fid=fid, keyword='cat crism detector temp', value=float(fstrval)
		ncat_kw++
	endif
endif
if (n_elements(cat_crism_bench_temp) eq 1) then begin
	fstrval = cat_strip_nondigit(cat_crism_bench_temp, /allow_float)
	if (strlen(strtrim(fstrval)) gt 0) then begin
		envi_assign_header_value, fid=fid, keyword='cat crism bench temp', value=float(fstrval)
		ncat_kw++
	endif
endif
if (n_elements(cat_crism_housing_temp) gt 0) then begin
	fstrval = cat_strip_nondigit(cat_crism_housing_temp, /allow_float)
	if (strlen(strtrim(fstrval)) gt 0) then begin
		envi_assign_header_value, fid=fid, keyword='cat crism housing temp', value=float(fstrval)
		ncat_kw++
	endif
endif
if (n_elements(cat_solar_longitude) gt 0) then begin
	fstrval = cat_strip_nondigit(cat_solar_longitude, /allow_float)
	if (strlen(strtrim(fstrval)) gt 0) then begin
		envi_assign_header_value, fid=fid, keyword='cat solar longitude', value=float(fstrval)
		ncat_kw++
	endif
endif
if ((n_elements(cat_pds_label_file) gt 0) && (strtrim(cat_pds_label_file[0]) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat pds label file', value=cat_pds_label_file
	ncat_kw++
endif
if (n_elements(cat_spectrum_resampled) eq 1) then begin
	cv = cat_strip_nondigit(string(cat_spectrum_resampled),/allow_minus)
	if ((cv ne '') && (fix(cv) ge 0)) then begin
		envi_assign_header_value, fid=fid, keyword='cat spectrum resampled', value=fix(cv)
		ncat_kw++
	endif
endif
if ((n_elements(cat_tile_wave_file) eq 1) && (strtrim(cat_tile_wave_file) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat tile wave file', value=cat_tile_wave_file
	ncat_kw++
endif
if ((n_elements(cat_sweetspot_wave_file) eq 1) && (strtrim(cat_sweetspot_wave_file) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat sweetspot wave file', value=cat_sweetspot_wave_file
	ncat_kw++
endif
if ((n_elements(cat_wa_wave_file) eq 1) && (strtrim(cat_wa_wave_file) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat WA wave file', value=cat_wa_wave_file
	ncat_kw++
endif
if ((n_elements(cat_ir_waves_reversed) eq 1) && (strtrim(cat_ir_waves_reversed) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat ir waves reversed', $
	   value=strupcase(strtrim(cat_ir_waves_reversed,2))
	ncat_kw++
endif
if ((n_elements(cat_photometric_corr_flag) eq 1) && (strtrim(cat_photometric_corr_flag) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat photometric correction flag', value=cat_photometric_corr_flag
	ncat_kw++
endif
if ((n_elements(cat_atmospheric_corr_flag) eq 1) && (strtrim(cat_atmospheric_corr_flag) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat atmospheric correction flag', value=cat_atmospheric_corr_flag
	ncat_kw++
endif
if ((n_elements(cat_ratio_shift_corr_flag) eq 1) && (strtrim(cat_ratio_shift_corr_flag) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat ratio shift correction flag', value=cat_ratio_shift_corr_flag
	ncat_kw++
endif
if ((n_elements(cat_empirical_geom_norm_flag) eq 1) && (strtrim(cat_empirical_geom_norm_flag) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat empirical geometric normalization flag', value=cat_empirical_geom_norm_flag
	ncat_kw++
endif
if ((n_elements(cat_empirical_smile_corr_flag) eq 1) && (strtrim(cat_empirical_smile_corr_flag) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat empirical smile correction flag', value=cat_empirical_smile_corr_flag
	ncat_kw++
endif
if ((n_elements(cat_sensor_space_transform_flag) eq 1) && (strtrim(cat_sensor_space_transform_flag) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat sensor space transform flag', value=cat_sensor_space_transform_flag
	ncat_kw++
endif
if ((n_elements(cat_history) eq 1) && (strtrim(cat_history) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat history', value=cat_history
	ncat_kw++
endif
if ((n_elements(cat_input_files) gt 0) && (strtrim(cat_input_files[0]) ne '')) then begin
	envi_assign_header_value, fid=fid, keyword='cat input files', value=cat_input_files
	ncat_kw++
endif

if (ncat_kw gt 0) then envi_write_file_header, fid


;=====================================
;   CLOSE IF REQUESTED
;=====================================

if (keyword_set(closefile)) then begin
	envi_file_mng, id=fid, /remove
	fid = -1
endif

return, 0
end
